@@ -1,6 +1,7 @@
 // http.cpp:  HyperText Transport Protocol handler for Cygnal, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -49,6 +50,7 @@
 //#include "handler.h"
 #include "utility.h"
 #include "buffer.h"
+#include "http.h"
 #include "diskstream.h"
 
 // Cygnal specific headers
@@ -84,91 +86,73 @@ HTTPServer::~HTTPServer()
 //    GNASH_REPORT_FUNCTION;
 }
 
-boost::shared_ptr<amf::Buffer>
+HTTP::http_method_e
 HTTPServer::processClientRequest(int fd)
 {
-//    GNASH_REPORT_FUNCTION;
-    boost::shared_ptr<amf::Buffer> buf(_que.peek());
-    boost::shared_ptr<amf::Buffer> result;
+    GNASH_REPORT_FUNCTION;
     
+    cygnal::Buffer *buf = new cygnal::Buffer;
+
+    // return processClientRequest(fd, buf);
+    return HTTP::HTTP_NONE;
+}
+
+HTTP::http_method_e
+HTTPServer::processClientRequest(Handler *hand, int fd, cygnal::Buffer *buf)
+{
+    GNASH_REPORT_FUNCTION;
+    
+    cygnal::Buffer result;
+
     if (buf) {
 	_cmd = extractCommand(buf->reference());
 	switch (_cmd) {
 	  case HTTP::HTTP_GET:
-	      result = processGetRequest(fd);
+	      result = processGetRequest(hand, fd, buf);
 	      break;
 	  case HTTP::HTTP_POST:
-	      result = processPostRequest(fd);
+	      result = processPostRequest(fd, buf);
 	      break;
 	  case HTTP::HTTP_HEAD:
-	      result = processHeadRequest(fd);
+	      result = processHeadRequest(fd, buf);
 	      break;
 	  case HTTP::HTTP_CONNECT:
-	      result = processConnectRequest(fd);
+	      result = processConnectRequest(fd, buf);
 	      break;
 	  case HTTP::HTTP_TRACE:
-	      result = processTraceRequest(fd);
+	      result = processTraceRequest(fd, buf);
 	      break;
 	  case HTTP::HTTP_OPTIONS:
-	      result = processOptionsRequest(fd);
+	      result = processOptionsRequest(fd, buf);
 	      break;
 	  case HTTP::HTTP_PUT:
-	      result = processPutRequest(fd);
+	      result = processPutRequest(fd, buf);
 	      break;
 	  case HTTP::HTTP_DELETE:
-	      result = processDeleteRequest(fd);
+	      result = processDeleteRequest(fd, buf);
 	      break;
 	  default:
 	      break;
 	}
     }
 
-    return result;
-}
+#if 0
 
-// A GET request asks the server to send a file to the client
-boost::shared_ptr<amf::Buffer> 
-HTTPServer::processGetRequest(int fd)
-{
-    GNASH_REPORT_FUNCTION;
+    writeNet(fd, result);
 
-//     boost::uint8_t buffer[readsize+1];
-//     const char *ptr = reinterpret_cast<const char *>(buffer);
-//     memset(buffer, 0, readsize+1);
+    _docroot = crcfile.getDocumentRoot();
     
-//    _handler->wait();
-//    _handler->dump();
-
-    cerr << "QUE = " << _que.size() << endl;
-    boost::shared_ptr<amf::Buffer> buf;
-    
-    if (_que.size() == 0) {
-	return buf;
-    }
-    
-    buf = _que.pop();
-//    cerr << "YYYYYYY: " << (char *)buf->reference() << endl;
-//    cerr << hexify(buf->reference(), buf->allocated(), false) << endl;
-    
-    if (buf == 0) {
-     //	log_debug("Que empty, net connection dropped for fd #%d", getFileFd());
-	log_debug("Que empty, net connection dropped for fd #%d", fd);
-	return buf;
-    }
-    
-    clearHeader();
-    processHeaderFields(*buf);
-
     string url = _docroot + _filespec;
+    
     // See if the file is in the cache and already opened.
-    boost::shared_ptr<DiskStream> filestream(cache.findFile(url));
+    boost::shared_ptr<DiskStream> filestream(cache.findFile(_filespec));
     if (filestream) {
-	cerr << "FIXME: found file in cache!" << endl;
+	log_debug("FIXME: found filestream %s in cache!", _filespec);
+	filestream->dump();
     } else {
 	filestream.reset(new DiskStream);
-//	    cerr << "New Filestream at 0x" << hex << filestream.get() << endl;
-	
-//	    cache.addFile(url, filestream);	FIXME: always reload from disk for now.
+	log_network("New filestream %s", _filespec);
+	// cache.addFile(url, filestream);	FIXME: always reload from disk for now.
 	
 	// Oopen the file and read the first chunk into memory
 	if (filestream->open(url)) {
@@ -178,47 +162,97 @@ HTTPServer::processGetRequest(int fd)
 	    if (filestream->getFileType() == DiskStream::FILETYPE_NONE) {
 		formatErrorResponse(HTTPServer::NOT_FOUND);
 	    } else {
-		cache.addPath(_filespec, filestream->getFilespec());
+		// cache.addPath(_filespec, filestream->getFilespec());
+		// cache.addFile(_filespec, filestream);
 	    }
 	}
+	// Close the file but leave resident for now.
+	if (filestream->fullyPopulated()) {
+	    filestream->close();
+	}
+//  	cache.addFile(_filespec, filestream);
+    }
+#endif
+    
+    return _cmd;
+}
+
+// A GET request asks the server to send a file to the client
+cygnal::Buffer &
+HTTPServer::processGetRequest(Handler *hand, int fd, cygnal::Buffer *buf)
+{
+    GNASH_REPORT_FUNCTION;
+
+    // cerr << "QUE = " << _que.size() << endl;
+    
+//    cerr << "YYYYYYY: " << (char *)buf->reference() << endl;
+//    cerr << hexify(buf->reference(), buf->allocated(), false) << endl;
+    
+    if (buf == 0) {
+     //	log_debug("Que empty, net connection dropped for fd #%d", getFileFd());
+	log_debug("Que empty, net connection dropped for fd #%d", fd);
+	cygnal::Buffer buf;
+	return buf;
     }
     
-    // Send the reply
-    amf::Buffer &reply = formatHeader(filestream->getFileType(),
-					  filestream->getFileSize(),
-					  HTTPServer::OK);
+    clearHeader();
+    processHeaderFields(buf);
+
+    _docroot = crcfile.getDocumentRoot();
+    
+    string url = _docroot + _filespec;
+
+    boost::shared_ptr<DiskStream> ds = hand->getDiskStream(fd);
+    if (ds) {
+	_diskstream = ds;
+    }
+    if (!_diskstream) {
+	_diskstream.reset(new DiskStream);
+	log_network("New filestream %s", _filespec);
+    } else {
+	log_network("Reusing filestream %s", _filespec);
+    }
+    
+    // Oopen the file and read the first chunk into memory
+    if (_diskstream->open(url)) {
+	formatErrorResponse(HTTPServer::NOT_FOUND);
+    } else {
+	// Get the file size for the HTTPServer header
+	if (_diskstream->getFileType() == DiskStream::FILETYPE_NONE) {
+	    formatErrorResponse(HTTPServer::NOT_FOUND);
+	} else {
+	    // cache.addPath(_filespec, filestream->getFilespec());
+	    // cache.addFile(_filespec, filestream);
+	}
+    }
+    // Closing the file closes the disk file, but leaves data resident
+    // in memory for future access to this file. If we've been opened,
+    // the next operation is to start writing the file next time
+    // ::play() is called.
+    if (_diskstream->fullyPopulated()) {
+	_diskstream->close();
+    }
+    _diskstream->setState(DiskStream::PLAY);
+// 	cache.addFile(_filespec, _diskstream);
+
+    // Create the reply message
+//     _close = true; Force sending the close connection in the header
+    cygnal::Buffer &reply = formatHeader(_diskstream->getFileType(),
+				      _diskstream->getFileSize(),
+				      HTTPServer::OK);
+
     writeNet(fd, reply);
 
-    size_t filesize = filestream->getFileSize();
-    size_t bytes_read = 0;
-    int ret;
-    size_t page = 0;
+    size_t filesize = _diskstream->getFileSize();
+    // size_t bytes_read = 0;
+    // int ret;
+    // size_t page = 0;
     if (filesize) {
 #ifdef USE_STATS_CACHE
 	struct timespec start;
 	clock_gettime (CLOCK_REALTIME, &start);
-#endif
-	size_t getbytes = 0;
-	if (filesize <= filestream->getPagesize()) {
-	    getbytes = filesize;
-	} else {
-	    getbytes = filestream->getPagesize();
-	}
-	if (filesize >= CACHE_LIMIT) {
-	    do {
-		filestream->loadToMem(page);
-		ret = writeNet(fd, filestream->get(), getbytes);
-		if (ret <= 0) {
-		    break;
-		}
-		bytes_read += ret;
-		page += filestream->getPagesize();
-	    } while (bytes_read <= filesize);
-	} else {
-	    filestream->loadToMem(filesize, 0);
-	    ret = writeNet(fd, filestream->get(), filesize);
-	}
-	filestream->close();
+#endif	
+	
 #ifdef USE_STATS_CACHE
 	struct timespec end;
 	clock_gettime (CLOCK_REALTIME, &end);
@@ -228,23 +262,21 @@ HTTPServer::processGetRequest(int fd)
 	     << time << " seconds for net fd #" << fd << endl;
 #endif
     }
-
-    log_debug("http_handler all done transferring requested file \"%s\".", _filespec);
     
-    return buf;
+    return reply;
 }
 
 // A POST request asks sends a data from the client to the server. After processing
 // the header like we normally do, we then read the amount of bytes specified by
 // the "content-length" field, and then write that data to disk, or decode the amf.
-boost::shared_ptr<amf::Buffer>
-HTTPServer::processPostRequest(int fd)
+boost::shared_ptr<cygnal::Buffer>
+HTTPServer::processPostRequest(int fd, cygnal::Buffer *bufFIXME)
 {
     GNASH_REPORT_FUNCTION;
 
 //    cerr << "QUE1 = " << _que.size() << endl;
 
-    boost::shared_ptr<amf::Buffer> buf;
+    boost::shared_ptr<cygnal::Buffer> buf;
     
     if (_que.size() == 0) {
 	return buf;
@@ -258,9 +290,9 @@ HTTPServer::processPostRequest(int fd)
 //    cerr << __FUNCTION__ << buf->allocated() << " : " << hexify(buf->reference(), buf->allocated(), true) << endl;
     
     clearHeader();
-    boost::uint8_t *data = processHeaderFields(*buf);
+    boost::uint8_t *data = processHeaderFields(buf.get());
     size_t length = strtol(getField("content-length").c_str(), NULL, 0);
-    boost::shared_ptr<amf::Buffer> content(new amf::Buffer(length));
+    boost::shared_ptr<cygnal::Buffer> content(new cygnal::Buffer(length));
     int ret = 0;
     if (buf->allocated() - (data - buf->reference()) ) {
 //	cerr << "Don't need to read more data: have " << buf->allocated() << " bytes" << endl;
@@ -283,7 +315,7 @@ HTTPServer::processPostRequest(int fd)
 	log_debug("Got AMF data in POST");
 #if 0
 	amf::AMF amf;
-	boost::shared_ptr<amf::Element> el = amf.extractAMF(content.reference(), content.end());
+	boost::shared_ptr<cygnal::Element> el = amf.extractAMF(content.reference(), content.end());
 	el->dump();		// FIXME: do something intelligent
 				// with this Element
 #endif
@@ -304,17 +336,17 @@ HTTPServer::processPostRequest(int fd)
   	cgis.startCGI(_filespec, true, CGIBIN_PORT);
  	cgis.createClient("localhost", CGIBIN_PORT);
 	cgis.writeNet(*content);
-	boost::shared_ptr<amf::Buffer> reply = cgis.readNet();
+	boost::shared_ptr<cygnal::Buffer> reply = cgis.readNet();
 	
 	writeNet(fd, *reply);
 //	cgis.stopCGI(_filespec);
 #else
-	vector<boost::shared_ptr<amf::Element> > headers = parseEchoRequest(*content);
-  	//boost::shared_ptr<amf::Element> &el0 = headers[0];
+	vector<boost::shared_ptr<cygnal::Element> > headers = parseEchoRequest(*content);
+  	//boost::shared_ptr<cygnal::Element> &el0 = headers[0];
 
 	if (headers.size() >= 4) {
 	    if (headers[3]) {
-		amf::Buffer &reply = formatEchoResponse(headers[1]->getName(), *headers[3]);
+		cygnal::Buffer &reply = formatEchoResponse(headers[1]->getName(), *headers[3]);
 // 	    cerr << "FIXME 3: " << hexify(reply.reference(), reply.allocated(), true) << endl;
 // 	    cerr << "FIXME 3: " << hexify(reply.reference(), reply.allocated(), false) << endl;
 		writeNet(fd, reply);
@@ -322,74 +354,74 @@ HTTPServer::processPostRequest(int fd)
  	}
 #endif
     } else {
-	amf::Buffer &reply = formatHeader(_filetype, _filesize, HTTPServer::OK);
+	cygnal::Buffer &reply = formatHeader(_filetype, _filesize, HTTPServer::OK);
 	writeNet(fd, reply);
     }
 
     return buf;
 }
 
-boost::shared_ptr<amf::Buffer>
-HTTPServer::processPutRequest(int /* fd */)
+boost::shared_ptr<cygnal::Buffer>
+HTTPServer::processPutRequest(int /* fd */, cygnal::Buffer */* buf */)
 {
-    boost::shared_ptr<amf::Buffer> buf;
+    boost::shared_ptr<cygnal::Buffer> buf;
 //    GNASH_REPORT_FUNCTION;
     log_unimpl("PUT request");
 
     return buf;
 }
 
-boost::shared_ptr<amf::Buffer> 
-HTTPServer::processDeleteRequest(int /* fd */)
+boost::shared_ptr<cygnal::Buffer> 
+HTTPServer::processDeleteRequest(int /* fd */, cygnal::Buffer */* buf */)
 {
 //    GNASH_REPORT_FUNCTION;
-    boost::shared_ptr<amf::Buffer> buf;
+    boost::shared_ptr<cygnal::Buffer> buf;
     log_unimpl("DELETE request");
     
     return buf;
 }
 
-boost::shared_ptr<amf::Buffer> 
-HTTPServer::processConnectRequest(int /* fd */)
+boost::shared_ptr<cygnal::Buffer> 
+HTTPServer::processConnectRequest(int /* fd */, cygnal::Buffer */* buf */)
 {
 //    GNASH_REPORT_FUNCTION;
-    boost::shared_ptr<amf::Buffer> buf;
+    boost::shared_ptr<cygnal::Buffer> buf;
     log_unimpl("CONNECT request");
 
     return buf;
 }
 
-boost::shared_ptr<amf::Buffer>
-HTTPServer::processOptionsRequest(int /* fd */)
+boost::shared_ptr<cygnal::Buffer>
+HTTPServer::processOptionsRequest(int /* fd */, cygnal::Buffer */* buf */)
 {
 //    GNASH_REPORT_FUNCTION;
-    boost::shared_ptr<amf::Buffer> buf;
+    boost::shared_ptr<cygnal::Buffer> buf;
     log_unimpl("OPTIONS request");
 
     return buf;
 }
 
-boost::shared_ptr<amf::Buffer>
-HTTPServer::processHeadRequest(int /* fd */)
+boost::shared_ptr<cygnal::Buffer>
+HTTPServer::processHeadRequest(int /* fd */, cygnal::Buffer */* buf */)
 {
 //    GNASH_REPORT_FUNCTION;
-    boost::shared_ptr<amf::Buffer> buf;
+    boost::shared_ptr<cygnal::Buffer> buf;
     log_unimpl("HEAD request");
     
     return buf;
 }
 
-boost::shared_ptr<amf::Buffer>
-HTTPServer::processTraceRequest(int /* fd */)
+boost::shared_ptr<cygnal::Buffer>
+HTTPServer::processTraceRequest(int /* fd */, cygnal::Buffer */* buf */)
 {
 //    GNASH_REPORT_FUNCTION;
-    boost::shared_ptr<amf::Buffer> buf;
+    boost::shared_ptr<cygnal::Buffer> buf;
     log_unimpl("TRACE request");
     
     return buf;
 }
 
-amf::Buffer &
+cygnal::Buffer &
 HTTPServer::formatErrorResponse(http_status_e code)
 {
 //    GNASH_REPORT_FUNCTION;
@@ -426,7 +458,7 @@ HTTPServer::formatErrorResponse(http_status_e code)
     return _buffer;
 }
 
-amf::Buffer &
+cygnal::Buffer &
 HTTPServer::formatGetReply(http_status_e code)
 {
 
@@ -435,7 +467,7 @@ HTTPServer::formatGetReply(http_status_e code)
     return formatHeader(_filesize, code);
 }
 
-amf::Buffer &
+cygnal::Buffer &
 HTTPServer::formatGetReply(size_t size, http_status_e code)
 {
 //    GNASH_REPORT_FUNCTION;
@@ -460,7 +492,7 @@ HTTPServer::formatGetReply(size_t size, http_status_e code)
     return _buffer;
 }
 
-amf::Buffer &
+cygnal::Buffer &
 HTTPServer::formatPostReply(rtmpt_cmd_e /* code */)
 {
     GNASH_REPORT_FUNCTION;
@@ -474,7 +506,7 @@ HTTPServer::formatPostReply(rtmpt_cmd_e /* code */)
 
 #if 0
     formatHeader(_filesize, code);
-    boost::shared_ptr<amf::Buffer> buf = new amf::Buffer;
+    boost::shared_ptr<cygnal::Buffer> buf = new cygnal::Buffer;
     if (_header.str().size()) {
 	buf->resize(_header.str().size());
 	string str = _header.str();
@@ -495,12 +527,12 @@ HTTPServer::formatPostReply(rtmpt_cmd_e /* code */)
 #ifndef USE_CGIBIN
 // Parse an Echo Request message coming from the Red5 echo_test. This
 // method should only be used for testing purposes.
-vector<boost::shared_ptr<amf::Element > >
+vector<boost::shared_ptr<cygnal::Element > >
 HTTPServer::parseEchoRequest(boost::uint8_t *data, size_t size)
 {
 //    GNASH_REPORT_FUNCTION;
     
-    vector<boost::shared_ptr<amf::Element > > headers;
+    vector<boost::shared_ptr<cygnal::Element > > headers;
 	
     // skip past the header bytes, we don't care about them.
     boost::uint8_t *tmpptr = data + 6;
@@ -511,7 +543,7 @@ HTTPServer::parseEchoRequest(boost::uint8_t *data, size_t size)
 
     // Get the first name, which is a raw string, and not preceded by
     // a type byte.
-    boost::shared_ptr<amf::Element > el1(new amf::Element);
+    boost::shared_ptr<cygnal::Element > el1(new cygnal::Element);
     
     // If the length of the name field is corrupted, then we get out of
     // range quick, and corrupt memory. This is a bit of a hack, but
@@ -530,7 +562,7 @@ HTTPServer::parseEchoRequest(boost::uint8_t *data, size_t size)
     // a type byte.
     length = ntohs((*(boost::uint16_t *)tmpptr) & 0xffff);
     tmpptr += sizeof(boost::uint16_t);
-    boost::shared_ptr<amf::Element > el2(new amf::Element);
+    boost::shared_ptr<cygnal::Element > el2(new cygnal::Element);
 
 //     std::string name2(reinterpret_cast<const char *>(tmpptr), length);
 //     el2->setName(name2.c_str(), name2.size());
@@ -550,11 +582,11 @@ HTTPServer::parseEchoRequest(boost::uint8_t *data, size_t size)
     // Get the last two pieces of data, which are both AMF encoded
     // with a type byte.
     amf::AMF amf;
-    boost::shared_ptr<amf::Element> el3 = amf.extractAMF(tmpptr, tmpptr + size);
+    boost::shared_ptr<cygnal::Element> el3 = amf.extractAMF(tmpptr, tmpptr + size);
     headers.push_back(el3);
     tmpptr += amf.totalsize();
     
-    boost::shared_ptr<amf::Element> el4 = amf.extractAMF(tmpptr, tmpptr + size);
+    boost::shared_ptr<cygnal::Element> el4 = amf.extractAMF(tmpptr, tmpptr + size);
     headers.push_back(el4);
 
      return headers;
@@ -563,14 +595,14 @@ HTTPServer::parseEchoRequest(boost::uint8_t *data, size_t size)
 // format a response to the 'echo' test used for testing Gnash. This
 // is only used for testing by developers. The format appears to be
 // two strings, followed by a double, followed by the "onResult".
-amf::Buffer &
-HTTPServer::formatEchoResponse(const std::string &num, amf::Element &el)
+cygnal::Buffer &
+HTTPServer::formatEchoResponse(const std::string &num, cygnal::Element &el)
 {
 //    GNASH_REPORT_FUNCTION;
-    boost::shared_ptr<amf::Buffer> data;
+    boost::shared_ptr<cygnal::Buffer> data;
 
-    amf::Element nel;
-    if (el.getType() == amf::Element::TYPED_OBJECT_AMF0) {
+    cygnal::Element nel;
+    if (el.getType() == cygnal::Element::TYPED_OBJECT_AMF0) {
 	nel.makeTypedObject();
 	string name = el.getName();
 	nel.setName(name);
@@ -578,7 +610,7 @@ HTTPServer::formatEchoResponse(const std::string &num, amf::Element &el)
 	    // FIXME: see about using std::reverse() instead.
 	    for (int i=el.propertySize()-1; i>=0; i--) {
 // 	    for (int i=0 ; i<el.propertySize(); i++) {
-		boost::shared_ptr<amf::Element> child = el.getProperty(i);
+		boost::shared_ptr<cygnal::Element> child = el.getProperty(i);
 		nel.addProperty(child);
 	    }
 	    data = nel.encode();
@@ -592,14 +624,14 @@ HTTPServer::formatEchoResponse(const std::string &num, amf::Element &el)
     return formatEchoResponse(num, data->reference(), data->allocated());
 }
 
-amf::Buffer &
-HTTPServer::formatEchoResponse(const std::string &num, amf::Buffer &data)
+cygnal::Buffer &
+HTTPServer::formatEchoResponse(const std::string &num, cygnal::Buffer &data)
 {
 //    GNASH_REPORT_FUNCTION;
     return formatEchoResponse(num, data.reference(), data.allocated());
 }
 
-amf::Buffer &
+cygnal::Buffer &
 HTTPServer::formatEchoResponse(const std::string &num, boost::uint8_t *data, size_t size)
 {
 //    GNASH_REPORT_FUNCTION;
@@ -607,8 +639,8 @@ HTTPServer::formatEchoResponse(const std::string &num, boost::uint8_t *data, siz
     //boost::uint8_t *tmpptr  = data;
     
     // FIXME: temporary hacks while debugging
-    amf::Buffer fixme("00 00 00 00 00 01");
-    amf::Buffer fixme2("ff ff ff ff");
+    cygnal::Buffer fixme("00 00 00 00 00 01");
+    cygnal::Buffer fixme2("ff ff ff ff");
     
     _buffer = "HTTPServer/1.1 200 OK\r\n";
     formatContentType(DiskStream::FILETYPE_AMF);
@@ -629,23 +661,23 @@ HTTPServer::formatEchoResponse(const std::string &num, boost::uint8_t *data, siz
     // the request, a slash followed by a number like "/2".
     string result = num;
     result += "/onResult";
-    boost::shared_ptr<amf::Buffer> res = amf::AMF::encodeString(result);
+    boost::shared_ptr<cygnal::Buffer> res = amf::AMF::encodeString(result);
     _buffer.append(res->begin()+1, res->size()-1);
 
     // Add the null data item
-    boost::shared_ptr<amf::Buffer> null = amf::AMF::encodeString("null");
+    boost::shared_ptr<cygnal::Buffer> null = amf::AMF::encodeString("null");
     _buffer.append(null->begin()+1, null->size()-1);
 
     // Add the other binary blob
     _buffer += fixme2;
 
-    amf::Element::amf0_type_e type = static_cast<amf::Element::amf0_type_e>(*data);
-    if ((type == amf::Element::UNSUPPORTED_AMF0)
-	|| (type == amf::Element::NULL_AMF0)) {
+    cygnal::Element::amf0_type_e type = static_cast<cygnal::Element::amf0_type_e>(*data);
+    if ((type == cygnal::Element::UNSUPPORTED_AMF0)
+	|| (type == cygnal::Element::NULL_AMF0)) {
 	_buffer += type;
 	// Red5 returns a NULL object when it's recieved an undefined one in the echo_test
-    } else if (type == amf::Element::UNDEFINED_AMF0) {
-	_buffer += amf::Element::NULL_AMF0;
+    } else if (type == cygnal::Element::UNDEFINED_AMF0) {
+	_buffer += cygnal::Element::NULL_AMF0;
     } else {
 	// Add the AMF data we're echoing back
 	if (size) {
@@ -840,7 +872,7 @@ HTTPServer::extractCommand(boost::uint8_t *data)
 	    // This is fine as long as end is within the buffer.
 	    _filespec = std::string(start, end);
 	}
-	log_debug("Requesting file: \"%s\"", _filespec);
+	// log_debug("Requesting file: \"%s\"", _filespec);
 
 	// The third field is always the HTTP version
 	// The version is the last field and is the protocol name
@@ -849,14 +881,14 @@ HTTPServer::extractCommand(boost::uint8_t *data)
 	// in it. It's actually two separate integers.
 	_version.major = *(end+6) - '0';
 	_version.minor = *(end+8) - '0';
-	log_debug (_("Version: %d.%d"), _version.major, _version.minor);
+	// log_debug (_("Version: %d.%d"), _version.major, _version.minor);
     }
 
     return cmd;
 }
 
 boost::uint8_t *
-HTTPServer::processHeaderFields(amf::Buffer &buf)
+HTTPServer::processHeaderFields(cygnal::Buffer &buf)
 {
   //    GNASH_REPORT_FUNCTION;
     string head(reinterpret_cast<const char *>(buf.reference()));
@@ -923,7 +955,7 @@ HTTPServer::processHeaderFields(amf::Buffer &buf)
 		    // in it. It's actually two separate integers.
 		    _version.major = i->at(pos+5) - '0';
 		    _version.minor = i->at(pos+7) - '0';
-		    log_debug (_("Version: %d.%d"), _version.major, _version.minor);
+		    // log_debug (_("Version: %d.%d"), _version.major, _version.minor);
 		    // the filespec in the request is the middle field, deliminated
 		    // by a space on each end.
 		    if (params != string::npos) {
@@ -933,7 +965,7 @@ HTTPServer::processHeaderFields(amf::Buffer &buf)
 		    } else {
 			_filespec = i->substr(start+1, pos-start-2);
 		    }
-		    log_debug("Requesting file: \"%s\"", _filespec);
+		    // log_debug("Requesting file: \"%s\"", _filespec);
 
 		    // HTTP 1.1 enables persistant network connections
 		    // by default.
@@ -955,99 +987,97 @@ void
 HTTPServer::dump()
 {
 //    GNASH_REPORT_FUNCTION;
+    if (_diskstream) {
+	_diskstream->dump();
+    }
 }
-
-extern "C" {
-
+    
 bool
-http_handler(Network::thread_params_t *args)
+HTTPServer::http_handler(Handler *hand, int netfd, cygnal::Buffer *buf)
 {
     GNASH_REPORT_FUNCTION;
 
-    string url, filespec, parameters;
-    HTTPServer *www = new HTTPServer;
-    bool result = false;
+    // Handler *hand = reinterpret_cast<Handler *>(args->handler);
+    // cygnal::Buffer *buf = args->buffer;
+    // boost::shared_ptr<HTTPServer> www(new HTTPServer); // = hand->getHTTPHandler    (args->netfd);
     
-//    Network *net = reinterpret_cast<Network *>(args->handler);
-    bool done = false;
-//    www.setHandler(net);
-
-    log_network(_("Starting HTTP Handler for fd #%d, tid %d"),
-	      args->netfd, args->tid);
+    string url, parameters;
+    // by default, only look once unless changed later
+    // www->setDocRoot(crcfile.getDocumentRoot());
+    // log_network("Docroot for HTTP files is %s", crcfile.getDocumentRoot());
+    log_network("Processing HTTP data for fd #%d", netfd);
     
-    www->setDocRoot(crcfile.getDocumentRoot());
-
-    log_network("Docroot for HTTP files is %s", crcfile.getDocumentRoot());
-
-    log_network("Starting to wait for data in net for fd #%d", args->netfd);
-
     // Wait for data, and when we get it, process it.
-    do {
-	
 #ifdef USE_STATISTICS
-	struct timespec start;
-	clock_gettime (CLOCK_REALTIME, &start);
+    struct timespec start;
+    clock_gettime (CLOCK_REALTIME, &start);
 #endif
 
+    if (buf) {
+	log_network("FIXME: Existing data in packet!");
+    } else {
+	log_network("FIXME: No existing data in packet!");    
 	// See if we have any messages waiting
-	if (www->recvMsg(args->netfd) == 0) {
-	    done = true;
+	if (recvMsg(netfd) == 0) {
+	    log_debug("Net HTTP server failed to read from fd #%d...", netfd);
+	    return false;
 	}
-
-	// Process incoming messages
-	if (!www->processClientRequest(args->netfd)) {
-//	    hand->die();	// tell all the threads for this connection to die
-//	    hand->notifyin();
-	    log_debug("Net HTTP server done for fd #%d...", args->netfd);
-//	    done = true;
+    }
+    
+    // Process incoming messages
+    HTTP::http_method_e cmd = processClientRequest(hand, netfd, buf);    
+    if (cmd != HTTP::HTTP_GET) {
+	log_debug("No active DiskStreams for fd #%d: %s...", netfd, _filespec);
+    } else {
+	if (_diskstream) {
+	    log_debug("Found active DiskStream! for fd #%d: %s", netfd, _filespec);
+	    hand->setDiskStream(netfd, _diskstream);
+ 	    cache.addFile(_filespec, _diskstream);
+// Send the first chunk of the file to the client.
+	    // log_network("Sending first chunk of %s", _filespec);
+	    _diskstream->play(netfd, false);
 	}
+    }
+    
 //	www->dump();
-	if ((www->getField("content-type") == "application/x-amf")
-	    && (www->getField("content-type") == "application/x-amf")
-	    && (www->getFilespec() == "/echo/gateway")) {
-	    cerr << "GOT A GATEWAY REQUEST" << endl;
-	}
-	
+    if ((getField("content-type") == "application/x-amf")
+	&& (getField("content-type") == "application/x-amf")
+	&& (getFilespec() == "/echo/gateway")) {
+	cerr << "GOT A GATEWAY REQUEST" << endl;
+    }
+    
 #if 0
-	string response = cache.findResponse(filestream->getFilespec());
-	if (response.empty()) {
-	    cerr << "FIXME no cache hit for: " << www.getFilespec() << endl;
+    string response = cache.findResponse(filestream->getFilespec());
+    if (response.empty()) {
+	cerr << "FIXME no cache hit for: " << www.getFilespec() << endl;
 //	    www.clearHeader();
-// 	    amf::Buffer &ss = www.formatHeader(filestream->getFileSize(), HTTP::LIFE_IS_GOOD);
+// 	    cygnal::Buffer &ss = www.formatHeader(filestream->getFileSize(), HTTP::LIFE_IS_GOOD);
 // 	    www.writeNet(args->netfd, (boost::uint8_t *)www.getHeader().c_str(), www.getHeader().size());
 // 	    cache.addResponse(www.getFilespec(), www.getHeader());
-	} else {
-	    cerr << "FIXME cache hit on: " << www.getFilespec() << endl;
-	    www.writeNet(args->netfd, (boost::uint8_t *)response.c_str(), response.size());
-	}	
+    } else {
+	cerr << "FIXME cache hit on: " << www.getFilespec() << endl;
+	www.writeNet(args->netfd, (boost::uint8_t *)response.c_str(), response.size());
+    }	
 #endif
-	
-	// Unless the Keep-Alive flag is set, this isn't a persisant network
-	// connection.
-	if (!www->keepAlive()) {
-	    log_debug("Keep-Alive is off", www->keepAlive());
-	    result = false;
-	    done = true;
-	} else {
-	    log_debug("Keep-Alive is on", www->keepAlive());
-	    result = true;
-//	    done = true;
-	}
+    
+    // Unless the Keep-Alive flag is set, this isn't a persisant network
+    // connection.
+    if (!keepAlive()) {
+	log_debug("Keep-Alive is off", keepAlive());
+    } else {
+	log_debug("Keep-Alive is on", keepAlive());
+    }
 #ifdef USE_STATISTICS
-	struct timespec end;
-	clock_gettime (CLOCK_REALTIME, &end);
-	log_debug("Processing time for GET request was %f seconds",
-		  (float)((end.tv_sec - start.tv_sec) + ((end.tv_nsec - start.tv_nsec)/1e9)));
+    struct timespec end;
+    clock_gettime (CLOCK_REALTIME, &end);
+    log_debug("Processing time for GET request was %f seconds",
+	      static_cast<float>(((end.tv_sec - start.tv_sec) +
+				  ((end.tv_nsec - start.tv_nsec)/1e9))));
 #endif
-    } while(done != true);
     
-//    hand->notify();
+    return keepAlive();
     
-    log_debug("http_handler all done now finally...");
-
-    return result;
-} // end of httphandler
-} // end of extern C
+} // end of http_handler
     
 } // end of gnash namespace
 
