@@ -59,7 +59,6 @@
 #endif 
 using namespace gnash;
 using namespace std;
-using namespace amf;
 
 namespace cygnal
 {
@@ -92,7 +91,7 @@ RTMPServer::~RTMPServer()
 }
 
 
-boost::shared_ptr<amf::Element>
+boost::shared_ptr<cygnal::Element>
 RTMPServer::processClientHandShake(int fd)
 {
     GNASH_REPORT_FUNCTION;
@@ -109,17 +108,17 @@ RTMPServer::processClientHandShake(int fd)
     
     // These store the information we need from the initial
     /// NetConnection object.
-    boost::scoped_ptr<amf::Element> nc;
-    boost::shared_ptr<amf::Buffer>  pkt;
-    boost::shared_ptr<amf::Element> tcurl;
-    boost::shared_ptr<amf::Element> swfurl;
-    boost::shared_ptr<amf::Element> encoding;
+    boost::scoped_ptr<cygnal::Element> nc;
+    boost::shared_ptr<cygnal::Buffer>  pkt;
+    boost::shared_ptr<cygnal::Element> tcurl;
+    boost::shared_ptr<cygnal::Element> swfurl;
+    boost::shared_ptr<cygnal::Element> encoding;
 
 //     RTMP::rtmp_headersize_e response_head_size = RTMP::HEADER_12;
     
     // Read the handshake bytes sent by the client when requesting
     // a connection.
-    boost::shared_ptr<amf::Buffer> handshake1 = RTMP::recvMsg(fd);
+    boost::shared_ptr<cygnal::Buffer> handshake1 = RTMP::recvMsg(fd);
     // See if we have data in the handshake, we should have 1537 bytes
     if (!handshake1) {
 	log_error("Failed to read the handshake from the client.");
@@ -134,7 +133,7 @@ RTMPServer::processClientHandShake(int fd)
     
     // Read the response from the client from the handshale reponse we
     // just sent.
-    boost::shared_ptr<amf::Buffer> handshake2 = RTMP::recvMsg(fd);
+    boost::shared_ptr<cygnal::Buffer> handshake2 = RTMP::recvMsg(fd);
     // See if we have data in the handshake, we should have 1536 bytes
     if (handshake2 == 0) {
 	log_error("failed to read the handshake from the client.");
@@ -148,7 +147,7 @@ RTMPServer::processClientHandShake(int fd)
     // Wmake sure we got data before trying to process it
     if (!pkt) {
 	log_error("Didn't receive any data in handshake!");
-	tcurl.reset(new amf::Element);
+	tcurl.reset(new cygnal::Element);
 	return tcurl;		// nc is empty
     }
     
@@ -181,7 +180,7 @@ RTMPServer::processClientHandShake(int fd)
     // now build a copy of the data but skip over the RTMP header
     // bytes every chunk size biundary. All RTMP headers at this stage
     // are 1 byte ones.
-    boost::scoped_ptr<amf::Buffer> newptr(new amf::Buffer(qhead->bodysize));
+    boost::scoped_ptr<cygnal::Buffer> newptr(new cygnal::Buffer(qhead->bodysize));
     if (qhead->bodysize > RTMP_VIDEO_PACKET_SIZE) {
 	log_network("De chunkifying the NetConnection packet.");
 	int nbytes = 0;
@@ -227,7 +226,7 @@ RTMPServer::processClientHandShake(int fd)
     // causes Async I/O errors in the client.
     if (!encoding) {
 	// Send a onBWDone to the client to start the new NetConnection,
-	boost::shared_ptr<amf::Buffer> bwdone = encodeBWDone(2.0);
+	boost::shared_ptr<cygnal::Buffer> bwdone = encodeBWDone(2.0);
 	if (RTMP::sendMsg(fd, qhead->channel, RTMP::HEADER_8,
 			  bwdone->size(), RTMP::INVOKE, RTMPMsg::FROM_SERVER, *bwdone)) {
 	    log_network("Sent onBWDone to client");
@@ -239,7 +238,7 @@ RTMPServer::processClientHandShake(int fd)
     }
     
     // Send a Set Client Window Size to the client
-    boost::shared_ptr<amf::Buffer> winsize(new amf::Buffer(sizeof(boost::uint32_t)));
+    boost::shared_ptr<cygnal::Buffer> winsize(new cygnal::Buffer(sizeof(boost::uint32_t)));
     boost::uint32_t swapped = 0x20000;
     swapBytes(&swapped, sizeof(boost::uint32_t));
     *winsize += swapped;
@@ -253,7 +252,7 @@ RTMPServer::processClientHandShake(int fd)
     }
 
     // Send a ping to the client to reset the new NetConnection,
-    boost::shared_ptr<amf::Buffer> ping_reset =
+    boost::shared_ptr<cygnal::Buffer> ping_reset =
 	encodePing(RTMP::PING_RESET, 0);
     if (RTMP::sendMsg(fd, RTMP_SYSTEM_CHANNEL, RTMP::HEADER_8,
 		      ping_reset->size(), RTMP::USER, RTMPMsg::FROM_SERVER, *ping_reset)) {
@@ -267,7 +266,7 @@ RTMPServer::processClientHandShake(int fd)
     // Send the packet to notify the client that the
     // NetConnection::connect() was sucessful. After the client
     // receives this, the handhsake is completed.
-    boost::shared_ptr<amf::Buffer> response =
+    boost::shared_ptr<cygnal::Buffer> response =
 	encodeResult(RTMPMsg::NC_CONNECT_SUCCESS);
     if (RTMP::sendMsg(fd, 3, RTMP::HEADER_8, response->allocated(),
 		      RTMP::INVOKE, RTMPMsg::FROM_SERVER, *response)) {
@@ -286,7 +285,7 @@ RTMPServer::processClientHandShake(int fd)
 // in one big packet as doing otherwise seems to cause subtle timing
 // problems with the Adobe player. This way it connects every time.
 bool
-RTMPServer::handShakeResponse(int fd, amf::Buffer &handshake)
+RTMPServer::handShakeResponse(int fd, cygnal::Buffer &handshake)
 {
     GNASH_REPORT_FUNCTION;
 
@@ -295,7 +294,7 @@ RTMPServer::handShakeResponse(int fd, amf::Buffer &handshake)
 
     // the response handshake is twice the size of the one we just
     // received for a total of 3072 bytes, plus room for the version.
-    boost::scoped_ptr<amf::Buffer> zeros(new amf::Buffer(RTMP_HANDSHAKE_SIZE*2
+    boost::scoped_ptr<cygnal::Buffer> zeros(new cygnal::Buffer(RTMP_HANDSHAKE_SIZE*2
 					 + RTMP_HANDSHAKE_VERSION_SIZE));
     zeros->clear();		// set entire buffer to zeros
 
@@ -342,11 +341,11 @@ RTMPServer::handShakeResponse(int fd, amf::Buffer &handshake)
     return true;    
 }
 
-boost::shared_ptr<amf::Buffer>
-RTMPServer::serverFinish(int fd, amf::Buffer &handshake1, amf::Buffer &handshake2)
+boost::shared_ptr<cygnal::Buffer>
+RTMPServer::serverFinish(int fd, cygnal::Buffer &handshake1, cygnal::Buffer &handshake2)
 {
     GNASH_REPORT_FUNCTION;
-    boost::shared_ptr<amf::Buffer> buf;
+    boost::shared_ptr<cygnal::Buffer> buf;
 
     // sanity check our input data. We do this seperately as an empty
     // buffer means data wasn't read correctly from the network. We
@@ -407,7 +406,7 @@ RTMPServer::serverFinish(int fd, amf::Buffer &handshake1, amf::Buffer &handshake
 }
 
 bool
-RTMPServer::packetSend(amf::Buffer &/* buf */)
+RTMPServer::packetSend(cygnal::Buffer &/* buf */)
  {
     GNASH_REPORT_FUNCTION;
     return false;
@@ -415,7 +414,7 @@ RTMPServer::packetSend(amf::Buffer &/* buf */)
 
 // This overrides using same method from the base RTMP class.
 bool
-RTMPServer::packetRead(amf::Buffer &buf)
+RTMPServer::packetRead(cygnal::Buffer &buf)
 {
     GNASH_REPORT_FUNCTION;
 
@@ -459,12 +458,12 @@ RTMPServer::packetRead(amf::Buffer &buf)
     boost::uint8_t* tooFar = ptr+300+sizeof(int); // FIXME:
     
     AMF amf_obj;
-    boost::shared_ptr<amf::Element> el1 = amf_obj.extractAMF(ptr, tooFar);
+    boost::shared_ptr<cygnal::Element> el1 = amf_obj.extractAMF(ptr, tooFar);
     ptr += amf_obj.totalsize();
-    boost::shared_ptr<amf::Element> el2 = amf_obj.extractAMF(ptr, tooFar);
+    boost::shared_ptr<cygnal::Element> el2 = amf_obj.extractAMF(ptr, tooFar);
 
     int size = 0;
-    boost::shared_ptr<amf::Element> el;
+    boost::shared_ptr<cygnal::Element> el;
     while ( size < static_cast<boost::uint16_t>(_header.bodysize) - 24 ) {
 	if (ptr) {
 	    el = amf_obj.extractProperty(ptr, tooFar);
@@ -490,7 +489,7 @@ RTMPServer::packetRead(amf::Buffer &buf)
     log_network (_("Reading AMF packets till we're done..."));
     // buf->dump();
     while (ptr < end) {
-	boost::shared_ptr<amf::Element> el(new amf::Element);
+	boost::shared_ptr<cygnal::Element> el(new cygnal::Element);
 	ptr = amf.extractProperty(el, ptr);
 	addProperty(el);
 	// el->dump();
@@ -504,7 +503,7 @@ RTMPServer::packetRead(amf::Buffer &buf)
 	buf = _que->merge(buf);
     }
     while ((ptr - buf->begin()) < static_cast<int>(actual_size)) {
-	boost::shared_ptr<amf::Element> el(new amf::Element);
+	boost::shared_ptr<cygnal::Element> el(new cygnal::Element);
 	if (ptr) {
 	    ptr = amf.extractProperty(el, ptr);
 	    addProperty(el);
@@ -610,7 +609,7 @@ RTMPServer::encodeResult(RTMPMsg::rtmp_status_e status)
     return encodeResult(status, _filespec, _streamid);
 }
     
-boost::shared_ptr<amf::Buffer> 
+boost::shared_ptr<cygnal::Buffer> 
 RTMPServer::encodeResult(gnash::RTMPMsg::rtmp_status_e status, const std::string &filename)
 {
 //    GNASH_REPORT_FUNCTION;
@@ -618,14 +617,14 @@ RTMPServer::encodeResult(gnash::RTMPMsg::rtmp_status_e status, const std::string
     return encodeResult(status, filename, _streamid, clientid);
 }
 
-boost::shared_ptr<amf::Buffer> 
+boost::shared_ptr<cygnal::Buffer> 
 RTMPServer::encodeResult(gnash::RTMPMsg::rtmp_status_e status, const std::string &filename, double &clientid)
 {
 //    GNASH_REPORT_FUNCTION;
     return encodeResult(status, filename, _streamid, clientid);
 }
 
-boost::shared_ptr<amf::Buffer> 
+boost::shared_ptr<cygnal::Buffer> 
 RTMPServer::encodeResult(gnash::RTMPMsg::rtmp_status_e status, double &transid)
 {
 //    GNASH_REPORT_FUNCTION;
@@ -633,7 +632,7 @@ RTMPServer::encodeResult(gnash::RTMPMsg::rtmp_status_e status, double &transid)
     return encodeResult(status, "", transid, clientid);
 }
 
-boost::shared_ptr<amf::Buffer> 
+boost::shared_ptr<cygnal::Buffer> 
 RTMPServer::encodeResult(gnash::RTMPMsg::rtmp_status_e status, const std::string &filename, double &transid, double &clientid)
 {
 //    GNASH_REPORT_FUNCTION;
@@ -674,15 +673,15 @@ RTMPServer::encodeResult(gnash::RTMPMsg::rtmp_status_e status, const std::string
       {
 // 	  errstr = new Element;
 // 	  errstr->makeString("error");
-	  boost::shared_ptr<amf::Element> level(new Element);
+	  boost::shared_ptr<cygnal::Element> level(new Element);
 	  level->makeString("level", "error");
 	  top.addProperty(level);
 
-	  boost::shared_ptr<amf::Element> description(new Element);
+	  boost::shared_ptr<cygnal::Element> description(new Element);
 	  description->makeString("description", "Connection Failed.");
 	  top.addProperty(description);
 	  
-	  boost::shared_ptr<amf::Element> code(new Element);
+	  boost::shared_ptr<cygnal::Element> code(new Element);
 	  code->makeString("code", "Connection.Connect.Failed");
 	  top.addProperty(code);
       }
@@ -692,29 +691,29 @@ RTMPServer::encodeResult(gnash::RTMPMsg::rtmp_status_e status, const std::string
 // 	  delete str;
 // 	  str = new Element;
 // 	  str->makeString("error");
-	  boost::shared_ptr<amf::Element> level(new Element);
+	  boost::shared_ptr<cygnal::Element> level(new Element);
 	  level->makeString("level", "error");
 	  top.addProperty(level);
 
-	  boost::shared_ptr<amf::Element> description(new Element);
+	  boost::shared_ptr<cygnal::Element> description(new Element);
 	  description->makeString("description", "Connection Rejected.");
 	  top.addProperty(description);
 	  
-	  boost::shared_ptr<amf::Element> code(new Element);
+	  boost::shared_ptr<cygnal::Element> code(new Element);
 	  code->makeString("code", "NetConnection.Connect.Rejected");
 	  top.addProperty(code);
       }
       case RTMPMsg::NC_CONNECT_SUCCESS:
       {
-	  boost::shared_ptr<amf::Element> level(new Element);
+	  boost::shared_ptr<cygnal::Element> level(new Element);
 	  level->makeString("level", "status");
 	  top.addProperty(level);
 	  
-	  boost::shared_ptr<amf::Element> code(new Element);
+	  boost::shared_ptr<cygnal::Element> code(new Element);
 	  code->makeString("code", "NetConnection.Connect.Success");
 	  top.addProperty(code);
 
-	  boost::shared_ptr<amf::Element> description(new Element);
+	  boost::shared_ptr<cygnal::Element> description(new Element);
 	  description->makeString("description", "Connection succeeded.");
 	  top.addProperty(description);
       }
@@ -732,15 +731,15 @@ RTMPServer::encodeResult(gnash::RTMPMsg::rtmp_status_e status, const std::string
       {
 	  str->makeString("onStatus");
 
-	  boost::shared_ptr<amf::Element> level(new Element);
+	  boost::shared_ptr<cygnal::Element> level(new Element);
 	  level->makeString("level", "status");
 	  top.addProperty(level);
 
-	  boost::shared_ptr<amf::Element> code(new Element);
+	  boost::shared_ptr<cygnal::Element> code(new Element);
 	  code->makeString("code", "NetStream.Pause.Notify");
 	  top.addProperty(code);
 
-	  boost::shared_ptr<amf::Element> description(new Element);
+	  boost::shared_ptr<cygnal::Element> description(new Element);
 	  string field = "Pausing ";
 	  if (!filename.empty()) {
 	      field += filename;
@@ -748,11 +747,11 @@ RTMPServer::encodeResult(gnash::RTMPMsg::rtmp_status_e status, const std::string
 	  description->makeString("description", field);
 	  top.addProperty(description);
 	  
-	  boost::shared_ptr<amf::Element> details(new Element);
+	  boost::shared_ptr<cygnal::Element> details(new Element);
 	  details->makeString("details", filename);
 	  top.addProperty(details);
   
-	  boost::shared_ptr<amf::Element> cid(new Element);
+	  boost::shared_ptr<cygnal::Element> cid(new Element);
 	  cid->makeNumber("clientid", clientid);
 	  top.addProperty(cid);
 
@@ -771,15 +770,15 @@ RTMPServer::encodeResult(gnash::RTMPMsg::rtmp_status_e status, const std::string
       {
 	  str->makeString("onStatus");
 //	  "clientid"
-	  boost::shared_ptr<amf::Element> level(new Element);
+	  boost::shared_ptr<cygnal::Element> level(new Element);
 	  level->makeString("level", "status");
 	  top.addProperty(level);
 
-	  boost::shared_ptr<amf::Element> code(new Element);
+	  boost::shared_ptr<cygnal::Element> code(new Element);
 	  code->makeString("code", "NetStream.Play.Reset");
 	  top.addProperty(code);
 
-	  boost::shared_ptr<amf::Element> description(new Element);
+	  boost::shared_ptr<cygnal::Element> description(new Element);
 	  string field = "Playing and resetting ";
 	  if (!filename.empty()) {
 	      field += filename;
@@ -787,11 +786,11 @@ RTMPServer::encodeResult(gnash::RTMPMsg::rtmp_status_e status, const std::string
 	  description->makeString("description", field);
 	  top.addProperty(description);
 	  
-	  boost::shared_ptr<amf::Element> details(new Element);
+	  boost::shared_ptr<cygnal::Element> details(new Element);
 	  details->makeString("details", filename);
 	  top.addProperty(details);
 	  
-	  boost::shared_ptr<amf::Element> cid(new Element);
+	  boost::shared_ptr<cygnal::Element> cid(new Element);
 #ifdef CLIENT_ID_NUMERIC
 	  double clientid = createClientID();
 	  cid->makeNumber("clientid", clientid);
@@ -813,15 +812,15 @@ RTMPServer::encodeResult(gnash::RTMPMsg::rtmp_status_e status, const std::string
       {
 	  str->makeString("onStatus");
 
-	  boost::shared_ptr<amf::Element> level(new Element);
+	  boost::shared_ptr<cygnal::Element> level(new Element);
 	  level->makeString("level", "status");
 	  top.addProperty(level);
 
-	  boost::shared_ptr<amf::Element> code(new Element);
+	  boost::shared_ptr<cygnal::Element> code(new Element);
 	  code->makeString("code", "NetStream.Play.Start");
 	  top.addProperty(code);
 
-	  boost::shared_ptr<amf::Element> description(new Element);
+	  boost::shared_ptr<cygnal::Element> description(new Element);
 	  string field = "Started playing ";
 	  if (!filename.empty()) {
 	      field += filename;
@@ -829,11 +828,11 @@ RTMPServer::encodeResult(gnash::RTMPMsg::rtmp_status_e status, const std::string
 	  description->makeString("description", field);
 	  top.addProperty(description);
 	  
-	  boost::shared_ptr<amf::Element> details(new Element);
+	  boost::shared_ptr<cygnal::Element> details(new Element);
 	  details->makeString("details", filename);
 	  top.addProperty(details);
   
-	  boost::shared_ptr<amf::Element> cid(new Element);
+	  boost::shared_ptr<cygnal::Element> cid(new Element);
 #ifdef CLIENT_ID_NUMERIC
 	  double clientid = createClientID();
 	  cid->makeNumber("clientid", clientid);
@@ -854,15 +853,15 @@ RTMPServer::encodeResult(gnash::RTMPMsg::rtmp_status_e status, const std::string
       case RTMPMsg::NS_PLAY_STOP:
       case RTMPMsg::NS_PLAY_STREAMNOTFOUND:
       {
-	  boost::shared_ptr<amf::Element> level(new Element);
+	  boost::shared_ptr<cygnal::Element> level(new Element);
 	  level->makeString("level", "error");
 	  top.addProperty(level);
 
-	  boost::shared_ptr<amf::Element> description(new Element);
+	  boost::shared_ptr<cygnal::Element> description(new Element);
 	  description->makeString("description", "NetStream.Play.StreamNotFound.");
 	  top.addProperty(description);
 	  
-	  boost::shared_ptr<amf::Element> code(new Element);
+	  boost::shared_ptr<cygnal::Element> code(new Element);
 	  code->makeString("code", "NetStream.Play.StreamNotFound");
 	  top.addProperty(code);
 	  break;
@@ -898,7 +897,7 @@ RTMPServer::encodeResult(gnash::RTMPMsg::rtmp_status_e status, const std::string
 	  // Don't encode as an object, just the properties
 	  notobject = true;
 
-	  boost::shared_ptr<amf::Element> id2(new Element);
+	  boost::shared_ptr<cygnal::Element> id2(new Element);
 	  
 	  double sid = createStreamID();
 	  id2->makeNumber(sid);
@@ -912,11 +911,11 @@ RTMPServer::encodeResult(gnash::RTMPMsg::rtmp_status_e status, const std::string
 	  break;
     };
     
-    boost::shared_ptr<amf::Buffer> strbuf = str->encode();
-    boost::shared_ptr<amf::Buffer> numbuf = number->encode();
-    boost::shared_ptr<amf::Buffer> topbuf = top.encode(notobject);
+    boost::shared_ptr<cygnal::Buffer> strbuf = str->encode();
+    boost::shared_ptr<cygnal::Buffer> numbuf = number->encode();
+    boost::shared_ptr<cygnal::Buffer> topbuf = top.encode(notobject);
 
-    boost::shared_ptr<amf::Buffer> buf(new Buffer(strbuf->size() + numbuf->size() + topbuf->size()));
+    boost::shared_ptr<cygnal::Buffer> buf(new Buffer(strbuf->size() + numbuf->size() + topbuf->size()));
     *buf += strbuf;
     *buf += numbuf;
     boost::uint8_t byte = static_cast<boost::uint8_t>(RTMP::WINDOW_SIZE & 0x000000ff);
@@ -961,7 +960,7 @@ RTMPServer::encodePing(rtmp_ping_e type, boost::uint32_t milliseconds)
 //    GNASH_REPORT_FUNCTION;
 
     // An encoded ping message 
-    boost::shared_ptr<amf::Buffer> buf(new Buffer(sizeof(boost::uint16_t) * 3));
+    boost::shared_ptr<cygnal::Buffer> buf(new Buffer(sizeof(boost::uint16_t) * 3));
 //    boost::uint8_t *ptr = buf->reference();
 
     // Set the type of this ping message
@@ -1017,7 +1016,7 @@ RTMPServer::encodePing(rtmp_ping_e type, boost::uint32_t milliseconds)
 }
 
 // Encode a onBWDone message for the client. These are of a fixed size.
-boost::shared_ptr<amf::Buffer>
+boost::shared_ptr<cygnal::Buffer>
 RTMPServer::encodeBWDone(double id)
 {
 //    GNASH_REPORT_FUNCTION;
@@ -1032,11 +1031,11 @@ RTMPServer::encodeBWDone(double id)
     Element null;
     null.makeNull();
 
-    boost::shared_ptr<amf::Buffer> enccmd  = cmd.encode();
-    boost::shared_ptr<amf::Buffer> encnum  = num.encode();
-    boost::shared_ptr<amf::Buffer> encnull  = null.encode();
+    boost::shared_ptr<cygnal::Buffer> enccmd  = cmd.encode();
+    boost::shared_ptr<cygnal::Buffer> encnum  = num.encode();
+    boost::shared_ptr<cygnal::Buffer> encnull  = null.encode();
 
-    boost::shared_ptr<amf::Buffer> buf(new amf::Buffer(enccmd->size()
+    boost::shared_ptr<cygnal::Buffer> buf(new cygnal::Buffer(enccmd->size()
 						       + encnum->size()
 						       + encnull->size()));
 
@@ -1047,16 +1046,16 @@ RTMPServer::encodeBWDone(double id)
     return buf;
 }
 
-boost::shared_ptr<amf::Buffer>
+boost::shared_ptr<cygnal::Buffer>
 RTMPServer::encodeAudio(boost::uint8_t *data, size_t size)
 {
     GNASH_REPORT_FUNCTION;
     
-    boost::shared_ptr<amf::Buffer> buf;
+    boost::shared_ptr<cygnal::Buffer> buf;
     
     if (size) {
 	if (data) {
-	    buf.reset(new amf::Buffer(size));
+	    buf.reset(new cygnal::Buffer(size));
 	    buf->copy(data, size);
 	}
     }
@@ -1064,7 +1063,7 @@ RTMPServer::encodeAudio(boost::uint8_t *data, size_t size)
     return buf;
 }
 
-boost::shared_ptr<amf::Buffer>
+boost::shared_ptr<cygnal::Buffer>
 RTMPServer::encodeVideo(boost::uint8_t *data, size_t size)
 {
     GNASH_REPORT_FUNCTION;
@@ -1073,30 +1072,30 @@ RTMPServer::encodeVideo(boost::uint8_t *data, size_t size)
 #if 0
 // Parse an Echo Request message coming from the Red5 echo_test. This
 // method should only be used for testing purposes.
-vector<boost::shared_ptr<amf::Element > >
+vector<boost::shared_ptr<cygnal::Element > >
 RTMPServer::parseEchoRequest(boost::uint8_t *ptr, size_t size)
 {
 //    GNASH_REPORT_FUNCTION;
     AMF amf;
-    vector<boost::shared_ptr<amf::Element > > headers;
+    vector<boost::shared_ptr<cygnal::Element > > headers;
 
     // The first element is the name of the test, 'echo'
-    boost::shared_ptr<amf::Element> el1 = amf.extractAMF(ptr, ptr+size);
+    boost::shared_ptr<cygnal::Element> el1 = amf.extractAMF(ptr, ptr+size);
     ptr += amf.totalsize();
     headers.push_back(el1);
 
     // The second element is the number of the test,
-    boost::shared_ptr<amf::Element> el2 = amf.extractAMF(ptr, ptr+size);
+    boost::shared_ptr<cygnal::Element> el2 = amf.extractAMF(ptr, ptr+size);
     ptr += amf.totalsize();
     headers.push_back(el2);
 
     // This one has always been a NULL object from my tests
-    boost::shared_ptr<amf::Element> el3 = amf.extractAMF(ptr, ptr+size);
+    boost::shared_ptr<cygnal::Element> el3 = amf.extractAMF(ptr, ptr+size);
     ptr += amf.totalsize();
     headers.push_back(el3);
 
     // This one has always been an NULL or Undefined object from my tests
-    boost::shared_ptr<amf::Element> el4 = amf.extractAMF(ptr, ptr+size);
+    boost::shared_ptr<cygnal::Element> el4 = amf.extractAMF(ptr, ptr+size);
     if (!el4) {
 	log_error("Couldn't reliably extract the echo data!");
     }
@@ -1110,22 +1109,22 @@ RTMPServer::parseEchoRequest(boost::uint8_t *ptr, size_t size)
 // is only used for testing by developers. The format appears to be
 // a string '_result', followed by the number of the test, and then two
 // NULL objects.
-boost::shared_ptr<amf::Buffer>
-RTMPServer::formatEchoResponse(double num, amf::Element &el)
+boost::shared_ptr<cygnal::Buffer>
+RTMPServer::formatEchoResponse(double num, cygnal::Element &el)
 {
 //    GNASH_REPORT_FUNCTION;
-    boost::shared_ptr<amf::Buffer> data = amf::AMF::encodeElement(el);
+    boost::shared_ptr<cygnal::Buffer> data = amf::AMF::encodeElement(el);
     return formatEchoResponse(num, data->reference(), data->allocated());
 }
 
-boost::shared_ptr<amf::Buffer>
-RTMPServer::formatEchoResponse(double num, amf::Buffer &data)
+boost::shared_ptr<cygnal::Buffer>
+RTMPServer::formatEchoResponse(double num, cygnal::Buffer &data)
 {
 //    GNASH_REPORT_FUNCTION;
     return formatEchoResponse(num, data.reference(), data.allocated());
 }
 
-boost::shared_ptr<amf::Buffer>
+boost::shared_ptr<cygnal::Buffer>
 RTMPServer::formatEchoResponse(double num, boost::uint8_t *data, size_t size)
 {
 //    GNASH_REPORT_FUNCTION;
@@ -1140,11 +1139,11 @@ RTMPServer::formatEchoResponse(double num, boost::uint8_t *data, size_t size)
     Element null;
     null.makeNull();
 
-    boost::shared_ptr<amf::Buffer> encecho = echo.encode();
-    boost::shared_ptr<amf::Buffer> encidx  = index.encode();   
-    boost::shared_ptr<amf::Buffer> encnull  = null.encode();   
+    boost::shared_ptr<cygnal::Buffer> encecho = echo.encode();
+    boost::shared_ptr<cygnal::Buffer> encidx  = index.encode();   
+    boost::shared_ptr<cygnal::Buffer> encnull  = null.encode();   
 
-    boost::shared_ptr<amf::Buffer> buf(new amf::Buffer(encecho->size()
+    boost::shared_ptr<cygnal::Buffer> buf(new cygnal::Buffer(encecho->size()
 						       + encidx->size()
 						       + encnull->size() + size));
 
@@ -1316,7 +1315,7 @@ RTMPServer::sendFile(int fd, const std::string &filespec)
 }
 
 size_t
-RTMPServer::sendToClient(std::vector<int> &fds, amf::Buffer &data)
+RTMPServer::sendToClient(std::vector<int> &fds, cygnal::Buffer &data)
 {
 //    GNASH_REPORT_FUNCTION;
     return sendToClient(fds, data.reference(), data.allocated());
@@ -1365,10 +1364,10 @@ rtmp_handler(Network::thread_params_t *args)
     // Adjust the timeout
     rtmp->setTimeout(10);
     
-    boost::shared_ptr<amf::Buffer>  pkt;
-    boost::shared_ptr<amf::Element> tcurl;
-    boost::shared_ptr<amf::Element> swfurl;
-    boost::shared_ptr<amf::Buffer> response;
+    boost::shared_ptr<cygnal::Buffer>  pkt;
+    boost::shared_ptr<cygnal::Element> tcurl;
+    boost::shared_ptr<cygnal::Element> swfurl;
+    boost::shared_ptr<cygnal::Buffer> response;
 
     // Keep track of the network statistics
     // See if we have any messages waiting. After the initial connect, this is
@@ -1376,7 +1375,7 @@ rtmp_handler(Network::thread_params_t *args)
     
     // Adjust the timeout
     rtmp->setTimeout(30);
-//    boost::shared_ptr<amf::Buffer> buf;
+//    boost::shared_ptr<cygnal::Buffer> buf;
 
     // If we have active disk streams, send those packets first.
     // 0 is a reserved stream, so we start with 1, as the reserved
@@ -1419,7 +1418,7 @@ rtmp_handler(Network::thread_params_t *args)
 		}
 		boost::shared_ptr<RTMP::rtmp_head_t> qhead;
 		for (size_t i=0; i<que->size(); i++) {
-		    boost::shared_ptr<amf::Buffer> bufptr = que->at(i)->pop();
+		    boost::shared_ptr<cygnal::Buffer> bufptr = que->at(i)->pop();
 		    // que->at(i)->dump();
 		    if (bufptr) {
 			// bufptr->dump();
@@ -1539,7 +1538,7 @@ rtmp_handler(Network::thread_params_t *args)
 			  } else if (body->getMethodName() == "play") {
 			      string filespec;
 			      boost::shared_ptr<gnash::RTMPMsg> nc = rtmp->getNetConnection();
-			      boost::shared_ptr<amf::Element> tcurl = nc->findProperty("tcUrl");
+			      boost::shared_ptr<cygnal::Element> tcurl = nc->findProperty("tcUrl");
 			      URL url(tcurl->to_string());
 			      filespec += url.hostname() + url.path();
 			      filespec += '/';
@@ -1641,7 +1640,7 @@ rtmp_handler(Network::thread_params_t *args)
 			      log_error("Received an _error message from the client!");
 			  } else {
 			      /* size_t ret = */ hand->writeToPlugin(tmpptr, qhead->bodysize);
-			      boost::shared_ptr<amf::Buffer> result = hand->readFromPlugin();
+			      boost::shared_ptr<cygnal::Buffer> result = hand->readFromPlugin();
 			      if (result) {
 				  if (rtmp->sendMsg(args->netfd, qhead->channel,
 						    RTMP::HEADER_8, result->allocated(),
@@ -1666,7 +1665,7 @@ rtmp_handler(Network::thread_params_t *args)
 
 		    // size_t ret = hand->writeToPlugin(tmpptr, qhead->bodysize);
 #if 0
-		    boost::shared_ptr<amf::Buffer> result = hand->readFromPlugin();
+		    boost::shared_ptr<cygnal::Buffer> result = hand->readFromPlugin();
  		    if (result) { // FIXME: this needs a real channel number
 			if (rtmp->sendMsg(args->netfd, 0x3, RTMP::HEADER_8, ret,
 					  RTMP::INVOKE, RTMPMsg::FROM_SERVER, *result)) {
@@ -1685,7 +1684,7 @@ rtmp_handler(Network::thread_params_t *args)
 		log_network("Never read any data from fd #%d", args->netfd);
 #if 0
 		// Send a ping to reset the new stream
-		boost::shared_ptr<amf::Buffer> ping_reset =
+		boost::shared_ptr<cygnal::Buffer> ping_reset =
 		    rtmp->encodePing(RTMP::PING_CLEAR, 0);
 		if (rtmp->sendMsg(args->netfd, RTMP_SYSTEM_CHANNEL,
 			  RTMP::HEADER_12, ping_reset->size(),
