@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,34 +16,18 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-#ifndef __DEJAGNU_PLUGIN_H__
-#define __DEJAGNU_PLUGIN_H__
+#ifndef GNASH_DEJAGNU_PLUGIN_H
+#define GNASH_DEJAGNU_PLUGIN_H
 
 #ifdef HAVE_CONFIG_H
 #include "gnashconfig.h"
 #endif
 
-#include <memory> // for auto_ptr
 #include "as_object.h"
 
 namespace gnash
 {
 
-class DejaGnu {
-public:
-    DejaGnu();
-    ~DejaGnu();
-    const char *pass (const char *msg);
-    const char *fail (const char *msg);
-    const char *xpass (const char *msg);
-    const char *xfail (const char *msg);
-    void totals ();
-private:
-    int passed;
-    int failed;
-    int xpassed;
-    int xfailed;
-};
 
 extern "C" {
     void dejagnu_class_init(as_object &obj);  
