@@ -1,5 +1,6 @@
 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,21 +23,26 @@
 #endif
 
 extern "C"{
-#ifdef HAVE_GETOPT_H
-	#include <getopt.h>
-#endif
+# ifdef HAVE_GETOPT_H
+#  include <getopt.h>
+# endif
+# ifndef __GNUC__
+  extern int getopt(int, char *const *, const char *);
+# endif
 }
 
 #include "gnash.h"
-#include "gui.h"
 #include "aquasup.h"
 #include "log.h"
 #include "movie_root.h"
+#include "RunResources.h"
 
-#include "render_handler.h"
+#include "Renderer.h"
 
 #include <Carbon/Carbon.h>
 
+using namespace std;
+
 namespace gnash {
 
 /* Main window widget */
@@ -57,8 +63,8 @@ void DoAdvanceMovie ( EventLoopTimerRef inTimer, void* data)
 }
   
 	
-AquaGui::AquaGui(unsigned long xid, float scale, bool loop, unsigned int depth)
-	: Gui(xid, scale, loop, depth),
+AquaGui::AquaGui(unsigned long xid, float scale, bool loop, RunResources& r)
+	: Gui(xid, scale, loop, r),
           _advance_timer(NULL)
 {
 }
@@ -118,10 +124,10 @@ bool AquaGui::init(int argc, char **argv[]) /* Self-explainatory */
       	
   	_glue.init(argc, argv);
 
-    _renderer = _glue.createRenderHandler();
+    _renderer.reset(_glue.createRenderHandler());
     if(!_renderer)return false;
 
-    set_render_handler(_renderer);
+	_runResources.setRenderer(boost::shared_ptr<Renderer>(_renderer));
     return true;
 
 }
@@ -151,7 +157,8 @@ void AquaGui::setCursor(gnash_cursor_type newcursor)
 }
 
 
-bool AquaGui::createWindow(const char* title, int width, int height)
+bool AquaGui::createWindow(const char* title, int width, int height,
+                           int xPosition, int yPosition)
 {
 	CFStringRef	windowTitle = NULL;
 	OSStatus	result;
@@ -199,7 +206,7 @@ bool AquaGui::createMenu()
 	GetIndMenuItemWithCommandID(NULL, kHICommandPreferences, 1, &rApplicationMenu, outIndex);
 
 	/* Enable 'About' */
-	InsertMenuItemTextWithCFString(rApplicationMenu, CFSTR("About gnash"), (short) 0, 0, kHICommandAbout);
+	InsertMenuItemTextWithCFString(rApplicationMenu, CFSTR("About Gnash"), (short) 0, 0, kHICommandAbout);
 
 	return true;
 }
