@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,9 +20,8 @@
 #ifndef AQUA_OGL_GLUE_H
 #define AQUA_OGL_GLUE_H
 
+#include "Renderer_ogl.h"
 #include "aqua_glue.h"
-#include <AGL/agl.h>
-
 
 namespace gnash
 {
@@ -32,8 +32,8 @@ class AquaOglGlue : public AquaGlue
     AquaOglGlue();
     virtual ~AquaOglGlue();
 
-    bool init(int argc, char ***argv);
-    render_handler* createRenderHandler();
+    bool init(int argc, char **argv[]);
+    Renderer* createRenderHandler();
     bool prepDrawingArea(int width, int height, AGLDrawable drawable);
     void setInvalidatedRegions(const InvalidatedRanges& /* ranges */);
     void render();
