@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,16 +23,24 @@
 #include "gnashconfig.h"
 #endif
 
-#include "rect.h"  // for composition
+#include "SWFRect.h"  // for composition
 #include "snappingrange.h"  // for InvalidatedRanges
 #include "GnashKey.h" // for gnash::key::code type
-#include "smart_ptr.h"
+#include "Renderer.h" // for gnash::key::code type
+#include "VirtualClock.h"
+#include "SystemClock.h"
+#include "gnash.h" // for Quality
+#include "movie_root.h"
 
 #ifdef USE_SWFTREE
 #include "tree.hh" // for tree
 #endif
 
-#include <cstdlib> // exit
+#include <boost/intrusive_ptr.hpp>
+#include <boost/scoped_ptr.hpp>
+#include <boost/function.hpp>
+#include <vector>
+#include <cstdlib> 
 #include <string>
 #include <map>
 
@@ -39,12 +48,6 @@
 // gnashconfig.h
 //#define GNASH_FPS_DEBUG
 
-// Define the following macro if you want to skip rendering
-// when late on FPS time.
-// This is an experimental feature, so it's off by default
-//#define SKIP_RENDERING_IF_LATE
-
-
 /// Define this to disable region updates debugging altogether. If undefined,
 /// debugging will be a runtime option. The flag and flag-setting functions
 /// will not be disabled (too ugly).
@@ -64,14 +67,12 @@
 // Forward declarations
 namespace gnash
 {
-    class render_handler;
+    class RunResources;
     class movie_root;
     class movie_definition;
 }
 
-namespace gnash
-{
-
+namespace gnash {
 
 /// Enumerates mouse cursor types.
 enum gnash_cursor_type {
@@ -80,6 +81,72 @@ enum gnash_cursor_type {
   CURSOR_INPUT
 };
 
+
+/// Handles screen dumps.
+class ScreenShotter
+{
+public:
+
+    typedef std::vector<size_t> FrameList;
+
+    /// Create a ScreenShotter with renderer and output name.
+    ScreenShotter(boost::shared_ptr<Renderer> r, const std::string& fileName)
+        :
+        _renderer(r),
+        _immediate(false),
+        _fileName(fileName),
+        _last(false)
+    {}
+
+    /// Take a screenshot at the next possible moment.
+    void now() {
+        _immediate = true;
+    }
+
+    /// Take a screenshot when the last frame is reached.
+    void lastFrame() {
+        _last = true;
+    }
+
+    /// Called on the last frame before exit.
+    //
+    /// Which frame is last depends on the execution path of the SWF, whether
+    /// the SWF loops, whether a timeout was requested or a maximum number of
+    /// advances set. Those conditions are not knowable in advance, so
+    /// the last frame is a special case.
+    void last() const;
+
+    /// Takes a screenshot if required.
+    //
+    /// Called on each advance.
+    //
+    /// @param frameAdvance     used to check whether a screenshot is required
+    ///                         as well as to construct the filename.
+    void screenShot(size_t frameAdvance);
+
+    /// Request a list of frames to be rendered to image files.
+    void setFrames(const FrameList& frames);
+
+private:
+
+    /// Take the screenshot.
+    void saveImage(const std::string& filename) const;
+
+    boost::shared_ptr<Renderer> _renderer;
+
+    /// If true, the next call to screenshot will take a screenshot
+    bool _immediate;
+
+    /// Name used to generate output file.
+    const std::string _fileName;
+
+    /// Whether to take a screenshot on the last frame.
+    bool _last;
+
+    FrameList _frames;
+
+};
+
 /// Parent class from which all GUI implementations will depend.
 class Gui {
 
@@ -101,27 +168,57 @@ public:
       _interval = interval;
     }
 
+    /// Instructs the Gui to monitor a file descriptor.
+    //
+    /// @param fd The file descriptor to monitor
+    /// @param callback The callback to fire
+    void setFDCallback(int fd, boost::function<void ()> callback);
+
+    /// Calls the callback function registered for a file descriptor.
+    //
+    /// @param fd The file descriptor for which the registered callback should
+    ///        be fired.
+    void callCallback(int fd);
+
+    /// Return the clock provided by this Gui.
+    //
+    /// The Gui clock will be paused when the gui is put
+    /// in pause mode and resumed when gui playback is resumed.
+    ///
+    VirtualClock& getClock() { return _virtualClock; }
+
     /// Set the time in milliseconds after which the programme should exit.
     virtual void setTimeout(unsigned int timeout) = 0;
 
+    /// Request a list of screenshots
+    //
+    /// @param l        A list of frames to render to an image file
+    /// @param last     Whether to render the last frame before exist
+    /// @param filename The filename pattern to save images as.
+    void requestScreenShots(const ScreenShotter::FrameList& l, bool last,
+            const std::string& filename);
+
     /** \brief
      * Create and display our window.
      *
      * @param title The window title.
      * @param width The desired window width in pixels.
      * @param height The desired window height in pixels.
+     * @param xPosition The desired window X position from the top left corner.
+     * @param yPosition The desired window Y position from the top left corner.
      */   
-    virtual bool createWindow(const char* title, int width, int height) = 0;
+    virtual bool createWindow(const char* title, int width, int height,
+                       int xPosition = 0, int yPosition = 0) = 0;
+
+    virtual void resizeWindow(int width, int height);
 
     /// Start main rendering loop.
     virtual bool run() = 0;
 
-    /// End main rendering loop, making the call to run() return.
+    /// Always called on exit.
     //
-    /// The default implementation calls exit(0), which isn't nice.
-    /// Please implement the proper main loop quitter in the subclasses.
-    ///
-    virtual void quit()  { std::exit(0); }
+    /// Handles any common functions, then calls virtual quitUI().
+    void quit();
 
     /// Render the current buffer.
     /// For OpenGL, this means that the front and back buffers are swapped.
@@ -145,7 +242,7 @@ public:
     // does not need to be implemented (optional feature),
     // but still needs to be available.
     //
-    // Why "rect" (floats)? Because the gui does not really
+    // Why "SWFRect" (floats)? Because the gui does not really
     // know about the scale the renderer currently uses... 
     //
     // <strk> but it does not about the "semantic" of the TWIPS
@@ -153,7 +250,7 @@ public:
     //        The question really is: why floats for TWIPS ?
     //        (guess this goes deep in the core/server libs)
     //
-    virtual void setInvalidatedRegion(const rect& bounds);
+    virtual void setInvalidatedRegion(const SWFRect& bounds);
     virtual void setInvalidatedRegions(const InvalidatedRanges& ranges);
     
     // Called right before rendering anything (after setInvalidatedRegion).
@@ -190,18 +287,22 @@ public:
 
     /// Mouse notification callback to be called when the mouse is moved.
     //
-    /// @param x The mouse coordinate X component in user/window coordinate space (pixels).
-    /// @param y The mouse coordinate Y component in user/window coordinate space (pixels).
-    ///
-    void notify_mouse_moved(int x, int y);
+    /// @param x    The mouse coordinate X component in user/window
+    ///             coordinate space (pixels).
+    /// @param y    The mouse coordinate Y component in user/window
+    ///             coordinate space (pixels).
+    void notifyMouseMove(int x, int y);
 
     /// Mouse notification callback to be called when the mouse is clicked.
     //
     /// @param mouse_pressed Determines whether the mouse button is being
     ///                      pressed (true) or being released (false)
-    /// @param mask A binary representation of the buttons currently pressed.
-    ///
-    void notify_mouse_clicked(bool mouse_pressed, int mask);
+    void notifyMouseClick(bool mouse_pressed);
+
+    /// Send a mouse wheel event to the stage.
+    //
+    /// @param delta    A number expressing the extent of the wheel scroll.
+    void notifyMouseWheel(int delta);
 
     /// Key event notification to be called when a key is pressed or depressed
     //
@@ -218,7 +319,7 @@ public:
     /// @param height The desired height in pixels.
     void resize_view(int width, int height);
 
-    /// Update stage matrix accordingly to window size and flash Stage
+    /// Update stage SWFMatrix accordingly to window size and flash Stage
     /// configuration (scaleMode, alignment)
     //
     /// This method should be called from the core lib when Stage configuration
@@ -232,8 +333,7 @@ public:
     bool advanceMovie();
 
     /// Convenience static wrapper around advanceMovie for callbacks happiness.
-    static bool advance_movie(Gui* gui)
-    {
+    static bool advance_movie(Gui* gui) {
         return gui->advanceMovie();
     }
 
@@ -253,6 +353,10 @@ public:
     /// Return from fullscreen to normal mode.
     ///
     virtual void unsetFullscreen();
+
+    /// Hide the menu bar when using standalone player
+    ///
+    virtual void hideMenu();
     
     /// Sets whether the gui should show the system mouse pointer
     //
@@ -302,23 +406,27 @@ public:
     bool isStopped() const { return _stopped; }
     
     /// Whether gnash is is running as a plugin
-    bool isPlugin() const { return (( _xid )); }
+    bool isPlugin() const { return ((_xid)); }
+
+    /// Take a screenshot now!
+    void takeScreenShot();
 
     /// Set the maximum number of frame advances before Gnash exits.
     void setMaxAdvances(unsigned long ul) { if (ul) _maxAdvances = ul; }
     
     void showUpdatedRegions(bool x) { _showUpdatedRegions = x; }
-    bool showUpdatedRegions() { return _showUpdatedRegions; }
+    bool showUpdatedRegions() const { return _showUpdatedRegions; }
 
     /// Instruct the core to restart the movie and
     /// set state to play(). This does not change pause
     /// state.
     void restart();
 
-    void menu_step_forward();
-    void menu_step_backward();
-    void menu_jump_forward();
-    void menu_jump_backward();
+    /// Set rendering quality, if not locked by RC file..
+    void setQuality(Quality q);
+
+    /// Get current rendering quality
+    Quality getQuality() const;
 
     /// Toggle sound state between muted and unmuted. If
     /// there is no active sound handler this does nothing.
@@ -354,12 +462,24 @@ public:
     /// Add variables to set into instances of the top-level movie definition
     void addFlashVars(VariableMap& vars);
 
+    /// Add a variable used by ExternalInterface
+    void addScriptableVar(const std::string &name, const std::string &value);
+
     /// Set the definition of top-level movie
     void setMovieDefinition(movie_definition* md);
 
     /// Set the stage to advance/display
     void setStage(movie_root* stage);
 
+    /// The root movie, or "Stage"
+    movie_root *getStage() { return _stage; };
+    
+    /// Handle error message from the core
+    //
+    /// @param msg        The error message recieved
+    ///
+    virtual void error(const std::string& /*msg*/) {}
+
     /// Prompt user with a question she can answer with yes/no
     //
     /// @param question
@@ -372,10 +492,16 @@ public:
     ///
     virtual bool yesno(const std::string& question);
 
+    /// Width of a window pixel, in stage pseudopixel units.
+    float getXScale() { return _xscale; };
+
+    /// Height of a window pixel, in stage pseudopixel units.
+    float getYScale() { return _yscale; };
+
 protected:
 
     /// Default constructor. Initialises members to safe defaults.
-    Gui();
+    Gui(RunResources& r);
 
     /** \brief
      * Expanded constructor for more control over member values.
@@ -391,7 +517,31 @@ protected:
      *
      * @param depth Colour depth to be used in the client area of our window.
      */
-    Gui(unsigned long xid, float scale, bool loop, unsigned int depth);
+    Gui(unsigned long xid, float scale, bool loop, RunResources& r);
+    
+    /// End main rendering loop calling GUI-specific exit functions.
+    //
+    /// Do not call this directly. Call quit() instead.
+    //
+    /// The default implementation calls exit(EXIT_SUCCESS), which isn't nice.
+    /// Please implement the proper main loop quitter in the subclasses.
+    virtual void quitUI() {
+        std::exit(EXIT_SUCCESS);
+    }
+
+    /// Watch a file descriptor.
+    //
+    /// An implementing Gui should monitor the file descriptor in its main
+    /// loop. When the file descriptor is triggered, the implementation should
+    /// call callCallback().
+    ///
+    /// @param fd The file descriptor to be watched
+    virtual bool watchFD(int /* fd */)
+    {
+        log_unimpl("This GUI does not implement FD watching.");
+        return false;
+    }
+
 
     /// Determines if playback should restart after the movie ends.
     bool            _loop;
@@ -406,23 +556,23 @@ protected:
     geometry::Range2d<int> _validbounds;
 
     /// Desired window width.
-    int             _width;
+    int _width;
 
     /// Desired window height.
-    int             _height;
+    int _height;
 
     /// Desired colour depth in bits.
-    int             _depth;
+    RunResources& _runResources;
 
     /// Main loop interval: the time between successive advance_movie calls.
-    unsigned int    _interval;
+    unsigned int _interval;
 
     /// The handler which is called to update the client area of our window.
-    render_handler* _renderer;
+    boost::shared_ptr<Renderer> _renderer;
 
     /// Signals that the next frame must be re-rendered completely because the
     /// window size did change.
-    bool            _redraw_flag;
+    bool _redraw_flag;
 
     // True if Gnash is running in fullscreen
     bool _fullscreen;
@@ -444,23 +594,25 @@ protected:
     virtual void playHook() {}
 
 private:
+    std::map<int /* fd */, boost::function<void ()> > _fd_callbacks;
 
     /// Width of a window pixel, in stage pseudopixel units.
-    float           _xscale;
+    float _xscale;
 
     /// Height of a window pixel, in stage pseudopixel units.
-    float           _yscale;
+    float _yscale;
 
     /// Window pixel X offset of stage origin
-    boost::int32_t   _xoffset;
+    boost::int32_t _xoffset;
 
     /// Window pixel Y offset of stage origin
-    boost::int32_t   _yoffset;
+    boost::int32_t _yoffset;
 
     bool display(movie_root* m);
     
 #ifdef GNASH_FPS_DEBUG
     unsigned int fps_counter;
+
     float fps_rate_min, fps_rate_max;   
 
     // Number of calls to fpsCounterTick, which is also
@@ -475,6 +627,9 @@ private:
     ///
     float fps_timer_interval;
     
+    /// Number of frames rendering of which was dropped
+    unsigned int frames_dropped;
+
     /// \brief
     /// Should be called on every frame advance (including inter-frames caused
     /// by mouse events).
@@ -485,45 +640,49 @@ private:
 
 #endif // def GNASH_FPS_DEBUG
 
-#ifdef SKIP_RENDERING_IF_LATE
-    /// Estimated max number of milliseconds required for a call to ::display
-    /// This should be incremented everytime we take more
-    boost::uint32_t estimatedDisplayTime;
-#endif // SKIP_RENDERING_IF_LATE
-
     VariableMap _flashVars;
+    VariableMap _scriptableVars;
 
     boost::intrusive_ptr<movie_definition> _movieDef;
-
+    
+    /// The root movie, or "Stage"
     movie_root* _stage;
 
     /// True if the application has been put into "stop" mode
-    bool            _stopped;
+    bool _stopped;
 
     /// True if the application didn't start yet
-    bool            _started;
-    
+    bool _started;
+
     /// If true, updated regions (invalidated ranges) are visibly outlined.
     bool _showUpdatedRegions;
-    
-#ifdef ENABLE_KEYBOARD_MOUSE_MOVEMENTS 
-	int _xpointer;
-	int _ypointer;
-	bool _keyboardMouseMovements;
-	int _keyboardMouseMovementsStep;
-#endif // ENABLE_KEYBOARD_MOUSE_MOVEMENTS
 
+    SystemClock _systemClock;
+    InterruptableVirtualClock _virtualClock;
+    
+    /// Checked on each advance for screenshot activity if it exists.
+    boost::scoped_ptr<ScreenShotter> _screenShotter;
+
+#ifdef ENABLE_KEYBOARD_MOUSE_MOVEMENTS 
+    int _xpointer;
+    int _ypointer;
+    bool _keyboardMouseMovements;
+    int _keyboardMouseMovementsStep;
+#endif // ENABLE_KEYBOARD_MOUSE_MOVEMENTS
 };
 
 /// Named constructors
-std::auto_ptr<Gui> createGTKGui(unsigned long xid, float scale, bool loop, unsigned int depth);
-std::auto_ptr<Gui> createKDEGui(unsigned long xid, float scale, bool loop, unsigned int depth);
-std::auto_ptr<Gui> createSDLGui(unsigned long xid, float scale, bool loop, unsigned int depth);
-std::auto_ptr<Gui> createFLTKGui(unsigned long xid, float scale, bool loop, unsigned int depth);
-std::auto_ptr<Gui> createFBGui(unsigned long xid, float scale, bool loop, unsigned int depth);
-std::auto_ptr<Gui> createAQUAGui(unsigned long xid, float scale, bool loop, unsigned int depth);
-std::auto_ptr<Gui> createRISCOSGui(unsigned long xid, float scale, bool loop, unsigned int depth);
-std::auto_ptr<Gui> createDumpGui(unsigned long xid, float scale, bool loop, unsigned int depth);
+std::auto_ptr<Gui> createGTKGui(unsigned long xid, float scale, bool loop, RunResources& r);
+std::auto_ptr<Gui> createKDEGui(unsigned long xid, float scale, bool loop, RunResources& r);
+std::auto_ptr<Gui> createKDE4Gui(unsigned long xid, float scale, bool loop, RunResources& r);
+std::auto_ptr<Gui> createSDLGui(unsigned long xid, float scale, bool loop, RunResources& r);
+std::auto_ptr<Gui> createFLTKGui(unsigned long xid, float scale, bool loop, RunResources& r);
+std::auto_ptr<Gui> createFBGui(unsigned long xid, float scale, bool loop, RunResources& r);
+std::auto_ptr<Gui> createAQUAGui(unsigned long xid, float scale, bool loop, RunResources& r);
+std::auto_ptr<Gui> createRISCOSGui(unsigned long xid, float scale, bool loop, RunResources& r);
+std::auto_ptr<Gui> createAOS4Gui(unsigned long xid, float scale, bool loop, RunResources& r);
+std::auto_ptr<Gui> createHaikuGui(unsigned long xid, float scale, bool loop, RunResources& r);
+std::auto_ptr<Gui> createDumpGui(unsigned long xid, float scale, bool loop, RunResources& r);
 
  
 } // end of gnash namespace
@@ -533,5 +692,5 @@ std::auto_ptr<Gui> createDumpGui(unsigned long xid, float scale, bool loop, unsi
 
 // Local Variables:
 // mode: C++
-// indent-tabs-mode: t
+// indent-tabs-mode: nil
 // End:
