@@ -1,5 +1,6 @@
 //
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -28,7 +29,7 @@ class RiscosGlue
     virtual bool init(int argc, char **argv[]) = 0;
 
     virtual void prepFramebuffer(void *framebuffer, int width, int height) = 0;
-    virtual render_handler* createRenderHandler() = 0;
+    virtual Renderer* createRenderHandler() = 0;
     virtual void setRenderHandlerSize(int /*width*/, int /*height*/) { };
     virtual void render(int /* x */, int /* y */) = 0;
     virtual void render(int x, int y, int /*minx*/, int /*miny*/, int /*maxx*/, int /*maxy*/)
