@@ -1,6 +1,7 @@
 // WallClockTimer.cpp:  Wall clock timer, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,37 +18,32 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "WallClockTimer.h"
 #include "ClockTime.h" // for "portable" get_ticks (contains implementation for win32)
 
 namespace gnash {
 
 WallClockTimer::WallClockTimer()
-	:
-	startTimer(clocktime::getTicks())
+    :
+    startTimer(clocktime::getTicks())
 {
 }
 
 void
 WallClockTimer::restart()
 {
-	startTimer = clocktime::getTicks();
+    startTimer = clocktime::getTicks();
 }
 
 boost::uint32_t
 WallClockTimer::elapsed() const
 {
-	boost::uint64_t currTime = clocktime::getTicks();
-	
-	// be aware of time glitches
-	if ( currTime <= startTimer ) return 0;
+    boost::uint64_t currTime = clocktime::getTicks();
+    
+    // be aware of time glitches
+    if ( currTime <= startTimer ) return 0;
 
-	return currTime - startTimer;
+    return currTime - startTimer;
 
 }
 
