@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,6 +19,7 @@
 #ifndef GNASH_EXTENSION_H
 #define GNASH_EXTENSION_H
 
+#include <map>
 #include <vector>
 #include <string>
 #include <boost/tokenizer.hpp>
@@ -64,11 +66,11 @@ public:
     /// @param dir       A directory to scan.
     bool scanAndLoad(const std::string& dir, as_object &where);
 
-	// open a module, initialize the module within Gnash. Known function name.
-	bool initModuleWithFunc(const std::string& module, const std::string& func, as_object &obj);
+    // open a module, initialize the module within Gnash. Known function name.
+    bool initModuleWithFunc(const std::string& module, const std::string& func, as_object &obj);
     bool initNewObject(as_object &obj);
     void dumpModules();
-private:
+protected:
 
     /// Initialize the named module within Gnash
     //
