@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,7 +22,6 @@
 
 #include "dsodefs.h"
 
-#include <string>
 
 
 namespace gnash {
@@ -43,7 +43,7 @@ namespace noseek_fd_adapter
 /// Specify a cachefilename if you want to be able to access
 /// the full cache after deletion of the returned IOChannel.
 ///
-DSOEXPORT IOChannel* make_stream(int fd, const char* cachefilename=NULL);
+DSOEXPORT IOChannel* make_stream(int fd, const char* cachefilename = 0);
 
 } // namespace gnash::noseek_fd_adapter
 } // namespace gnash
