@@ -27,6 +27,7 @@
 
 #include <cstdlib>
 #include <cassert>
+#include <cstring>
 #include <string>
 #include <typeinfo>
 
@@ -60,6 +61,30 @@ namespace std
 };
 #endif
 
+#if defined(__HAIKU__)
+namespace std {
+	class wstring : public std::basic_string<char>
+	{
+	public:
+		wstring(const char *t)
+			: std::basic_string<char>(t)
+		{
+		}
+		wstring()
+		{
+		}
+		wstring(const wstring &that)
+			: std::basic_string<char>(that.c_str())
+		{
+		}
+		wstring(const std::basic_string<char> &that)
+			: std::basic_string<char>(that)
+		{
+		}
+	};
+};
+#endif
+
 namespace gnash {
 
 /// Return (unmangled) name of this instance type. Used for
