@@ -31,7 +31,6 @@
 #include "MovieClip.h"
 #include "movie_root.h"
 #include "VM.h"
-#include "builtin_function.h"
 #include "NativeFunction.h"
 #include "fn_call.h" 
 #include "ExecutableCode.h"
@@ -40,6 +39,8 @@
 #include "GnashKey.h" 
 #include "SoundInfoRecord.h" 
 #include "Global_as.h" 
+#include "RunResources.h"
+#include "sound_definition.h"
 
 #include <boost/bind.hpp>
 
@@ -224,16 +225,16 @@ private:
 namespace {
     void addInstanceProperty(Button& b, DisplayObject* d) {
         if (!d) return;
-        const std::string& name = d->get_name();
-        if (name.empty()) return;
+        const string_table::key name = d->get_name();
+        if (!name) return;
         getObject(&b)->init_member(name, getObject(d), 0);
     }
 
     void removeInstanceProperty(Button& b, DisplayObject* d) {
         if (!d) return;
-        const std::string& name = d->get_name();
-        if (name.empty()) return;
-        getObject(&b)->delProperty(getStringTable(*getObject(&b)).find(name));
+        const string_table::key name = d->get_name();
+        if (!name) return;
+        getObject(&b)->delProperty(name);
     }
 }
 
@@ -287,8 +288,6 @@ Button::Button(as_object* object, const SWF::DefineButtonTag* def,
         DisplayObject* parent)
     :
     InteractiveObject(object, parent),
-    _lastMouseFlags(FLAG_IDLE),
-    _mouseFlags(FLAG_IDLE),
     _mouseState(MOUSESTATE_UP),
     _def(def)
 {
@@ -561,7 +560,7 @@ Button::mouseEvent(const event_id& event)
     // check for built-in event handler.
     std::auto_ptr<ExecutableCode> code ( get_event_handler(event) );
     if (code.get()) {
-        mr.pushAction(code, movie_root::apDOACTION);
+        mr.pushAction(code, movie_root::PRIORITY_DOACTION);
     }
 
     callMethod(getObject(this), event.functionKey());
@@ -708,7 +707,7 @@ Button::set_current_state(MouseState new_state)
                 set_invalidated();
                 _stateCharacters[i] = ch;
                 addInstanceProperty(*this, ch);
-                ch->stagePlacementCallback(); 
+                ch->construct(); 
             }
         }
     }
@@ -738,7 +737,7 @@ Button::add_invalidated_bounds(InvalidatedRanges& ranges, bool force)
     getActiveCharacters(actChars);
     std::for_each(actChars.begin(), actChars.end(),
             boost::bind(&DisplayObject::add_invalidated_bounds, _1,
-                boost::ref(ranges), force||m_invalidated)
+                boost::ref(ranges), force || invalidated())
     );
 }
 
@@ -777,14 +776,16 @@ Button::pointInShape(boost::int32_t x, boost::int32_t y) const
 }
 
 void
-Button::stagePlacementCallback(as_object* initObj)
+Button::construct(as_object* initObj)
 {
-
-    // Not sure how this can happen, but blip.tv does it.
+    // This can happen if attachMovie is called with an exported Button and
+    // an init object. The attachment happens, but the init object is not used
+    // (see misc-ming.all/attachMovieTest.swf).
     if (initObj) {
-        log_unimpl("Button placed with an initObj. How did this happen? "
-                "We'll copy the properties anyway");
-        getObject(this)->copyProperties(*initObj);
+        IF_VERBOSE_ASCODING_ERRORS(
+            log_aserror("Button placed with an init object. This will "
+                "be ignored.");
+        );
     }
 
     saveOriginalTarget(); // for soft refs
@@ -824,20 +825,17 @@ Button::stagePlacementCallback(as_object* initObj)
 
         _stateCharacters[rno] = ch;
         addInstanceProperty(*this, ch);
-        ch->stagePlacementCallback(); // give this DisplayObject a life
+        ch->construct();
     }
 
     // There is no INITIALIZE/CONSTRUCT/LOAD/ENTERFRAME/UNLOAD event 
     // for Buttons
 }
 
-#ifdef GNASH_USE_GC
 void
 Button::markOwnResources() const
 {
 
-    _def->setReachable();
-
     // Mark state DisplayObjects as reachable
     for (DisplayObjects::const_iterator i = _stateCharacters.begin(),
             e = _stateCharacters.end(); i != e; ++i)
@@ -851,10 +849,9 @@ Button::markOwnResources() const
             std::mem_fun(&DisplayObject::setReachable));
 
 }
-#endif // GNASH_USE_GC
 
 bool
-Button::unload()
+Button::unloadChildren()
 {
 
     bool childsHaveUnload = false;
@@ -877,9 +874,7 @@ Button::unload()
     //       hit instance off the GC).
     _hitCharacters.clear();
 
-    bool hasUnloadEvent = DisplayObject::unload();
-
-    return hasUnloadEvent || childsHaveUnload;
+    return childsHaveUnload;
 }
 
 void
