@@ -26,6 +26,7 @@
 #include "smart_ptr.h" // GNASH_USE_GC
 #include "InteractiveObject.h" // for inheritance
 
+#include <boost/intrusive_ptr.hpp>
 #include <vector>
 #include <set>
 
@@ -113,22 +114,19 @@ public:
 	bool pointInShape(boost::int32_t x, boost::int32_t y) const;
 
 	bool isEnabled();
-	
-	/// Receive a stage placement notification
-	//
-	/// This callback will:
-	///
-	/// (1) Register this button instance as a live DisplayObject
-	/// (2) Setup the state DisplayObjects calling stagePlacementCallback
-    /// on all [WRONG]
-	virtual void stagePlacementCallback(as_object* initObj = 0);
-
-	/// Properly unload contained DisplayObjects
-	bool unload();
 
 	/// Properly destroy contained DisplayObjects
 	void destroy();
 
+    /// Do ActionScript construction of the Button.
+    //
+    /// (1) Register this button instance as a live DisplayObject
+    /// (2) Construct all button state DisplayObjects.
+    //
+    /// @param init     An init object, which can be passed when constructing
+    ///                 Buttons with attachMovie, but is never used.
+    virtual void construct(as_object* init = 0);
+
 #ifdef USE_SWFTREE
 	// Override to append button DisplayObjects info, see dox in DisplayObject.h
 	virtual InfoTree::iterator getMovieInfo(InfoTree& tr,
@@ -136,6 +134,9 @@ public:
 #endif
 
 protected:
+	
+	/// Properly unload contained DisplayObjects
+	virtual bool unloadChildren();
 
 #ifdef GNASH_USE_GC
 	/// Mark reachabe resources (for the GC)
@@ -150,16 +151,6 @@ protected:
 
 private:
 
-	int	_lastMouseFlags, _mouseFlags;
-
-	MouseState _mouseState;
-    
-    const boost::intrusive_ptr<const SWF::DefineButtonTag> _def;
-
-	DisplayObjects _stateCharacters;
-
-	DisplayObjects _hitCharacters;
-
 	/// Returns all DisplayObjects that are active based on the current state.
 	//
 	/// The "_visible" property does not matter here. 
@@ -193,6 +184,15 @@ private:
 
 	/// Return version of the SWF containing the button definition.
     virtual int getDefinitionVersion() const;
+	
+	MouseState _mouseState;
+    
+    const boost::intrusive_ptr<const SWF::DefineButtonTag> _def;
+
+	DisplayObjects _stateCharacters;
+
+	DisplayObjects _hitCharacters;
+
 
 };
 
