@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,19 +24,18 @@
 #include "gnashconfig.h" // USE_SWFTREE
 #endif
 
-#include "smart_ptr.h" // GNASH_USE_GC
 #include "event_id.h" // for inlines
-#include "as_object.h" // for inheritance
-#include "rect.h" // for composition (invalidated bounds)
+#include "SWFRect.h" // for composition (invalidated bounds)
 #include "SWFMatrix.h" // for composition
 #include "cxform.h" // for composition
-#include "log.h"
+#include "dsodefs.h" //for DSOEXPORT
 #include "snappingrange.h"
-#include "Range2d.h"
+#include "VM.h"
 #ifdef USE_SWFTREE
 # include "tree.hh"
 #endif
 
+#include <vector>
 #include <map>
 #include <string>
 #include <cassert>
@@ -54,6 +54,9 @@ namespace gnash {
     class StaticText;
     class InteractiveObject;
     class Renderer;
+    class as_object;
+    class as_value;
+    class as_environment;
     namespace SWF {
         class TextRecord;
     }
@@ -61,10 +64,68 @@ namespace gnash {
 
 namespace gnash {
 
+/// Returns true if the DisplayObject is referenceable in ActionScript
+//
+/// A DisplayObject is referenceable if it has an associated object.
+bool isReferenceable(const DisplayObject& d);
+
+/// Set special properties
+//
+/// This sets the magic properties of DisplayObjects.
+//
+/// @param key      The string table key of the property to set.
+/// @param obj      The DisplayObject whose property should be set
+/// @param val      An as_value representing the new value of the property.
+///                 Some values may be rejected.
+bool setDisplayObjectProperty(DisplayObject& obj, string_table::key key,
+        const as_value& val);
+
+/// Get special properties
+//
+/// This gets the magic properties of DisplayObjects and handles special
+/// MovieClip properties such as DisplayList members.
+//
+/// @param key      The string table key of the property to get.
+/// @param obj      The DisplayObject whose property should be got
+/// @param val      An as_value to be set to the value of the property.
+bool getDisplayObjectProperty(DisplayObject& obj, string_table::key key,
+        as_value& val);
+
+/// Get a property by its numeric index.
+//
+/// Used by ASHandlers to get the DisplayObject properties indexed by number
+//
+/// @param index    The index of the property to get.
+/// @param o        The DisplayObject whose property should be got
+/// @param val      An as_value to be set to the value of the property.
+void getIndexedProperty(size_t index, DisplayObject& o, as_value& val);
+
+/// Set a property by its numeric index.
+//
+/// Used by ASHandlers to set the DisplayObject properties indexed by number
+//
+/// @param index    The index of the property to set.
+/// @param o        The DisplayObject whose property should be set
+/// @param val      An as_value representing the new value of the property.
+///                 Some values may be rejected.
+void setIndexedProperty(size_t index, DisplayObject& o, const as_value& val);
+
+/// Copy SWFMatrix and caches from given DisplayObjecta
+//
+/// @param from     The DisplayObject to copy from
+/// @param to       The DisplayObject to copy to.
+void copyMatrix(const DisplayObject& from, DisplayObject& to);
+
 /// DisplayObject is the base class for all DisplayList objects.
 //
 /// It represents a single active element in a movie. This class does not
-/// provide any interactivity.
+/// supply any interactivity. The hierarchy of DisplayObjects in a movie
+/// provides all visual elements in a SWF. The DisplayObject hierarchy
+/// is independent of ActionScript resources, but can be controlled via AS.
+//
+/// DisplayObjects that can be controlled through ActionScript have an
+/// associated as_object. DisplayObjects such as Shape, do not have an
+/// associated object and cannot be referenced in AS.
 //
 /// Derived classes include InteractiveObject, StaticText, Bitmap,
 /// Video, and Shape.
@@ -81,11 +142,21 @@ namespace gnash {
 /// dynamic DisplayObjects, but tags are not always stored. They are not
 /// stored in most InteractiveObjects because most properties can be
 /// overridden during SWF execution.
-class DisplayObject : public as_object, boost::noncopyable
+class DisplayObject : public GcResource, boost::noncopyable
 {
 public:
 
-    DisplayObject(DisplayObject* parent, int id);
+    /// Construct a DisplayObject
+    //
+    /// @param mr       The movie_root containing the DisplayObject hierarchy.
+    ///                 All DisplayObjects may need movie_root resources.
+    /// @param object   An object to be associated with this DisplayObject.
+    ///                 If this is non-null, the DisplayObject will be
+    ///                 referenceable in ActionScript. Referenceable
+    ///                 DisplayObjects may access AS resources through their
+    ///                 associated object.
+    /// @param parent   The parent of the new DisplayObject. This may be null.
+    DisplayObject(movie_root& mr, as_object* object, DisplayObject* parent);
 
     virtual ~DisplayObject() {}
 
@@ -122,23 +193,11 @@ public:
     /// So, to recap:
     ///   1:  -32769 to -16385 are removed
     ///   2:  -16384 to      0 are statics
-    ///   3:  Max depth for a PlaceoObject call is 16384 (which becomes 
+    ///   3:  Max depth for a PlaceObject call is 16384 (which becomes 
     ///       0 in the statics)
     /// (all of the above correct?)
     static const int removedDepthOffset = -32769; 
 
-    /// Return true if the given depth is in the removed zone
-    static bool depthInRemovedZone(int depth)
-    {
-        return depth < staticDepthOffset;
-    }
-
-    /// Return true if this DisplayObject's depth is in the removed zone
-    bool depthInRemovedZone()
-    {
-        return depthInRemovedZone(get_depth());
-    }
-    
     /// This value is used for m_clip_depth when 
     /// the DisplayObject is not a layer mask.
     //
@@ -155,19 +214,26 @@ public:
         // MovieClip must override this
         // and any other DisplayObject will have
         // a parent!
-        assert(m_parent != NULL);
-        return m_parent->get_environment();
+        assert(_parent != NULL);
+        return _parent->get_environment();
     }
 
-    // Accessors for basic display info.
-    int get_id() const { return m_id; }
+    /// Enumerate any non-proper properties
+    //
+    /// This function is called by enumerateProperties(as_environment&) 
+    /// to allow for enumeration of properties that are derived from the
+    /// DisplayObject type, e.g. DisplayList members.
+    ///
+    /// The default implementation adds nothing
+    ///
+    virtual void enumerateNonProperties(as_environment&) const {}
 
     /// \brief
     /// Return the parent of this DisplayObject, or NULL if
     /// the DisplayObject has no parent.
     DisplayObject* get_parent() const
     {
-        return m_parent.get();
+        return _parent;
     }
 
     /// Set the parent of this DisplayObject
@@ -176,18 +242,20 @@ public:
     /// a parent. In AS2, this is only used for external movies
     void set_parent(DisplayObject* parent)
     {
-        m_parent = parent;
+        _parent = parent;
     }
 
-    int get_depth() const { return m_depth; }
+    virtual MovieClip* to_movie() { return 0; }
 
-    void  set_depth(int d) { m_depth = d; }
+    int get_depth() const { return _depth; }
+
+    void set_depth(int d) { _depth = d; }
 
     /// Get sound volume for this DisplayObject
     int getVolume() const { return _volume; }
 
     /// Set sound volume for this DisplayObject
-    void setVolume(int vol) { _volume=vol; }
+    void setVolume(int vol) { _volume = vol; }
 
     /// Get concatenated sound volume for this DisplayObject
     //
@@ -198,6 +266,11 @@ public:
     ///
     int getWorldVolume() const;
 
+    /// DisplayObjects can return the version of the SWF they were parsed from.
+    virtual int getDefinitionVersion() const {
+        return -1;
+    }
+
     /// Get local transform SWFMatrix for this DisplayObject
     const SWFMatrix& getMatrix() const { return m_matrix; }
 
@@ -208,7 +281,7 @@ public:
     /// @param updateCache if true, updates the cache values
     ///        from the SWFMatrix (only if SWFMatrix != current SWFMatrix)
     ///
-    void setMatrix(const SWFMatrix& m, bool updateCache=false);
+    void setMatrix(const SWFMatrix& m, bool updateCache = false);
 
     /// Set the xscale value of current SWFMatrix
     //
@@ -219,9 +292,6 @@ public:
     ///
     void set_x_scale(double factor);
 
-    /// Copy SWFMatrix and caches from given DisplayObject
-    void copyMatrix(const DisplayObject& ch);
-
     /// Set the yscale value of current SWFMatrix
     //
     /// This is used when setting _yscale 
@@ -245,24 +315,23 @@ public:
     /// Set the width of this DisplayObject, modifying its SWFMatrix
     //
     /// This is used when setting _width
-    /// See width_getset
     ///
-    /// @param w new width, in TWIPS. TODO: should this be an integer ?
-    ///
-    void set_width(double width);
+    /// @param w new width, in TWIPS. 
+    //
+    /// TextField does this differently (caches not updated).
+    virtual void setWidth(double width);
 
     /// Set the height of this DisplayObject, modifying its SWFMatrix
     //
     /// This is used when setting _height
-    /// See height_getset
     ///
-    /// @param h new height, in TWIPS. TODO: should this be an integer ?
+    /// @param h new height, in TWIPS. 
     ///
-    void set_height(double height);
+    virtual void setHeight(double height);
 
     const cxform& get_cxform() const { return m_color_transform; }
 
-    void  set_cxform(const cxform& cx) 
+    void set_cxform(const cxform& cx) 
     {       
         if (cx != m_color_transform) {
             set_invalidated(__FILE__, __LINE__);
@@ -270,23 +339,17 @@ public:
         }
     }
 
-    void concatenate_cxform(const cxform& cx) {
-        m_color_transform.concatenate(cx);
-    }
-
-    void concatenateMatrix(const SWFMatrix& m) { m_matrix.concatenate(m); }
-
-    int get_ratio() const { return m_ratio; }
+    int get_ratio() const { return _ratio; }
 
     void set_ratio(int r)
     {
-        if (r != m_ratio) set_invalidated(__FILE__, __LINE__); 
-        m_ratio = r;       
+        if (r != _ratio) set_invalidated(__FILE__, __LINE__); 
+        _ratio = r;       
     }
 
     /// Returns the clipping depth (if any) of this DisplayObject.
     /// The parameter tells us to use the DisplayObject as a mask for
-    /// all the objects contained in the display list from m_depth
+    /// all the objects contained in the display list from _depth
     /// to m_clipping_depth inclusive.
     /// 
     /// The value returned by get_clip_depth() is only valid when isMaskLayer()
@@ -326,18 +389,12 @@ public:
         return (_maskee);
     }
 
-    DisplayObject* toDisplayObject() { return this; }
-
     /// Return the DisplayObject masking this instance (if any)
     DisplayObject* getMask() const
     {
-        if ( ! _mask ) return NULL;
-        if ( _mask->_maskee != this )
-        {
-            // TODO: fix this !
-            log_error("Our mask maskee is not us");
-            return NULL; // for correctness;
-        }
+#if GNASH_PARANOIA_LEVEL > 1
+        if (_mask) assert(_mask->_maskee == this);
+#endif
         return _mask;
     }
 
@@ -350,20 +407,12 @@ public:
     ///
     void setMask(DisplayObject* mask);
 
-    /// Returns true if this DisplayObject is a mask (either layer or
-    /// dynamic mask)
-    bool isMask() const
-    {
-        return isDynamicMask() || isMaskLayer();
-    }
-
     /// Set DisplayObject name, initializing the original target member
-    void set_name(const std::string& name)
-    {
+    void set_name(string_table::key name) {
         _name = name;
     }
 
-    const std::string& get_name() const { return _name; }
+    string_table::key get_name() const { return _name; }
 
     /// \brief
     /// Get our concatenated SWFMatrix (all our ancestor transforms,
@@ -430,25 +479,7 @@ public:
         return 0;
     }
 
-    /// Returns local, untransformed height of this DisplayObject in TWIPS
-    //
-    /// Use getBounds() if you need more then simply the height.
-    ///
-    boost::int32_t get_height() const
-    {
-        return getBounds().height();
-    }
-
-    /// Returns local, untransformed width of this DisplayObject in TWIPS
-    //
-    /// Use getBounds() if you need more then simply the width.
-    ///
-    boost::int32_t get_width() const
-    {
-        return getBounds().width();
-    }
-
-	virtual rect getBounds() const = 0;
+	virtual SWFRect getBounds() const = 0;
 
     /// Return true if the given point falls in this DisplayObject's bounds
     //
@@ -458,7 +489,7 @@ public:
     ///                 This ignores _root's transform. 
     bool pointInBounds(boost::int32_t x, boost::int32_t y) const
     {
-        rect bounds = getBounds();
+        SWFRect bounds = getBounds();
         SWFMatrix wm = getWorldMatrix(false);
         wm.transform(bounds);
         return bounds.point_test(x, y);
@@ -485,8 +516,8 @@ public:
     ///
     virtual bool pointInVisibleShape(boost::int32_t x, boost::int32_t y) const
     {
-        if ( ! visible() ) return false;
-        if ( isMask() ) return false;
+        if (!visible()) return false;
+        if (isDynamicMask() || isMaskLayer()) return false;
         return pointInShape(x, y);
     }
 
@@ -529,22 +560,7 @@ public:
     /// In ActionScript 1.0, everything seems to be CASE
     /// INSENSITIVE.
     ///
-    virtual as_object* get_path_element(string_table::key key)
-    {
-        return getPathElementSeparator(key);
-    }
-
-    /// Advance this DisplayObject to next frame.
-    //
-    /// Character advancement is only meaningful for sprites
-    /// and sprite containers (button DisplayObjects) because
-    /// sprites are the only DisplayObjects that have frames.
-    /// 
-    /// Frame advancement include execution of all control tags.
-    /// 
-    virtual void advance()
-    {
-    }
+    virtual as_object* pathElement(string_table::key key);
 
     /// \brief
     /// Return true if PlaceObjects tag are allowed to move
@@ -608,20 +624,15 @@ public:
     // Return true if this DisplayObject should be rendered
     bool visible() const { return _visible; }
 
-    /// ActionScript event handler.    Returns true if a handler was called.
-    //
-    /// Must be overridden or will always return false.
-    ///
-    virtual bool on_event(const event_id& /* id */)
+    /// Notify clip events (and also user-defined ones).
+    virtual void notifyEvent(const event_id& /*id*/)
     {
-        return false;
     }
 
     /// Queue event in the global action queue.
     //
-    /// on_event(id) will be called by execution of the queued
+    /// notifyEvent(id) will be called by execution of the queued
     /// action
-    ///
     void queueEvent(const event_id& id, int lvl);
 
     /// Return true if an handler for the given event is defined
@@ -631,7 +642,6 @@ public:
     /// this in a non-virtual function. Main use for this method
     /// is for being called by ::unload() to verify an Unload handler
     /// is available.
-    ///
     bool hasEventHandler(const event_id& id) const;
 
 	/// DisplayObjects are not a mouse entity by default.
@@ -646,7 +656,6 @@ public:
     /// point and is not the DisplayObject being dragged or any of its childs.
     //
     /// Point coordinates in global twips.
-    ///
     virtual const DisplayObject* findDropTarget(boost::int32_t x, 
             boost::int32_t y, DisplayObject* dragging) const
     {
@@ -657,38 +666,21 @@ public:
         return 0;
     }
 
-    /// Returns true when the object (type) should get a instance name even 
-    /// if none is provided manually.
-    virtual bool wantsInstanceName() const
-    {
-        return false; 
+    /// Return whether this DisplayObject has been invalidated or not
+    bool invalidated() const {
+        return _invalidated;
     }
 
-    /// Returns true when the object (type) can be referenced by ActionScipt
-    bool isActionScriptReferenceable() const
-    {
-        // The way around
-        // [ wantsInstanceName() returning isActionScriptReferenceable() ]
-        // would be cleaner, but I wouldn't want to touch all files now.
-        return wantsInstanceName();
+    /// Return whether this DisplayObject has and invalidated child or not
+    bool childInvalidated() const {
+        return _child_invalidated;
     }
 
-    /// Returns the closest as-referenceable ancestor
-    DisplayObject* getClosestASReferenceableAncestor() 
-    {
-        if ( isActionScriptReferenceable() ) return this;
-        assert(m_parent);
-        return m_parent->getClosestASReferenceableAncestor();
+    /// Notify a change in the DisplayObject's appearance.
+    virtual void update() {
+        set_invalidated();
     }
 
-    const DisplayObject* getClosestASReferenceableAncestor() const
-    {
-        DisplayObject* nonconst_this = const_cast<DisplayObject*>(this);
-        return nonconst_this->getClosestASReferenceableAncestor();
-    }
-
-    /// @}
-
     /// \brief
     /// This function marks the DisplayObject as being modified in aspect
     /// and keeps track of current invalidated bounds the first time
@@ -706,7 +698,7 @@ public:
     /// clear_invalidated() is called in between.
     ///
     /// NOTE: Marking a DisplayObject as invalidated automatically marks
-    ///             it's parent as being invalidated.
+    ///             its parent as being invalidated.
     ///
     /// @see \ref region_update
     ///
@@ -723,9 +715,8 @@ public:
     /// Called by a child to signalize it has changed visibily. The
     /// difference to set_invalidated() is that *this* DisplayObject does
     /// not need to redraw itself completely. This function will 
-    /// recursively inform all it's parents of the change.
+    /// recursively inform all its parents of the change.
     void set_child_invalidated();
-    
 
     /// Clear invalidated flag and reset m_old_invalidated_bounds to null.
     ///
@@ -740,8 +731,8 @@ public:
     /// prevent the parent to be informed when this DisplayObject (or a
     /// child) is invalidated again (see set_invalidated() recursion).
     void clear_invalidated() {
-        m_invalidated = false;
-        m_child_invalidated = false;        
+        _invalidated = false;
+        _child_invalidated = false;        
         m_old_invalidated_ranges.setNull();
     }
     
@@ -761,7 +752,7 @@ public:
     ///
     /// It is used to determine what area needs to be re-rendered.
     /// The coordinates are world coordinates (in TWIPS).
-    /// Only instances with m_invalidated flag set are checked unless
+    /// Only instances with _invalidated flag set are checked unless
     /// force is set.
     ///
     virtual void add_invalidated_bounds(InvalidatedRanges& ranges, bool force);
@@ -781,7 +772,8 @@ public:
     /// If you override the method remember to call saveOriginalTarget()
     /// as the first thing.
     ///
-    virtual void stagePlacementCallback(as_object* = 0)
+    /// This handles all ActionScript construction and initialization events.
+    virtual void construct(as_object* /*init*/ = 0)
     {
         saveOriginalTarget();
     }
@@ -797,11 +789,13 @@ public:
     /// @return true if any onUnload event handler was defined
     ///                 by either this or any child DisplayObjects, false
     ///                 otherwise.
-    ///
-    virtual bool unload();
+    bool unload();
+
+    /// Accept a loaded Movie
+    virtual void getLoadedMovie(Movie* newMovie);
 
     /// Return true if this DisplayObject was unloaded from the stage
-    bool unloaded() const { return _unloaded; }
+    bool unloaded() const;
 
     /// Mark this DisplayObject as destroyed
     //
@@ -865,7 +859,7 @@ public:
     /// Return true if this DisplayObject allows turning the cursor
     /// into an hand shape when it happens to be the one receiving
     /// mouse events.
-    virtual bool allowHandCursor() const { return true; }
+    bool DSOEXPORT allowHandCursor() const;
 
 #ifdef USE_SWFTREE
     typedef std::pair<std::string, std::string> StringPair; 
@@ -886,7 +880,7 @@ public:
 #endif
     
     /// Used to assign a name to unnamed instances
-    static std::string getNextUnnamedInstanceName();
+    string_table::key getNextUnnamedInstanceName();
 
     enum BlendMode
     {
@@ -936,60 +930,43 @@ public:
     /// Default is a no-op. TextField implements this function.
     virtual void killFocus() {}
   
-    /// Getter-setter for _highquality.
-    static as_value highquality(const fn_call& fn);
-  
-    /// Getter-setter for _quality.
-    static as_value quality(const fn_call& fn);
-  
+    double rotation() const {
+        return _rotation;
+    }
+
+    double scaleX() const {
+        return _xscale;
+    }
+
+    double scaleY() const {
+        return _yscale;
+    }
+
+    as_object* object() const;
+
     /// Getter-setter for blendMode.
     static as_value blendMode(const fn_call& fn);
   
-    /// Getter-setter for _x
-    static as_value x_getset(const fn_call& fn);
-  
-    /// Getter-setter for _y
-    static as_value y_getset(const fn_call& fn);
-  
-    /// Getter-setter for _xscale
-    static as_value xscale_getset(const fn_call& fn);
-  
-    /// Getter-setter for _yscale
-    static as_value yscale_getset(const fn_call& fn);
-  
-    /// Getter-setter for _xmouse
-    static as_value xmouse_get(const fn_call& fn);
-  
-    /// Getter-setter for _ymouse
-    static as_value ymouse_get(const fn_call& fn);
-  
-    /// Getter-setter for _alpha
-    static as_value alpha_getset(const fn_call& fn);
-  
-    /// Getter-setter for _visible
-    static as_value visible_getset(const fn_call& fn);
-  
-    /// Getter-setter for _width
-    static as_value width_getset(const fn_call& fn);
-  
-    /// Getter-setter for _height
-    static as_value height_getset(const fn_call& fn);
-  
-    /// Getter-setter for _rotation
-    static as_value rotation_getset(const fn_call& fn);
-  
-    /// Getter-setter for _parent 
-    static as_value parent_getset(const fn_call& fn);
-  
-    /// Getter-setter for _target 
-    static as_value target_getset(const fn_call& fn);
-  
-    /// Getter-setter for _name
-    static as_value name_getset(const fn_call& fn);
-  
-    /// @} Common ActionScript getter-setters for DisplayObjects
-  
+    /// Mark all reachable resources.
+    //
+    /// Try not to override this function in derived classes. This always
+    /// marks the base class's resources and calls markOwnResources() to
+    /// take care of any further GC resources.
+    virtual void markReachableResources() const;
+
+    /// Called by markReachableResources()
+    //
+    /// DisplayObjects should mark their own resources in this function.
+    virtual void markOwnResources() const {}
+
 protected:
+    
+    virtual bool unloadChildren() { return false; }
+
+    /// Get the movie_root to which this DisplayObject belongs.
+    movie_root& stage() const {
+        return _stage;
+    }
 
     /// Register currently computable target as
     /// the "original" one. This will be used by
@@ -1002,65 +979,18 @@ protected:
         _origTarget=getTarget();
     }
 
-#ifdef GNASH_USE_GC
-    /// Mark all reachable resources, override from as_object.
-    //
-    /// The default implementation calls markDisplayObjectReachable().
-    ///
-    /// If a derived class provides access to more GC-managed
-    /// resources, it should override this method and call 
-    /// markDisplayObjectReachableResources() as the last step.
-    ///
-    virtual void markReachableResources() const
-    {
-        markDisplayObjectReachable();
-    }
-
-    /// Mark DisplayObject-specific reachable resources
-    //
-    /// These are: the DisplayObject's parent, mask, maskee and the default
-    ///                         as_object reachable stuff.
-    ///
-    void markDisplayObjectReachable() const;
-#endif // GNASH_USE_GC
 
     const Events& get_event_handlers() const
     {
         return _event_handlers;
     }
 
-    /// Return a user defined event handler, if any
-    //
-    /// @param name
-    ///     Function name to fetch. It will be converted to 
-    /// lowercase if current VM has been initialized against
-    /// an SWF version inferior to 7.
-    ///
-    /// @return
-    /// A function if a member with the given name exists and
-    /// casts to an as_function. A NULL pointer otherwise.
-    ///
-    boost::intrusive_ptr<as_function> getUserDefinedEventHandler(
-            const std::string& name) const;
-
-    /// Return a user defined event handler, if any
-    //
-    /// @param key
-    ///     Function key to fetch. 
-    ///
-    /// @return
-    /// A function if a member with the given key exists and
-    /// casts to an as_function. A NULL pointer otherwise.
-    ///
-    boost::intrusive_ptr<as_function> getUserDefinedEventHandler(
-            string_table::key key) const;
-
     void set_event_handlers(const Events& copyfrom);
 
     /// Name of this DisplayObject (if any)
-    std::string _name;
+    string_table::key _name;
 
-    boost::intrusive_ptr<DisplayObject> m_parent;
+    DisplayObject* _parent;
 
     /// look for '.', 'this',    '..', '_parent', '_level0' and '_root'
     //
@@ -1068,21 +998,6 @@ protected:
     ///
     as_object* getPathElementSeparator(string_table::key key);
 
-    /// \brief
-    /// Set when the visual aspect of this particular DisplayObject or movie
-    /// has been changed and redrawing is necessary.    
-    //
-    /// This is initialized to true as the initial state for
-    /// any DisplayObject is the "invisible" state (it wasn't there)
-    /// so it starts in invalidated mode.
-    ///
-    bool m_invalidated;
-
-    /// Just like m_invalidated but set when a child is invalidated instead
-    /// of this DisplayObject instance. m_invalidated and m_child_invalidated
-    /// can be set at the same time. 
-    bool m_child_invalidated;
-
     /// \brief
     /// Bounds of this DisplayObject instance before first invalidation
     /// since last call to clear_invalidated().
@@ -1105,20 +1020,26 @@ private:
     /// Register a DisplayObject masked by this instance
     void setMaskee(DisplayObject* maskee);
 
-    /// Build the _target member recursive on parent
-    std::string computeTargetPath() const;
+    /// The as_object to which this DisplayObject is attached.
+    as_object* _object;
 
-    int m_id;
+    /// The movie_root to which this DisplayObject belongs.
+    movie_root& _stage;
 
-    int m_depth;
     cxform m_color_transform;
+    
     SWFMatrix m_matrix;
+    
+    Events _event_handlers;
 
     /// Cache values for ActionScript access.
     /// NOTE: not all DisplayObjects need this, just the
     ///       ones which are ActionScript-referenceable
     double _xscale, _yscale, _rotation;
 
+    /// The depth of this DisplayObject.
+    boost::int32_t _depth;
+
     /// Volume control associated to this DisplayObject
     //
     /// This is used by Sound objects
@@ -1129,18 +1050,8 @@ private:
     ///
     int _volume;
 
-    int m_ratio;
+    int _ratio;
     int m_clip_depth;
-    Events _event_handlers;
-
-    /// Used to assign a name to unnamed instances
-    static unsigned int _lastUnnamedInstanceNum;
-
-    /// Set to yes when this instance has been unloaded
-    bool _unloaded;
-
-    /// This flag should be set to true by a call to destroy()
-    bool _destroyed;
 
     /// The DisplayObject masking this instance (if any)
     DisplayObject* _mask;
@@ -1166,8 +1077,47 @@ private:
 
     bool _dynamicallyCreated;
 
+    /// Set to yes when this instance has been unloaded
+    bool _unloaded;
+
+    /// This flag should be set to true by a call to destroy()
+    bool _destroyed;
+
+    /// \brief
+    /// Set when the visual aspect of this particular DisplayObject or movie
+    /// has been changed and redrawing is necessary.    
+    //
+    /// This is initialized to true as the initial state for
+    /// any DisplayObject is the "invisible" state (it wasn't there)
+    /// so it starts in invalidated mode.
+    ///
+    bool _invalidated;
+
+    /// Just like _invalidated but set when a child is invalidated instead
+    /// of this DisplayObject instance. _invalidated and _child_invalidated
+    /// can be set at the same time. 
+    bool _child_invalidated;
+
+
 };
 
+inline bool
+isReferenceable(const DisplayObject& d)
+{
+    return d.object();
+}
+
+/// Return the as_object associated with a DisplayObject if it exists
+//
+/// @param d    The DisplayObject to check. May be null.
+/// @return     null if either the DisplayObject or the associated object is
+///             null. Otherwise the associated object.
+inline as_object*
+getObject(const DisplayObject* d)
+{
+    return d ? d->object() : 0;
+}
+
 /// Stream operator for DisplayObject blend mode.
 std::ostream&
 operator<<(std::ostream& o, DisplayObject::BlendMode bm);
