@@ -22,8 +22,6 @@
 
 #include "PropFlags.h"
 #include "as_value.h"
-#include "string_table.h"
-#include "log.h"
 #include "ObjectURI.h"
 
 #include <boost/variant.hpp>
@@ -113,22 +111,6 @@ public:
 		return undefVal;
 	}
 
-	/// Set a user-defined getter
-	void setGetter(as_function* fun)
-	{
-		if (_getset.which() == 0) {
-			boost::get<UserDefinedGetterSetter>(_getset).setGetter(fun);
-		}
-	}
-
-	/// Set a user-defined setter
-	void setSetter(as_function* fun)
-	{
-		if (_getset.which() == 0) {
-			boost::get<UserDefinedGetterSetter>(_getset).setSetter(fun);
-		}
-	}
-
 	void markReachableResources() const
 	{
 		if (_getset.which() == 0) {
@@ -164,12 +146,6 @@ private:
 		/// Set the underlying value
 		void setUnderlying(const as_value& v) { _underlyingValue = v; }
 
-		/// Set the setter
-		void setSetter(as_function* setter) { _setter = setter; }
-
-		/// Set the getter
-		void setGetter(as_function* getter) { _getter = getter; }
-
 		void markReachableResources() const;
 
 	private:
@@ -251,6 +227,12 @@ private:
 };
 
 /// An abstract property
+//
+/// A Property is a holder for a value or a getter-setter.
+//
+/// Properties have special const semantics: the value of a Property does
+/// not affect its outward state, so the value of a const Property can be
+/// changed.
 class Property
 {
 public:
@@ -259,18 +241,7 @@ public:
         : 
 		_bound(as_value()),
         _destructive(false),
-        _uri(uri),
-		_orderID(0)
-	{}
-
-	/// Copy constructor
-	Property(const Property& p)
-        :
-		_flags(p._flags),
-        _bound(p._bound),
-        _destructive(p._destructive),
-        _uri(p._uri),
-        _orderID(p._orderID)
+        _uri(uri)
 	{}
 
 	Property(const ObjectURI& uri, const as_value& value,
@@ -279,8 +250,7 @@ public:
 		_flags(flags),
         _bound(value),
         _destructive(false),
-		_uri(uri),
-		_orderID(0)
+		_uri(uri)
 	{}
 
 	Property(const ObjectURI& uri,
@@ -290,8 +260,7 @@ public:
 		_flags(flags), 
         _bound(GetterSetter(getter, setter)),
 		_destructive(destroy),
-        _uri(uri),
-		_orderID(0)
+        _uri(uri)
 	{}
 
 	Property(const ObjectURI& uri, as_function *getter, as_function *setter,
@@ -300,8 +269,7 @@ public:
 		_flags(),
         _bound(GetterSetter(getter, setter)),
         _destructive(destroy),
-        _uri(uri),
-		_orderID(0)
+        _uri(uri)
 	{}
 
 	Property(const ObjectURI& uri, as_c_function_ptr getter,
@@ -311,19 +279,25 @@ public:
 		_flags(flags),
         _bound(GetterSetter(getter, setter)),
         _destructive(destroy),
-        _uri(uri),
-		_orderID(0)
+        _uri(uri)
+	{}
+	
+    /// Copy constructor
+	Property(const Property& p)
+        :
+		_flags(p._flags),
+        _bound(p._bound),
+        _destructive(p._destructive),
+        _uri(p._uri)
 	{}
-
-	/// Set a user-defined setter
-	void setSetter(as_function* fun);
-
-	/// Set a user-defined getter
-	void setGetter(as_function* fun);
 
 	/// accessor to the properties flags
 	const PropFlags& getFlags() const { return _flags; }
-	PropFlags& getFlags() { return _flags; }
+
+    /// Set the flags of the property
+    void setFlags(const PropFlags& flags) const {
+        _flags = flags;
+    }
 
 	/// Get value of this property
 	//
@@ -343,7 +317,6 @@ public:
 	/// to watch for infinitely recurse on calling the getter
 	/// or setter; Native getter-setter has no cache,
 	/// undefined will be returned for them.
-	///
 	const as_value& getCache() const;
 
 	/// Set internal cached value of this property
@@ -353,7 +326,6 @@ public:
 	/// to watch for infinitely recurse on calling the getter
 	/// or setter; Native getter-setter has no cache,
 	/// nothing would happen for them.
-	///
 	void setCache(const as_value& v);
 
 	/// Set value of this property
@@ -371,25 +343,7 @@ public:
 	///	The new value for this property. It will be used as first
 	///	argument of the 'setter' function if this is a Getter/Setter
 	///	property. @see isGetterSetter().
-	///
-	void setValue(as_object& this_ptr, const as_value &value);
-
-	/// Set the order id
-	//
-	/// NOTE: this field is used by one of the indexes
-	///       in the boost::multi_index used by PropertyList,
-	///       so changing this value on an instance which was
-	///       put in that index might result in corruption of
-	///       the index invariant. (at least this is what happens
-	///       with standard containers indexed on an element's member).
-	///
-	void setOrder(int order) { _orderID = order; }
-
-	/// Get the order id
-	int getOrder() const { return _orderID; }
-
-	/// is this a read-only member ?
-	bool isReadOnly() const { return _flags.get_read_only(); }
+	void setValue(as_object& this_ptr, const as_value &value) const;
 
 	/// Is this a getter/setter property?
 	bool isGetterSetter() const {
@@ -399,14 +353,6 @@ public:
 	/// is this a destructive property ?
 	bool isDestructive() const { return _destructive; }
 
-	/// Is this a static property?
-	bool isStatic() const { return _flags.get_static(); }
-
-	/// Is this member supposed to be visible by a VM of given version ?
-	bool visible(int swfVersion) const {
-        return _flags.get_visible(swfVersion);
-    }
-
 	/// Clear visibility flags
 	void clearVisible(int swfVersion) { _flags.clear_visible(swfVersion); }
 
@@ -419,9 +365,12 @@ public:
 	void setReachable() const;
 
 private:
+	
+    /// Get a value from a getter function.
+	as_value getDelayedValue(const as_object& this_ptr) const;
 
-	/// Properties flags
-	PropFlags _flags;
+	/// Set a value using a setter function.
+	void setDelayedValue(as_object& this_ptr, const as_value& value) const;
 
     enum Type {
         TYPE_EMPTY,
@@ -429,11 +378,13 @@ private:
         TYPE_GETTER_SETTER
     };
 
+	/// Properties flags
+	mutable PropFlags _flags;
+
 	// Store the various types of things that can be held.
 	typedef boost::variant<boost::blank, as_value, GetterSetter> BoundType;
 
-	// Changing this doesn't change the identity of the property, so it is
-	// mutable.
+    /// The value of the property.
 	mutable BoundType _bound;
 
 	// If true, as soon as getValue has been invoked once, the
@@ -444,17 +395,19 @@ private:
     // TODO: this should be const, but the assignment operator is still needed 
     ObjectURI _uri;
 
-	// An ordering number, for access by order
-	// (AS3 enumeration and slots, AS2 arrays)
-	int _orderID;
-
-	/// Get a value from a getter function.
-	as_value getDelayedValue(const as_object& this_ptr) const;
-
-	/// Set a value using a setter function.
-	void setDelayedValue(as_object& this_ptr, const as_value& value);
-
 };
+	
+/// is this a read-only member ?
+inline bool
+readOnly(const Property& prop) {
+    return prop.getFlags().get_read_only();
+}
+
+/// Is this member supposed to be visible by a VM of given version ?
+inline bool
+visible(const Property& prop, int version) {
+    return prop.getFlags().get_visible(version);
+}
 
 } // namespace gnash
 
