@@ -20,14 +20,23 @@
 #ifndef GNASH_RUN_INFO_H
 #define GNASH_RUN_INFO_H
 
-#include "TagLoadersTable.h"
-#include "StreamProvider.h"
-#include "Renderer.h"
-#include "sound_handler.h"
-
 #include <string>
 #include <boost/shared_ptr.hpp>
 
+namespace gnash {
+    class Renderer;
+    class StreamProvider;
+    namespace SWF {
+        class TagLoadersTable;
+    }
+    namespace sound {
+        class sound_handler;
+    }
+    namespace media {
+        class MediaHandler;
+    }
+}
+
 namespace gnash {
 
 /// Class to group together per-run and external resources for Gnash
@@ -39,7 +48,6 @@ namespace gnash {
 /// This must be kept alive for the entire duration of a run (presently
 /// until the last SWFMovieDefinition has been destroyed).
 /// @todo Check the lifetime and update documentation if it changes.
-/// @todo   Add MediaHandler.
 class RunResources
 {
 public:
@@ -51,8 +59,7 @@ public:
     RunResources(const std::string& baseURL)
         :
         _baseURL(baseURL)
-    {
-    }
+    {}
 
     /// Get the base URL for the run.
     //
@@ -62,8 +69,7 @@ public:
     /// Set the StreamProvider.
     //
     /// This can probably be changed during a run without ill effects.
-    void setStreamProvider(boost::shared_ptr<StreamProvider> sp)
-    {
+    void setStreamProvider(boost::shared_ptr<StreamProvider> sp) {
         _streamProvider = sp;
     }
 
@@ -97,6 +103,14 @@ public:
         return _soundHandler.get();
     }
 
+    void setMediaHandler(boost::shared_ptr<media::MediaHandler> s) {
+        _mediaHandler = s;
+    }
+
+    media::MediaHandler* mediaHandler() const {
+        return _mediaHandler.get();
+    }
+
     void setRenderer(boost::shared_ptr<Renderer> r) {
         _renderer = r;
     }
@@ -120,6 +134,20 @@ public:
         return *_tagLoaders;
     }
 
+#if 1
+    /// Set the renderer backend, agg, opengl, or cairo. This is set
+    /// in the users gnashrc file, or can be overridden with the
+    /// --hwaccel option to gnash.
+    void setRenderBackend(const std::string& x) { _renderer_backend = x; }
+    std::string& getRenderBackend() { return _renderer_backend; }
+
+    /// Set the hardware video accleration backend, none, vaapi, xv,
+    /// or cairo. This is set in the users gnashrc file, or can be
+    /// overridden with the --render-mode option to gnash.
+    std::string& getHWAccelBackend() { return _hwaccel_backend; }
+    void setHWAccelBackend(const std::string& x) { _hwaccel_backend = x; }
+#endif
+
 private:
 
     const std::string _baseURL;
@@ -127,13 +155,32 @@ private:
     boost::shared_ptr<StreamProvider> _streamProvider;
 
     boost::shared_ptr<sound::sound_handler> _soundHandler;
+    
+    boost::shared_ptr<media::MediaHandler> _mediaHandler;
 
     boost::shared_ptr<Renderer> _renderer;
 
     boost::shared_ptr<const SWF::TagLoadersTable> _tagLoaders;
 
+#if 1
+    /// Whether to ue HW video decoding support, no value means disabled.
+    /// The only currently supported values are: none, vaapi, or xv (omap)
+    /// support coming.  The default is none,
+    std::string _hwaccel_backend;
+
+    /// Which renderer backend to use, no value means use the default.
+    /// The currently supported values are agg, opengl, or cairo. AGG
+    /// being the default.
+    std::string _renderer_backend;
+#endif    
 };
 
-}
+} // end of gnash namespace
 
 #endif
+
+// local Variables:
+// mode: C++
+// indent-tabs-mode: t
+// End:
+
