@@ -23,17 +23,15 @@
 #include "gnashconfig.h"
 #endif
 
-#include "smart_ptr.h" // GNASH_USE_GC
 #include "string_table.h"
 #include "GC.h" // for inheritance from GcResource (to complete)
 #include "PropertyList.h"
-#include "as_value.h" 
-#include "smart_ptr.h"
 #include "PropFlags.h"
-#include "GnashException.h"
 #include "Relay.h"
 #include "ObjectURI.h"
 
+#include <map>
+#include <vector>
 #include <cmath>
 #include <utility> 
 #include <set>
@@ -44,10 +42,6 @@
 
 // Forward declarations
 namespace gnash {
-    namespace abc {
-        class Machine;
-        class Class;
-    }
     class as_function;
     class MovieClip;
     class DisplayObject;
@@ -57,6 +51,7 @@ namespace gnash {
     class movie_root;
     class RunResources;
     class Global_as;
+    class as_value;
 }
 
 namespace gnash {
@@ -198,11 +193,6 @@ public:
     ///                 uses the resources of the Global object.
     explicit as_object(Global_as& global);
 
-    /// Construct an ActionScript object with no prototype associated.
-    //
-    /// This constructor is deprecated!
-    as_object();
-
     /// Function dispatch
     //
     /// Various objects can be called, including functions and super objects.
@@ -267,11 +257,6 @@ public:
     virtual bool set_member(const ObjectURI& uri, const as_value& val,
         bool ifFound = false);
 
-    /// Reserve a slot
-    ///
-    /// Reserves a slot for a property to follow.
-    void reserveSlot(const ObjectURI& uri, boost::uint16_t slotId);
-
     /// Initialize a member value by string
     //
     /// This is just a wrapper around the other init_member method
@@ -280,13 +265,10 @@ public:
     //
     /// @param name         Name of the member.
     /// @param val          Value to assign to the member.
-    ///
     /// @param flags        Flags for the new member. By default dontDelete
     ///                     and dontEnum.
-    /// @param nsname       The id of the namespace to which this member
-    ///                     belongs.
     void init_member(const std::string& name, const as_value& val, 
-        int flags = DefaultFlags, string_table::key nsname = 0);
+        int flags = DefaultFlags);
 
     /// Initialize a member value by key
     //
@@ -303,13 +285,9 @@ public:
     ///
     /// @param flags    Flags for the new member. By default dontDelete
     ///                 and dontEnum.
-    /// @param slotId   If this is a non-negative value which will fit in
-    ///                 an unsigned short, this is used as the slotId and
-    ///                 can be subsequently found with get_slot
     void init_member(const ObjectURI& uri, const as_value& val, 
-        int flags = DefaultFlags, int slotId = -1);
+        int flags = DefaultFlags);
 
-    /// \brief
     /// Initialize a getter/setter property by name
     //
     /// This is just a wrapper around the other init_property method
@@ -331,13 +309,9 @@ public:
     /// @param flags
     ///     Flags for the new member. By default dontDelete and dontEnum.
     ///    See PropFlags::Flags.
-    ///
-    /// @param nsname
-    /// The id of the namespace to which this member belongs. 0 is a wildcard
-    /// and will be matched by anything not asking for a specific namespace.
     void init_property(const std::string& key, as_function& getter,
-        as_function& setter, int flags = DefaultFlags,
-        string_table::key nsname = 0);
+        as_function& setter, int flags = DefaultFlags);
+        
 
     /// Initialize a getter/setter property by name
     //
@@ -345,28 +319,16 @@ public:
     /// used as a trampoline to avoid changing all classes to 
     /// use string_table::key directly.
     ///
-    /// @param key
-    ///     Name of the property.
-    ///    Will be converted to lowercase if VM is initialized for SWF6 or lower.
-    ///
-    /// @param getter
-    ///    A function to invoke when this property value is requested.
-    ///    add_ref will be called on the function.
-    ///
-    /// @param setter
-    ///    A function to invoke when setting this property's value.
-    ///    add_ref will be called on the function.
-    ///
-    /// @param flags
-    ///     Flags for the new member. By default dontDelete and dontEnum.
-    ///    See PropFlags::Flags.
-    ///
-    /// @param nsname
-    /// The id of the namespace to which this member belongs. 0 is a wildcard
-    /// and will be matched by anything not asking for a specific namespace.
+    /// @param key      Name of the property. Will be converted to lowercase
+    ///                 if VM is initialized for SWF6 or lower.
+    /// @param getter   A function to invoke when this property value is
+    ///                 requested.
+    /// @param setter   A function to invoke when setting this property's
+    ///                 value.
+    /// @param flags    Flags for the new member. By default dontDelete and
+    ///                 dontEnum. See PropFlags::Flags.
     void init_property(const std::string& key, as_c_function_ptr getter,
-        as_c_function_ptr setter, int flags = DefaultFlags,
-        string_table::key nsname = 0);
+        as_c_function_ptr setter, int flags = DefaultFlags);
 
     /// Initialize a getter/setter property by key
     //
@@ -400,7 +362,6 @@ public:
     void init_property(const ObjectURI& uri, as_c_function_ptr getter,
         as_c_function_ptr setter, int flags = DefaultFlags);
 
-
     /// Initialize a destructive getter property
     //
     /// A destructive getter can be used as a place holder for the real
@@ -429,8 +390,6 @@ public:
     bool init_destructive_property(const ObjectURI& uri, 
             as_c_function_ptr getter, int flags = PropFlags::dontEnum);
 
-
-    /// \brief
     /// Use this method for read-only properties.
     //
     /// This method achieves the same as the above init_property method.
@@ -444,11 +403,8 @@ public:
     /// @param key      Property name id
     /// @param getter   The getter function
     /// @param flags    Property flags
-    /// @param nsname   The id of the namespace to which this member belongs.
-    ///                 0 is a wildcard and will be matched by anything not
-    ///                 asking for a specific namespace.
     void init_readonly_property(const std::string& key, as_function& getter,
-            int flags = DefaultFlags, string_table::key nsname = 0);
+            int flags = DefaultFlags);
 
     void init_readonly_property(const ObjectURI& uri,
             as_function& getter, int flags = DefaultFlags);
@@ -466,12 +422,8 @@ public:
     /// @param key      Property name id
     /// @param getter   The getter function
     /// @param flags    Property flags
-    /// @param nsname   The id of the namespace to which this member belongs.
-    ///                 0 is a wildcard and will be matched by anything not
-    ///                 asking for a specific namespace.
     void init_readonly_property(const std::string& key,
-            as_c_function_ptr getter, int flags = DefaultFlags,
-            string_table::key nsname = 0);
+            as_c_function_ptr getter, int flags = DefaultFlags);
 
     void init_readonly_property(const ObjectURI& uri,
             as_c_function_ptr getter, int flags = DefaultFlags);
@@ -564,7 +516,7 @@ public:
     ///                 - (false, false) : property not found
     ///                 - (true, false) : property protected from deletion
     ///                 - (true, true) : property successfully deleted
-    std::pair<bool,bool> delProperty(const ObjectURI& uri);
+    std::pair<bool, bool> delProperty(const ObjectURI& uri);
 
     /// Get this object's own named property, if existing.
     //
@@ -586,51 +538,14 @@ public:
     /// @return         true if the object has the property, false otherwise.
     bool hasOwnProperty(const ObjectURI& uri);
 
-    /// Get a property from this object (or a prototype) by ordering index.
-    ///
-    /// @param index
-    /// An index returned by nextIndex
-    ///
-    /// @return
-    /// The property associated with the order index.
-    const Property* getByIndex(int index);
-
-    /// Get the next index after the one whose index was used as a parameter.
-    ///
-    /// @param index
-    /// 0 is a starter index -- use it to get the first index. Using the
-    /// return value in subsequent calls will walk through all enumerable
-    /// properties in the list.
-    ///
-    /// @param owner
-    /// If owner is not NULL, it will be set to the exact object to which
-    /// the property used for the value of index belongs, if such a property
-    /// exists, and left untouched otherwise.
-    ///
-    /// @return
-    /// A value which can be fed to getByIndex, or 0 if there are no more.
-    int nextIndex(int index, as_object **owner = NULL);
-
     /// Set member flags (probably used by ASSetPropFlags)
     //
-    /// @param name
-    ///    Name of the property. Must be all lowercase
-    ///    if the current VM is initialized for a  target
-    ///    up to SWF6.
-    ///
-    /// @param setTrue
-    ///    the set of flags to set
-    ///
-    /// @param setFalse
-    ///    the set of flags to clear
-    ///
-    /// @param nsname
-    /// The id of the namespace to which this member belongs. 0 is a wildcard
-    /// and will be matched by anything not asking for a specific namespace.
-    ///
-    /// @return true on success, false on failure
-    ///    (non-existent or protected member)
-    bool set_member_flags(const ObjectURI& uri, int setTrue, int setFalse = 0);
+    /// @param name     Name of the property. Must be all lowercase
+    ///                 if the current VM is initialized for a  target
+    ///                 up to SWF6.
+    /// @param setTrue  The set of flags to set
+    /// @param setFalse The set of flags to clear
+    void set_member_flags(const ObjectURI& uri, int setTrue, int setFalse = 0);
 
     /// Cast to a as_function, or return NULL
     virtual as_function* to_function() { return 0; }
@@ -730,15 +645,15 @@ public:
     void add_property(const std::string& key, as_function& getter,
         as_function* setter);
 
-    /// Return this object's '__proto__' member.
+    /// Return this object's __proto__ member.
     //
-    /// The __proto__ member is the exported interface ('prototype')
+    /// The __proto__ member is the exported interface (prototype)
     /// of the class this object is an instance of.
     ///
     /// NOTE: can return NULL (and it is expected to do for Object.prototype)
     as_object* get_prototype() const;
 
-    /// Set this object's '__proto__' member
+    /// Set this object's __proto__ member
     //
     /// This does more or less what set_member("__proto__") does, but without
     /// the lookup process.
@@ -787,55 +702,34 @@ public:
         _array = array;
     }
 
-    /// Return true if this is a DisplayObject.
+    /// Return the DisplayObject associated with this object.
+    //
+    /// @return     A DisplayObject if this is as_object is associated with
+    ///             one, otherwise 0.
     DisplayObject* displayObject() const {
         return _displayObject;
     }
 
-    /// Indicate that this object is a DisplayObject
-    //
-    /// This enables DisplayObject properties such as _x and _y. A flag
-    /// is used to avoid RTTI on every get and set of properties.
+    /// Set the DisplayObject associated with this as_object.
     void setDisplayObject(DisplayObject* d) {
         _displayObject = d;
     }
 
-    ///Get a member value at a given slot.
-    //
-    /// @param order
-    /// The slot index of the property.
-    ///
-    /// @param val
-    /// The as_value to store a found variable's value in.
-    ///
-    /// @return true if a member exists at the given slot, 
-    /// and the member's value is successfully retrieved,
-    /// false otherwise.
-    bool get_member_slot(int order, as_value* val);
-
-    ///Set a member value at a given slot.
-    //
-    ///This is a wrapper around set_member_default.
-    /// @param order
-    ///
-    /// The slot index of the property.
-    /// @param val
-    ///    Value to assign to the named property.
-    ///
-    /// @param ifFound
-    ///    If true, don't create a new member, but only update
-    ///    an existing one.
-    ///
-    /// @return true if the member exists at the given slot, 
-    /// false otherwise.
-    ///    NOTE: the return doesn't tell if the member exists after
-    ///          the call, as watch triggers might have deleted it
-    ///          after setting.
-    ///
-    bool set_member_slot(int order, const as_value& val, bool ifFound = false);
-
 protected:
 
+    /// Construct an as_object associated with a VM.
+    //
+    /// This constructor is intended for subclasses. Although they could call
+    /// the public constructor that accepts a Global_as, this could imply
+    /// that that constructor can access members of the passed Global_as
+    /// other than getVM(), which might not be available because the Global_as
+    /// will not be fully constructed yet. While that is currently not the
+    /// case, using this constructor eliminates this potential initialization
+    /// order problem.
+    /// @param vm The VM to associate the newly created as_object with.
+    explicit as_object(VM& vm);
+
+
     /// Mark all reachable resources, override from GcResource.
     //
     /// The default implementation marks all properties
@@ -853,6 +747,22 @@ protected:
 
 private:
 
+    /// Find an existing property for update
+    //
+    /// Scans the inheritance chain only for getter/setters or statics.
+    //
+    /// NOTE: updatable here doesn't mean the property isn't protected
+    /// from update but only that a set_member will NOT create a new
+    /// property (either completely new or as an override).
+    ///
+    /// @returns a property if found, NULL if not found
+    ///          or not visible in current VM version
+    ///
+    Property* findUpdatableProperty(const ObjectURI& uri);
+
+    void executeTriggers(Property* prop, const ObjectURI& uri,
+            const as_value& val);
+
     /// A utility class for processing this as_object's inheritance chain
     template<typename T> class PrototypeRecursor;
 
@@ -878,27 +788,11 @@ private:
     boost::scoped_ptr<Relay> _relay;
 
     /// The VM containing this object.
-    VM& _vm;   
+    VM& _vm;
 
     /// Properties of this as_object
     PropertyList _members;
 
-    /// \brief
-    /// Find an existing property for update, only scanning the
-    /// inheritance chain for getter/setters or statics.
-    //
-    /// NOTE: updatable here doesn't mean the property isn't protected
-    /// from update but only that a set_member will NOT create a new
-    /// property (either completely new or as an override).
-    ///
-    /// @returns a property if found, NULL if not found
-    ///          or not visible in current VM version
-    ///
-    Property* findUpdatableProperty(const ObjectURI& uri);
-
-    void executeTriggers(Property* prop, const ObjectURI& uri,
-            const as_value& val);
-
     /// The constructors of the objects implemented by this as_object.
     //
     /// There is no need to use a complex container as the list of 
@@ -915,7 +809,7 @@ class IsVisible
 public:
     IsVisible(int version) : _version(version) {}
     bool operator()(const Property& prop) const {
-        return prop.visible(_version);
+        return visible(prop, _version);
     }
 private:
     const int _version;
@@ -1016,9 +910,10 @@ int getSWFVersion(const as_object& o);
 /// Get the Global object from an as_object.
 Global_as& getGlobal(const as_object& o);
 
-/// Return whether the object is an AS3 object.
-bool isAS3(const as_object& o);
-
+/// Return whether property matching is caseless
+inline bool caseless(const as_object& o) {
+    return getSWFVersion(o) < 7;
+}
 
 } // namespace gnash
 
