@@ -1,6 +1,7 @@
 // Color.cpp:  ActionScript class for colors, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,10 +18,6 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "Color_as.h"
 #include "as_object.h" // for inheritance
 #include "log.h"
@@ -29,11 +26,11 @@
 #include "smart_ptr.h" // for boost intrusive_ptr
 #include "builtin_function.h" // need builtin_function
 #include "NativeFunction.h" 
-#include "Object.h" // for getObjectInterface
 #include "cxform.h" // for composition
 #include "VM.h"
 #include "MovieClip.h"
 
+#include "namedStrings.h"
 #include <sstream>
 
 
@@ -72,7 +69,7 @@ color_class_init(as_object& where, const ObjectURI& uri)
             attachColorInterface, 0, uri);
 
     as_object* proto =
-        cl->getMember(NSV::PROP_PROTOTYPE).to_object(*getGlobal(where)).get();
+        cl->getMember(NSV::PROP_PROTOTYPE).to_object(getGlobal(where));
 
     if (!proto) return;
 
@@ -104,12 +101,12 @@ attachColorInterface(as_object& o)
 as_value
 color_getrgb(const fn_call& fn)
 {
-	boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
+	as_object* obj = ensure<ValidThis>(fn);
 
-    MovieClip* sp = getTarget(obj.get(), fn);
+    MovieClip* sp = getTarget(obj, fn);
     if (!sp) return as_value();
 
-	const cxform& trans = sp->get_user_cxform();
+	const cxform& trans = sp->get_cxform();
 
     const int r = trans.rb;
     const int g = trans.gb;
@@ -123,17 +120,17 @@ color_getrgb(const fn_call& fn)
 as_value
 color_gettransform(const fn_call& fn)
 {
-	boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
+	as_object* obj = ensure<ValidThis>(fn);
 
-    MovieClip* sp = getTarget(obj.get(), fn);
+    MovieClip* sp = getTarget(obj, fn);
     if (!sp) return as_value();
 
-	const cxform& cx = sp->get_user_cxform();
+	const cxform& cx = sp->get_cxform();
 
 	// Convert to as_object
 
-    Global_as* gl = getGlobal(fn);
-	as_object* ret = gl->createObject();
+    Global_as& gl = getGlobal(fn);
+	as_object* ret = gl.createObject();
 
 	ret->init_member("ra", double(cx.ra / 2.56));
 	ret->init_member("ga", double(cx.ga / 2.56));
@@ -151,7 +148,7 @@ color_gettransform(const fn_call& fn)
 as_value
 color_setrgb(const fn_call& fn)
 {
-	boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
+	as_object* obj = ensure<ValidThis>(fn);
 	
     if (!fn.nargs) {
 		IF_VERBOSE_ASCODING_ERRORS(
@@ -160,16 +157,16 @@ color_setrgb(const fn_call& fn)
 		return as_value();
 	}
 
-    MovieClip* sp = getTarget(obj.get(), fn);
+    MovieClip* sp = getTarget(obj, fn);
     if (!sp) return as_value();
 
-	boost::int32_t color = fn.arg(0).to_int();
+	boost::int32_t color = toInt(fn.arg(0));
 
 	const int r = (color & 0xff0000) >> 16;
 	const int g = (color & 0x00ff00) >> 8;
 	const int b = (color & 0x0000ff);
 
-	cxform newTrans = sp->get_user_cxform();
+	cxform newTrans = sp->get_cxform();
 	newTrans.rb = static_cast<boost::int16_t>(r);
 	newTrans.gb = static_cast<boost::int16_t>(g);
 	newTrans.bb = static_cast<boost::int16_t>(b);
@@ -177,14 +174,14 @@ color_setrgb(const fn_call& fn)
 	newTrans.ga = 0;
 	newTrans.ba = 0;
 
-    sp->set_user_cxform(newTrans);
+    sp->set_cxform(newTrans);
 
 	return as_value();
 }
 as_value
 color_settransform(const fn_call& fn)
 {
-	boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
+	as_object* obj = ensure<ValidThis>(fn);
 
 	if (!fn.nargs) {
 		IF_VERBOSE_ASCODING_ERRORS(
@@ -193,7 +190,7 @@ color_settransform(const fn_call& fn)
 		return as_value();
 	}
 
-	boost::intrusive_ptr<as_object> trans = fn.arg(0).to_object(*getGlobal(fn));
+	as_object* trans = fn.arg(0).to_object(getGlobal(fn));
 
     if (!trans) {
 		IF_VERBOSE_ASCODING_ERRORS(
@@ -204,12 +201,12 @@ color_settransform(const fn_call& fn)
 		return as_value();
 	}
 
-    MovieClip* sp = getTarget(obj.get(), fn);
+    MovieClip* sp = getTarget(obj, fn);
     if (!sp) return as_value();
 
 	string_table& st = getStringTable(*obj);
 
-	cxform newTrans = sp->get_user_cxform();
+	cxform newTrans = sp->get_cxform();
 
 	// multipliers
 	parseColorTransProp(*trans, st.find("ra"), newTrans.ra, true);
@@ -223,7 +220,7 @@ color_settransform(const fn_call& fn)
 	parseColorTransProp(*trans, st.find("bb"), newTrans.bb, false);
 	parseColorTransProp(*trans, st.find("ab"), newTrans.ab, false);
 
-	sp->set_user_cxform(newTrans);
+	sp->set_cxform(newTrans);
 
 	return as_value();
 }
@@ -238,7 +235,7 @@ as_value
 color_ctor(const fn_call& fn)
 {
 	
-    as_object* obj = fn.this_ptr;
+    as_object* obj = ensure<ValidThis>(fn);
     
     as_value target;
     if (fn.nargs) target = fn.arg(0);
@@ -247,6 +244,10 @@ color_ctor(const fn_call& fn)
 
     obj->init_member(NSV::PROP_TARGET, target, flags); 
 
+    Global_as& gl = getGlobal(fn);
+    as_object* null = 0;
+    callMethod(&gl, NSV::PROP_AS_SET_PROP_FLAGS, obj, null, 7);
+
 	return as_value(); 
 }
 
@@ -272,10 +273,10 @@ inline MovieClip*
 getTarget(as_object* obj, const fn_call& fn)
 {
     const as_value& target = obj->getMember(NSV::PROP_TARGET);
-    MovieClip* sp = target.to_sprite();
+    MovieClip* sp = target.toMovieClip();
     if (sp) return sp;
-    DisplayObject* d = fn.env().find_target(target.to_string());
-    if (d) return d->to_movie();
+    DisplayObject* o = fn.env().find_target(target.to_string());
+    if (o) return o->to_movie();
     return 0;
 }
 
