@@ -19,9 +19,14 @@
 #ifndef GNASH_ASOBJ_DATE_H
 #define GNASH_ASOBJ_DATE_H
 
-#include "as_object.h" // for inheritance
-#include "fn_call.h" // for inheritance
+#include <string>
 #include "ClockTime.h"
+#include "Relay.h"
+
+namespace gnash {
+    class as_object;
+    struct ObjectURI;
+}
 
 namespace gnash {
 
