@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,18 +23,19 @@
 namespace gnash {
 
 class as_object;
+struct ObjectURI;
 
 class Mouse_as
 {
 public:
     /// Register native functions with the VM
-    static void registerNative(as_object& global);
-
-    /// Initialize the global Mouse class
-    static void init(as_object& global);
+    static void registerNative(as_object& where);
 };
 
-} // end of gnash namespace
+void mouse_class_init(as_object& where, const ObjectURI& uri);
+
+} // gnash namespace
+
 
 #endif
 
