@@ -1,6 +1,7 @@
 // NetConnection_as.cpp:  Open local connections for FLV files or URLs.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,35 +23,29 @@
 #include "gnashconfig.h"
 #endif
 
-#include <iostream>
-#include <string>
-#include <boost/scoped_ptr.hpp>
-
-// FIXME: Get rid of this crap.
-#if defined(HAVE_WINSOCK_H) && !defined(__OS2__)
-# include <winsock.h>
-#else
-#include <arpa/inet.h> // for htons
-#endif
-
+#include "GnashSystemNetHeaders.h"
 #include "NetConnection_as.h"
 #include "log.h"
 #include "GnashException.h"
 #include "builtin_function.h"
 #include "movie_root.h"
-#include "Object.h" // for getObjectInterface
-
 #include "StreamProvider.h"
 #include "URLAccessManager.h"
 #include "URL.h"
-
-// for NetConnection_as.call()
 #include "VM.h"
-#include "amf.h"
 #include "SimpleBuffer.h"
-#include "timers.h"
 #include "namedStrings.h"
+#include "GnashAlgorithm.h"
+#include "fn_call.h"
+#include "Global_as.h"
+#include "AMFConverter.h"
+#include "smart_ptr.h"
+#include "RunResources.h"
+#include "IOChannel.h"
 
+#include <iostream>
+#include <string>
+#include <boost/scoped_ptr.hpp>
 
 //#define GNASH_DEBUG_REMOTING
 
@@ -61,7 +56,6 @@ namespace gnash {
 namespace {
     void attachProperties(as_object& o);
     void attachNetConnectionInterface(as_object& o);
-    as_object* getNetConnectionInterface();
     as_value netconnection_isConnected(const fn_call& fn);
     as_value netconnection_uri(const fn_call& fn);
     as_value netconnection_connect(const fn_call& fn);
@@ -79,7 +73,7 @@ namespace {
 
 }
 
-//---- ConnectionHandler --------------------------------------------------------------
+//---- ConnectionHandler -------------------------------------------------
 
 /// Abstract connection handler class
 //
@@ -122,7 +116,12 @@ public:
     ///
     /// Return true if wants to be advanced again, false otherwise.
     ///
-    virtual bool advance()=0;
+    virtual bool advance() = 0;
+
+    /// ConnectionHandlers may store references to as_objects
+    //
+    /// These include callback objects.
+    virtual void setReachable() const = 0;
 
     /// Return true if the connection has pending calls 
     //
@@ -131,19 +130,7 @@ public:
     /// queued and only really dropped when advance returns
     /// false
     ///
-    virtual bool hasPendingCalls() const=0;
-
-    /// Mark reachable resources, if any. 
-    virtual void setReachable() const
-    {
-        // NOTE: usually this function gets
-        //       called *by* the _nc's setReachable
-        //       but we do this just to be safe
-        //       in case the _nc object is deleted
-        //       and doesn't properly drops us
-        //
-        _nc.setReachable();
-    }
+    virtual bool hasPendingCalls() const = 0;
 
     virtual ~ConnectionHandler() {}
 
@@ -152,7 +139,9 @@ protected:
     /// Construct a connection handler bound to the given NetConnection object
     //
     /// The binding is used to notify statuses and errors
-    ///
+    //
+    /// The NetConnection_as owns all ConnectionHandlers, so there is no
+    /// need to mark it reachable.
     ConnectionHandler(NetConnection_as& nc)
         :
         _nc(nc)
@@ -184,7 +173,8 @@ ConnectionHandler::getStream(const std::string&)
 /// script specified a callback function, use the optional parameters to specify
 /// the identifier (which must be unique) and the callback object as an as_value
 ///
-class HTTPRemotingHandler : public ConnectionHandler {
+class HTTPRemotingHandler : public ConnectionHandler
+{
 
 public:
 
@@ -215,7 +205,6 @@ public:
         {
             i->second->setReachable();
         }
-        ConnectionHandler::setReachable();
     }
 
     // See dox in NetworkHandler class
@@ -272,12 +261,12 @@ private:
     {
         push_amf(amf);
         push_callback(identifier, callback);
-    };
+    }
 
     void enqueue(const SimpleBuffer &amf)
     {
         push_amf(amf);
-    };
+    }
     
 };
 
@@ -306,33 +295,27 @@ HTTPRemotingHandler::advance()
 #ifdef GNASH_DEBUG_REMOTING
     log_debug("advancing HTTPRemotingHandler");
 #endif
-    if(_connection)
-    {
-
-        VM& vm = _nc.getVM();
+    if (_connection) {
 
 #ifdef GNASH_DEBUG_REMOTING
         log_debug("have connection");
 #endif
 
         // Fill last chunk before reading in the next
-        size_t toRead = reply.capacity()-reply.size();
-        if ( ! toRead ) toRead = NCCALLREPLYCHUNK;
+        size_t toRead = reply.capacity() - reply.size();
+        if (! toRead) toRead = NCCALLREPLYCHUNK;
 
 #ifdef GNASH_DEBUG_REMOTING
         log_debug("Attempt to read %d bytes", toRead);
 #endif
 
-        //
         // See if we need to allocate more bytes for the next
         // read chunk
-        //
-        if ( reply.capacity() < reply.size()+toRead )
-        {
+        if (reply.capacity() < reply.size() + toRead) {
             // if _connection->size() >= 0, reserve for it, so
             // if HTTP Content-Length response header is correct
             // we'll be allocating only once for all.
-            size_t newCapacity = reply.size()+toRead;
+            const size_t newCapacity = reply.size() + toRead;
 
 #ifdef GNASH_DEBUG_REMOTING
             log_debug("NetConnection.call: reply buffer capacity (%d) "
@@ -348,13 +331,15 @@ HTTPRemotingHandler::advance()
 #endif
         }
 
-        int read = _connection->readNonBlocking(reply.data() + reply.size(), toRead);
-        if(read > 0) {
+        const int read =
+            _connection->readNonBlocking(reply.data() + reply.size(), toRead);
+
+        if (read > 0) {
 #ifdef GNASH_DEBUG_REMOTING
             log_debug("read '%1%' bytes: %2%", read, 
                     hexify(reply.data() + reply.size(), read, false));
 #endif
-            reply.resize(reply.size()+read);
+            reply.resize(reply.size() + read);
         }
 
         // There is no way to tell if we have a whole amf reply without
@@ -370,8 +355,7 @@ HTTPRemotingHandler::advance()
         // the buffer is full, 2) when we have a "length in bytes" value
         // thas is satisfied
 
-        if (_connection->bad())
-        {
+        if (_connection->bad()) {
             log_debug("connection is in error condition, calling "
                     "NetConnection.onStatus");
             reply.resize(0);
@@ -383,35 +367,35 @@ HTTPRemotingHandler::advance()
             // 'undefined'
             _nc.notifyStatus(NetConnection_as::CALL_FAILED);
         }
-        else if(_connection->eof() )
-        {
-            if ( reply.size() > 8)
-            {
-                std::vector<as_object*> objRefs;
+        else if (_connection->eof()) {
+
+            if (reply.size() > 8) {
+
 
 #ifdef GNASH_DEBUG_REMOTING
                 log_debug("hit eof");
 #endif
-                boost::int16_t si;
                 boost::uint16_t li;
                 const boost::uint8_t *b = reply.data() + reply_start;
                 const boost::uint8_t *end = reply.data() + reply.size();
+                
+                amf::Reader rd(b, end, getGlobal(_nc.owner()));
 
                 // parse header
                 b += 2; // skip version indicator and client id
 
                 // NOTE: this looks much like parsing of an OBJECT_AMF0
-                si = readNetworkShort(b); b += 2; // number of headers
+                boost::int16_t si = readNetworkShort(b);
+                b += 2; // number of headers
                 uint8_t headers_ok = 1;
-                if (si != 0)
-                {
+                if (si != 0) {
+
 #ifdef GNASH_DEBUG_REMOTING
                     log_debug("NetConnection::call(): amf headers "
                             "section parsing");
 #endif
                     as_value tmp;
-                    for(int i = si; i > 0; --i)
-                    {
+                    for (size_t i = si; i > 0; --i) {
                         if(b + 2 > end) {
                             headers_ok = 0;
                             break;
@@ -431,8 +415,7 @@ HTTPRemotingHandler::advance()
                             break;
                         }
                         b += 5; // skip past bool and length long
-                        if( !tmp.readAMF0(b, end, -1, objRefs, vm) )
-                        {
+                        if(!rd(tmp)) {
                             headers_ok = 0;
                             break;
                         }
@@ -442,14 +425,14 @@ HTTPRemotingHandler::advance()
 
                         { // method call for each header
                           // FIXME: it seems to me that the call should happen
-                            VM& vm = _nc.getVM();
+                            VM& vm = getVM(_nc.owner());
                             string_table& st = vm.getStringTable();
                             string_table::key key = st.find(headerName);
 #ifdef GNASH_DEBUG_REMOTING
                             log_debug("Calling NetConnection.%s(%s)",
                                     headerName, tmp);
 #endif
-                            _nc.callMethod(key, tmp);
+                            callMethod(&_nc.owner(), key, tmp);
                         }
                     }
                 }
@@ -512,9 +495,8 @@ HTTPRemotingHandler::advance()
                             log_debug("about to parse amf value");
 #endif
                             // this updates b to point to the next unparsed byte
-                            as_value reply_as_value;
-                            if ( ! reply_as_value.readAMF0(b, end, -1, objRefs, vm) )
-                            {
+                            as_value replyval;
+                            if (!rd(replyval)) {
                                 log_error("parse amf failed");
                                 // this will happen if we get
                                 // bogus data, or if the data runs
@@ -530,28 +512,35 @@ HTTPRemotingHandler::advance()
                             // update variable to show how much we've parsed
                             reply_start = b - reply.data();
 
-                            // if actionscript specified a callback object, call it
-                            boost::intrusive_ptr<as_object> callback = pop_callback(id);
-                            if(callback) {
+                            // if actionscript specified a callback object,
+                            // call it
+                            as_object* callback = pop_callback(id);
+                            if (callback) {
 
                                 string_table::key methodKey;
                                 if ( methodName == "onResult" ) {
                                     methodKey = NSV::PROP_ON_RESULT;
-                                } else if ( methodName == "onStatus" ) {
+                                }
+                                else if (methodName == "onStatus") {
                                     methodKey = NSV::PROP_ON_STATUS;
-                                } else {
-                                    // NOTE: the pp is known to actually invoke the custom
-                                    //       method, but with 7 undefined arguments (?)
-                                    //methodKey = _nc.getVM().getStringTable().find(methodName);
-                                    log_error("Unsupported HTTP Remoting response callback: '%s' (size %d)", methodName, methodName.size());
+                                }
+                                else {
+                                    // NOTE: the pp is known to actually
+                                    // invoke the custom method, but with 7
+                                    // undefined arguments (?)
+                                    log_error("Unsupported HTTP Remoting "
+                                            "response callback: '%s' "
+                                            "(size %d)", methodName,
+                                            methodName.size());
                                     continue;
                                 }
 
 #ifdef GNASH_DEBUG_REMOTING
                                 log_debug("calling onResult callback");
 #endif
-                                // FIXME check if above line can fail and we have to react
-                                callback->callMethod(methodKey, reply_as_value);
+                                // FIXME check if above line can fail and we
+                                // have to react
+                                callMethod(callback, methodKey, replyval);
 #ifdef GNASH_DEBUG_REMOTING
                                 log_debug("callback called");
 #endif
@@ -592,7 +581,7 @@ HTTPRemotingHandler::advance()
         // TODO: it might be useful for a Remoting Handler to have a 
         // StreamProvider member
         const StreamProvider& sp =
-            _nc.getVM().getRoot().runInfo().streamProvider();
+            getRunResources(_nc.owner()).streamProvider();
 
         _connection.reset(sp.getStream(_url, postdata_str, _headers).release());
 
@@ -608,57 +597,53 @@ HTTPRemotingHandler::advance()
     }
 
     return true;
-};
+}
 
 void
 HTTPRemotingHandler::call(as_object* asCallback, const std::string& methodName,
             const std::vector<as_value>& args, size_t firstArg)
 {
-    boost::scoped_ptr<SimpleBuffer> buf (new SimpleBuffer(32));
+    SimpleBuffer buf(32);
 
     // method name
-    buf->appendNetworkShort(methodName.size());
-    buf->append(methodName.c_str(), methodName.size());
+    buf.appendNetworkShort(methodName.size());
+    buf.append(methodName.c_str(), methodName.size());
 
     // client id (result number) as counted string
     // the convention seems to be / followed by a unique (ascending) number
     std::ostringstream os;
     os << "/";
     // Call number is not used if the callback is undefined
-    if ( asCallback )
-    {
+    if (asCallback) {
         os << ++_numCalls; 
     }
     const std::string callNumberString = os.str();
 
-    buf->appendNetworkShort(callNumberString.size());
-    buf->append(callNumberString.c_str(), callNumberString.size());
+    buf.appendNetworkShort(callNumberString.size());
+    buf.append(callNumberString.c_str(), callNumberString.size());
 
-    size_t total_size_offset = buf->size();
-    buf->append("\000\000\000\000", 4); // total size to be filled in later
-
-    std::map<as_object*, size_t> offsetTable;
+    size_t total_size_offset = buf.size();
+    buf.append("\000\000\000\000", 4); // total size to be filled in later
 
     // encode array of arguments to remote method
-    buf->appendByte(amf::Element::STRICT_ARRAY_AMF0);
-    buf->appendNetworkLong(args.size()-firstArg);
+    buf.appendByte(amf::STRICT_ARRAY_AMF0);
+    buf.appendNetworkLong(args.size() - firstArg);
 
-    VM& vm = _nc.getVM();
+    // STRICT_ARRAY encoding is allowed for remoting
+    amf::Writer w(buf, true);
 
     for (unsigned int i = firstArg; i < args.size(); ++i)
     {
         const as_value& arg = args[i];
-        // STRICT_ARRAY encoding is allowed for remoting
-        if ( ! arg.writeAMF0(*buf, offsetTable, vm, true) )
-        {
+        if (!arg.writeAMF0(w)) {
             log_error("Could not serialize NetConnection.call argument %d",
                     i);
         }
     }
 
     // Set the "total size" parameter.
-    *(reinterpret_cast<uint32_t*>(buf->data() + total_size_offset)) = 
-        htonl(buf->size() - 4 - total_size_offset);
+    *(reinterpret_cast<uint32_t*>(buf.data() + total_size_offset)) = 
+        htonl(buf.size() - 4 - total_size_offset);
 
 #ifdef GNASH_DEBUG_REMOTING
     log_debug(_("NetConnection.call(): encoded args: %s"),
@@ -669,75 +654,53 @@ HTTPRemotingHandler::call(as_object* asCallback, const std::string& methodName,
 #ifdef GNASH_DEBUG_REMOTING
         log_debug("calling enqueue with callback");
 #endif
-        enqueue(*buf, callNumberString, asCallback);
+        enqueue(buf, callNumberString, asCallback);
     }
     
     else {
 #ifdef GNASH_DEBUG_REMOTING
         log_debug("calling enqueue without callback");
 #endif
-        enqueue(*buf);
+        enqueue(buf);
     }
 }
 
 //----- NetConnection_as ----------------------------------------------------
 
-NetConnection_as::NetConnection_as()
+NetConnection_as::NetConnection_as(as_object* owner)
     :
-    as_object(getNetConnectionInterface()),
+    ActiveRelay(owner),
     _queuedConnections(),
     _currentConnection(0),
     _uri(),
-    _isConnected(false),
-    _advanceTimer(0)
+    _isConnected(false)
 {
-    attachProperties(*this);
-}
-
-// extern (used by Global.cpp)
-void
-netconnection_class_init(as_object& global)
-{
-    // This is going to be the global NetConnection "class"/"function"
-    static boost::intrusive_ptr<builtin_function> cl;
-
-    if ( cl == NULL )
-    {
-        cl=new builtin_function(&netconnection_new,
-                getNetConnectionInterface());
-        // replicate all interface to class, to be able to access
-        // all methods as static functions
-        attachNetConnectionInterface(*cl);
-             
-    }
-
-    // Register _global.String
-    global.init_member("NetConnection", cl.get());
 }
 
 // here to have HTTPRemotingHandler definition available
 NetConnection_as::~NetConnection_as()
 {
-    for (std::list<ConnectionHandler*>::iterator
-            i=_queuedConnections.begin(), e=_queuedConnections.end();
-            i!=e; ++i)
-    {
-        delete *i;
-    }
+    deleteChecked(_queuedConnections.begin(), _queuedConnections.end());
+}
+
+// extern (used by Global.cpp)
+void
+netconnection_class_init(as_object& where, const ObjectURI& uri)
+{
+    registerBuiltinClass(where, netconnection_new,
+            attachNetConnectionInterface, 0, uri);
 }
 
 
 void
 NetConnection_as::markReachableResources() const
 {
-    if ( _currentConnection.get() ) _currentConnection->setReachable();
-    for (std::list<ConnectionHandler*>::const_iterator
-            i=_queuedConnections.begin(), e=_queuedConnections.end();
-            i!=e; ++i)
-    {
-        (*i)->setReachable();
-    }
-    markAsObjectReachable();
+    owner().setReachable();
+
+    std::for_each(_queuedConnections.begin(), _queuedConnections.end(),
+            std::mem_fun(&ConnectionHandler::setReachable));
+
+    if (_currentConnection.get()) _currentConnection->setReachable();
 }
 
 
@@ -749,8 +712,7 @@ std::string
 NetConnection_as::validateURL() const
 {
 
-    const movie_root& mr = _vm.getRoot();
-    URL uri(_uri, mr.runInfo().baseURL());
+    URL uri(_uri, getRunResources(owner()).baseURL());
 
     std::string uriStr(uri.str());
     assert(uriStr.find("://") != std::string::npos);
@@ -773,14 +735,14 @@ NetConnection_as::notifyStatus(StatusCode code)
     getStatusCodeInfo(code, info);
 
     /// This is a new normal object each time (see NetConnection.as)
-    as_object* o = new as_object(getObjectInterface());
+    as_object* o = getGlobal(owner()).createObject();
 
     const int flags = 0;
 
     o->init_member("code", info.first, flags);
     o->init_member("level", info.second, flags);
 
-    callMethod(NSV::PROP_ON_STATUS, o);
+    callMethod(&owner(), NSV::PROP_ON_STATUS, o);
 
 }
 
@@ -855,29 +817,27 @@ NetConnection_as::connect(const std::string& uri)
         return;
     }
 
-    const movie_root& mr = _vm.getRoot();
-    URL url(uri, mr.runInfo().baseURL());
+    URL url(uri, getRunResources(owner()).baseURL());
 
-    if ( url.protocol() == "rtmp" )
-    {
-        LOG_ONCE( log_unimpl("NetConnection.connect(%s): RTMP not yet supported", url) );
-        notifyStatus(CONNECT_FAILED);
-        return;
-    }
+    if ((url.protocol() != "rtmp")
+        && (url.protocol() != "rtmpt")
+        && (url.protocol() != "rtmpts")
+        && (url.protocol() != "https")
+        && (url.protocol() != "http")) {
 
-    if ( url.protocol() != "http" )
-    {
         IF_VERBOSE_ASCODING_ERRORS(
-        log_aserror("NetConnection.connect(%s): invalid connection protocol", url);
-        );
+		 log_aserror("NetConnection.connect(%s): invalid connection "
+			     "protocol", url);
+				   );
         notifyStatus(CONNECT_FAILED);
         return;
     }
-
+    
     // This is for HTTP remoting
 
     if (!URLAccessManager::allow(url)) {
-        log_security(_("Gnash is not allowed to NetConnection.connect to %s"), url);
+        log_security(_("Gnash is not allowed to NetConnection.connect "
+                    "to %s"), url);
         notifyStatus(CONNECT_FAILED);
         return;
     }
@@ -928,7 +888,7 @@ NetConnection_as::close()
 void
 NetConnection_as::setURI(const std::string& uri)
 {
-    init_readonly_property("uri", &netconnection_uri);
+    owner().init_readonly_property("uri", &netconnection_uri);
     _uri = uri;
 }
 
@@ -955,7 +915,7 @@ NetConnection_as::call(as_object* asCallback, const std::string& methodName,
 std::auto_ptr<IOChannel>
 NetConnection_as::getStream(const std::string& name)
 {
-    const RunInfo& ri = _vm.getRoot().runInfo();
+    const RunResources& ri = getRunResources(owner());
 
     const StreamProvider& streamProvider = ri.streamProvider();
 
@@ -972,82 +932,50 @@ NetConnection_as::getStream(const std::string& name)
 
 }
 
-as_value
-NetConnection_as::advanceWrapper(const fn_call& fn)
-{
-    boost::intrusive_ptr<NetConnection_as> ptr = 
-        ensureType<NetConnection_as>(fn.this_ptr);
-    
-    ptr->advance();
-    return as_value();
-};
-
 void
 NetConnection_as::startAdvanceTimer() 
 {
-
-    if (_advanceTimer)
-    {
-        //log_debug("startAdvanceTimer: already running %d", _advanceTimer);
-        return;
-    }
-
-    boost::intrusive_ptr<builtin_function> ticker_as = 
-            new builtin_function(&NetConnection_as::advanceWrapper);
-
-    std::auto_ptr<Timer> timer(new Timer);
-    unsigned long delayMS = 50; 
-    timer->setInterval(*ticker_as, delayMS, this);
-    _advanceTimer = getVM().getRoot().add_interval_timer(timer, true);
-
-    log_debug("startAdvanceTimer: registered advance timer %d", _advanceTimer);
+    getRoot(owner()).addAdvanceCallback(this);
+    log_debug("startAdvanceTimer: registered NetConnection timer");
 }
 
 void
 NetConnection_as::stopAdvanceTimer() 
 {
-    if (!_advanceTimer)
-    {
-        log_debug("stopAdvanceTimer: not running");
-        return;
-    }
-
-    getVM().getRoot().clear_interval_timer(_advanceTimer);
-    log_debug("stopAdvanceTimer: deregistered timer %d", _advanceTimer);
-    _advanceTimer=0;
+    getRoot(owner()).removeAdvanceCallback(this);
+    log_debug("stopAdvanceTimer: deregistered NetConnection timer");
 }
 
 void
-NetConnection_as::advance()
+NetConnection_as::update()
 {
     // Advance
 
 #ifdef GNASH_DEBUG_REMOTING
-    log_debug("NetConnection_as::advance: %d calls to advance", _queuedConnections.size());
+    log_debug("NetConnection_as::advance: %d calls to advance",
+            _queuedConnections.size());
 #endif
 
-    while ( ! _queuedConnections.empty() )
-    {
+    while (!_queuedConnections.empty()) {
         ConnectionHandler* ch = _queuedConnections.front();
-        if ( ! ch->advance() )
-        {
+
+        if (!ch->advance()) {
             log_debug("ConnectionHandler done, dropping");
             _queuedConnections.pop_front();
             delete ch;
         }
+
         else break; // queues handling is serialized
     }
 
-    if ( _currentConnection.get() ) 
-    {
+    if (_currentConnection.get()) {
         _currentConnection->advance();
     }
 
     // Advancement of a connection might trigger creation
     // of a new connection, so we won't stop the advance
     // timer in that case
-    if ( _queuedConnections.empty() && ! _currentConnection.get() )
-    {
+    if (_queuedConnections.empty() && ! _currentConnection.get()) {
 #ifdef GNASH_DEBUG_REMOTING
         log_debug("stopping advance timer");
 #endif
@@ -1090,8 +1018,7 @@ namespace {
 as_value
 netconnection_call(const fn_call& fn)
 {
-    boost::intrusive_ptr<NetConnection_as> ptr = 
-        ensureType<NetConnection_as>(fn.this_ptr); 
+    NetConnection_as* ptr = ensure<ThisIsNative<NetConnection_as> >(fn);
 
     if (fn.nargs < 1)
     {
@@ -1114,7 +1041,7 @@ netconnection_call(const fn_call& fn)
     if (fn.nargs > 1) {
 
         if (fn.arg(1).is_object()) {
-            asCallback = (fn.arg(1).to_object());
+            asCallback = (fn.arg(1).to_object(getGlobal(fn)));
         }
         else {
             IF_VERBOSE_ASCODING_ERRORS(
@@ -1134,8 +1061,7 @@ netconnection_call(const fn_call& fn)
 as_value
 netconnection_close(const fn_call& fn)
 {
-    boost::intrusive_ptr<NetConnection_as> ptr =
-        ensureType<NetConnection_as>(fn.this_ptr); 
+    NetConnection_as* ptr = ensure<ThisIsNative<NetConnection_as> >(fn);
 
     ptr->close();
 
@@ -1147,8 +1073,7 @@ netconnection_close(const fn_call& fn)
 as_value
 netconnection_isConnected(const fn_call& fn)
 {
-    boost::intrusive_ptr<NetConnection_as> ptr =
-        ensureType<NetConnection_as>(fn.this_ptr); 
+    NetConnection_as* ptr = ensure<ThisIsNative<NetConnection_as> >(fn);
 
     return as_value(ptr->isConnected());
 }
@@ -1156,8 +1081,7 @@ netconnection_isConnected(const fn_call& fn)
 as_value
 netconnection_uri(const fn_call& fn)
 {
-    boost::intrusive_ptr<NetConnection_as> ptr =
-        ensureType<NetConnection_as>(fn.this_ptr); 
+    NetConnection_as* ptr = ensure<ThisIsNative<NetConnection_as> >(fn);
 
     return as_value(ptr->getURI());
 }
@@ -1165,10 +1089,12 @@ netconnection_uri(const fn_call& fn)
 void
 attachNetConnectionInterface(as_object& o)
 {
-    o.init_member("connect", new builtin_function(netconnection_connect));
-    o.init_member("addHeader", new builtin_function(netconnection_addHeader));
-    o.init_member("call", new builtin_function(netconnection_call));
-    o.init_member("close", new builtin_function(netconnection_close));
+    Global_as& gl = getGlobal(o);
+
+    o.init_member("connect", gl.createFunction(netconnection_connect));
+    o.init_member("addHeader", gl.createFunction(netconnection_addHeader));
+    o.init_member("call", gl.createFunction(netconnection_call));
+    o.init_member("close", gl.createFunction(netconnection_close));
 }
 
 void
@@ -1177,32 +1103,18 @@ attachProperties(as_object& o)
     o.init_readonly_property("isConnected", &netconnection_isConnected);
 }
 
-as_object*
-getNetConnectionInterface()
-{
-
-    static boost::intrusive_ptr<as_object> o;
-    if ( o == NULL )
-    {
-        o = new as_object(getObjectInterface());
-        attachNetConnectionInterface(*o);
-    }
-
-    return o.get();
-}
-
 /// \brief callback to instantiate a new NetConnection object.
 /// \param fn the parameters from the Flash movie
 /// \return nothing from the function call.
 /// \note The return value is returned through the fn.result member.
 as_value
-netconnection_new(const fn_call& /* fn */)
+netconnection_new(const fn_call& fn)
 {
-    GNASH_REPORT_FUNCTION;
 
-    NetConnection_as* nc = new NetConnection_as;
-
-    return as_value(nc);
+    as_object* obj = fn.this_ptr;
+    obj->setRelay(new NetConnection_as(obj));
+    attachProperties(*obj);
+    return as_value();
 }
 
 
@@ -1222,8 +1134,7 @@ as_value
 netconnection_connect(const fn_call& fn)
 {
 
-    boost::intrusive_ptr<NetConnection_as> ptr =
-        ensureType<NetConnection_as>(fn.this_ptr); 
+    NetConnection_as* ptr = ensure<ThisIsNative<NetConnection_as> >(fn);
     
     if (fn.nargs < 1)
     {
@@ -1236,14 +1147,14 @@ netconnection_connect(const fn_call& fn)
 
     const as_value& uri = fn.arg(0);
 
-    const VM& vm = ptr->getVM();
-    const std::string& uriStr = uri.to_string_versioned(vm.getSWFVersion());
+    const VM& vm = getVM(fn);
+    const std::string& uriStr = uri.to_string(vm.getSWFVersion());
     
     // This is always set without validification.
     ptr->setURI(uriStr);
 
     // Check first arg for validity 
-    if (uri.is_null() || (vm.getSWFVersion() > 6 && uri.is_undefined())) {
+    if (uri.is_null() || (getSWFVersion(fn) > 6 && uri.is_undefined())) {
         ptr->connect();
     }
     else {
@@ -1264,8 +1175,7 @@ netconnection_connect(const fn_call& fn)
 as_value
 netconnection_addHeader(const fn_call& fn)
 {
-    boost::intrusive_ptr<NetConnection_as> ptr =
-        ensureType<NetConnection_as>(fn.this_ptr); 
+    NetConnection_as* ptr = ensure<ThisIsNative<NetConnection_as> >(fn);
     UNUSED(ptr);
 
     log_unimpl("NetConnection.addHeader()");
@@ -1276,3 +1186,7 @@ netconnection_addHeader(const fn_call& fn)
 
 } // end of gnash namespace
 
+// local Variables:
+// mode: C++
+// indent-tabs-mode: t
+// End:
