@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,19 +20,12 @@
 #ifndef GNASH_NETSTREAM_H
 #define GNASH_NETSTREAM_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
 
 #ifndef __STDC_CONSTANT_MACROS
 #define __STDC_CONSTANT_MACROS
 #endif
 
-#include "smart_ptr.h" // GNASH_USE_GC
-#include "impl.h"
 #include "MediaParser.h"
-#include "as_function.h" // for visibility of destructor by intrusive_ptr
-#include "NetConnection_as.h"
 #include "PlayHead.h" // for composition
 
 #include "VideoDecoder.h" // for visibility of dtor
@@ -39,6 +33,10 @@
 
 #include "VirtualClock.h"
 
+#include "Relay.h" // for ActiveRelay inheritance
+
+#include <boost/intrusive_ptr.hpp>
+#include <string>
 #include <deque>
 #include <boost/scoped_ptr.hpp>
 
@@ -46,6 +44,10 @@
 namespace gnash {
     class CharacterProxy;
     class IOChannel;
+    class NetConnection_as;
+    class as_function;
+    class DisplayObject;
+    struct ObjectURI;
     namespace media {
         class MediaHandler;
     }
@@ -169,9 +171,156 @@ public:
 /// This class is responsible for handlign external
 /// media files. Provides interfaces for playback control.
 ///
-class NetStream_as : public as_object
+class NetStream_as : public ActiveRelay
 {
 
+public:
+
+    enum PauseMode {
+      pauseModeToggle = -1,
+      pauseModePause = 0,
+      pauseModeUnPause = 1  
+    };
+
+    NetStream_as(as_object* owner);
+
+    ~NetStream_as();
+
+    PlayHead::PlaybackStatus playbackState() const {
+        return _playHead.getState();
+    }
+
+    /// Get the real height of the video in pixels if the decoder exists.
+    //
+    /// @return the height of the video in pixels or 0 if no decoder exists.
+    ///         The width returned from the decoder may also vary, and will
+    ///         be 0 until it knows the width.
+    int videoHeight() const;
+
+    /// Get the real width of the video in pixels if the decoder exists.
+    //
+    /// @return the width of the video in pixels or 0 if no decoder exists.
+    ///         The width returned from the decoder may also vary, and will
+    ///         be 0 until it knows the width.
+    int videoWidth() const;
+
+    /// Closes the video session and frees all ressources used for decoding
+    /// except the FLV-parser (this might not be correct).
+    void close();
+
+    /// Make audio controlled by given DisplayObject
+    void setAudioController(DisplayObject* controller);
+ 
+    /// Pauses/starts the playback of the media played by the current instance
+    //
+    /// @param mode
+    /// Defines what mode to put the instance in.
+    void pause(PauseMode mode);
+
+    /// Starts the playback of the media
+    //
+    /// @param source
+    /// Defines what file to play
+    ///
+    void play(const std::string& source);
+
+    /// Seek in the media played by the current instance
+    //
+    /// @param pos
+    ///     Defines in seconds where to seek to
+    ///     @todo take milliseconds !!
+    ///
+    void seek(boost::uint32_t pos);
+
+    /// Tells where the playhead currently is
+    //
+    /// @return The time in milliseconds of the current playhead position
+    ///
+    boost::int32_t time();
+
+    /// Called at the SWF heart-beat. Used to process queued status messages
+    /// and (re)start after a buffering pause. In NetStreamFfmpeg it is also
+    /// used to find the next video frame to be shown, though this might
+    /// change.
+    void update();
+    
+    /// Returns the current framerate in frames per second.
+    double getCurrentFPS()  { return 0; }
+
+    /// Sets the NetConnection needed to access external files
+    //
+    /// @param nc
+    ///     The NetConnection object to use for network access
+    ///
+    void setNetCon(NetConnection_as* nc) {
+        _netCon = nc;
+    }
+
+    /// Return true if the NetStream has an associated NetConnection
+    bool isConnected() const { return (_netCon); }
+
+    /// Specifies the number of milliseconds to buffer before starting
+    /// to display the stream.
+    //
+    /// @param time
+    /// The time in milliseconds that should be buffered.
+    ///
+    void setBufferTime(boost::uint32_t time);
+
+    /// Returns what the buffer time has been set to. (100 milliseconds
+    /// is default)
+    //
+    /// @return The size of the buffer in milliseconds.
+    ///
+    boost::uint32_t bufferTime() { return m_bufferTime; }
+
+    /// Returns the number of bytes of the media file that have been buffered.
+    long bytesLoaded();
+
+    /// Returns the total number of bytes (size) of the media file
+    //
+    /// @return the total number of bytes (size) of the media file
+    ///
+    long bytesTotal();
+
+    /// Returns the number of millisecond of the media file that is
+    /// buffered and yet to be played
+    //
+    /// @return Returns the number of millisecond of the media file that is 
+    /// buffered and yet to be played
+    ///
+    long bufferLength();
+
+    /// Tells us if there is a new video frame ready
+    //
+    /// @return true if a frame is ready, false if not
+    bool newFrameReady();
+
+    /// Returns the video frame closest to current cursor. See time().
+    //
+    /// @return a image containing the video frame, a NULL auto_ptr if
+    /// none were ready
+    ///
+    std::auto_ptr<GnashImage> get_video();
+    
+    /// Register the DisplayObject to invalidate on video updates
+    void setInvalidatedVideo(DisplayObject* ch)
+    {
+        _invalidatedVideoCharacter = ch;
+    }
+
+    virtual void markReachableResources() const;
+
+    /// Callback used by sound_handler to get audio data
+    //
+    /// This is a sound_handler::aux_streamer_ptr type.
+    ///
+    /// It might be invoked by a separate thread (neither main,
+    /// nor decoder thread).
+    ///
+    static unsigned int audio_streamer(void *udata, boost::int16_t* samples,
+            unsigned int nSamples, bool& eof);
+
 protected:
     
     /// Status codes used for notifications
@@ -205,7 +354,7 @@ protected:
         invalidTime
     };
 
-    boost::intrusive_ptr<NetConnection_as> _netCon;
+    NetConnection_as* _netCon;
 
     boost::scoped_ptr<CharacterProxy> _audioController;
 
@@ -241,9 +390,6 @@ protected:
     ///
     void processStatusNotifications();
     
-    
-    void processNotify(const std::string& funcname, as_object* metadata_obj);
-
     // The size of the buffer in milliseconds
     boost::uint32_t m_bufferTime;
 
@@ -264,174 +410,19 @@ protected:
 
     // Are we playing a FLV?
     // The handler which is invoked on status change
-    boost::intrusive_ptr<as_function> m_statusHandler;
+    boost::intrusive_ptr<as_function> _statusHandler;
 
     // The position in the inputfile, only used when not playing a FLV
     long inputPos;
 
-#ifdef GNASH_USE_GC
-    /// Mark all reachable resources of a NetStream_as, for the GC
-    //
-    /// Reachable resources are:
-    /// - associated NetConnection object (_netCon)
-    /// - character to invalidate on video updates (_invalidatedVideoCharacter)
-    /// - onStatus event handler (m_statusHandler)
-    ///
-    virtual void markReachableResources() const;
-#endif // GNASH_USE_GC
-
     /// Unplug the advance timer callback
     void stopAdvanceTimer();
 
     /// Register the advance timer callback
     void startAdvanceTimer();
 
-    /// The character to invalidate on video updates
-    character* _invalidatedVideoCharacter;
-
-public:
-
-    enum PauseMode {
-      pauseModeToggle = -1,
-      pauseModePause = 0,
-      pauseModeUnPause = 1  
-    };
-
-    NetStream_as();
-
-    ~NetStream_as();
-
-    PlayHead::PlaybackStatus playbackState() const {
-        return _playHead.getState();
-    }
-
-    /// Get the real height of the video in pixels if the decoder exists.
-    //
-    /// @return the height of the video in pixels or 0 if no decoder exists.
-    ///         The width returned from the decoder may also vary, and will
-    ///         be 0 until it knows the width.
-    int videoHeight() const;
-
-    /// Get the real width of the video in pixels if the decoder exists.
-    //
-    /// @return the width of the video in pixels or 0 if no decoder exists.
-    ///         The width returned from the decoder may also vary, and will
-    ///         be 0 until it knows the width.
-    int videoWidth() const;
-
-    /// Closes the video session and frees all ressources used for decoding
-    /// except the FLV-parser (this might not be correct).
-    void close();
-
-    /// Make audio controlled by given character
-    void setAudioController(character* controller);
- 
-    /// Pauses/starts the playback of the media played by the current instance
-    //
-    /// @param mode
-    /// Defines what mode to put the instance in.
-    void pause(PauseMode mode);
-
-    /// Starts the playback of the media
-    //
-    /// @param source
-    /// Defines what file to play
-    ///
-    void play(const std::string& source);
-
-    /// Seek in the media played by the current instance
-    //
-    /// @param pos
-    ///     Defines in seconds where to seek to
-    ///     @todo take milliseconds !!
-    ///
-    void seek(boost::uint32_t pos);
-
-    /// Tells where the playhead currently is
-    //
-    /// @return The time in milliseconds of the current playhead position
-    ///
-    boost::int32_t time();
-
-    /// Called at the SWF framerate. Used to process queued status messages
-    /// and (re)start after a buffering pause. In NetStreamFfmpeg it is also
-    /// used to find the next video frame to be shown, though this might change.
-    void advance();
-    
-    /// Returns the current framerate in frames per second.
-    double getCurrentFPS()  { return 0; }
-    
-
-    /// Sets the NetConnection needed to access external files
-    //
-    /// @param nc
-    ///     The NetConnection object to use for network access
-    ///
-    void setNetCon(boost::intrusive_ptr<NetConnection_as> nc) {
-        _netCon = nc;
-    }
-
-    /// Return true if the NetStream has an associated NetConnection
-    bool isConnected() const { return (_netCon); }
-
-    /// Specifies the number of milliseconds to buffer before starting to display the stream.
-    //
-    /// @param time
-    /// The time in milliseconds that should be buffered.
-    ///
-    void setBufferTime(boost::uint32_t time);
-
-    /// Returns what the buffer time has been set to. (100 miliseconds is default)
-    //
-    /// @return The size of the buffer in milliseconds.
-    ///
-    boost::uint32_t bufferTime() { return m_bufferTime; }
-
-    /// Returns the number of bytes of the media file that have been buffered.
-    long bytesLoaded();
-
-    /// Returns the total number of bytes (size) of the media file
-    //
-    /// @return the total number of bytes (size) of the media file
-    ///
-    long bytesTotal();
-
-    /// Returns the number of millisecond of the media file that is buffered and 
-    /// yet to be played
-    //
-    /// @return Returns the number of millisecond of the media file that is 
-    /// buffered and yet to be played
-    ///
-    long bufferLength();
-
-    /// Tells us if there is a new video frame ready
-    //
-    /// @return true if a frame is ready, false if not
-    bool newFrameReady();
-
-    /// Returns the video frame closest to current cursor. See time().
-    //
-    /// @return a image containing the video frame, a NULL auto_ptr if none were ready
-    ///
-    std::auto_ptr<GnashImage> get_video();
-    
-    /// Register the character to invalidate on video updates
-    void setInvalidatedVideo(character* ch)
-    {
-        _invalidatedVideoCharacter = ch;
-    }
-
-
-
-    /// Callback used by sound_handler to get audio data
-    //
-    /// This is a sound_handler::aux_streamer_ptr type.
-    ///
-    /// It might be invoked by a separate thread (neither main, nor decoder thread).
-    ///
-    static unsigned int audio_streamer(void *udata, boost::int16_t* samples,
-            unsigned int nSamples, bool& eof);
-
+    /// The DisplayObject to invalidate on video updates
+    DisplayObject* _invalidatedVideoCharacter;
 
 private:
 
@@ -442,6 +433,18 @@ private:
         DEC_BUFFERING
     };
 
+    typedef std::pair<std::string, std::string> NetStreamStatus;
+
+    /// Get 'status' (first) and 'level' (second) strings for given status code
+    //
+    /// Any invalid code, out of bound or explicitly invalid (invalidCode) 
+    /// returns two empty strings.
+    ///
+    void getStatusCodeInfo(StatusCode code, NetStreamStatus& info);
+
+    /// Return a newly allocated information object for the given status
+    as_object* getStatusObject(StatusCode code);
+
     /// Initialize video decoder and (if successful) PlayHead consumer 
     //
     /// @param info Video codec information
@@ -454,12 +457,6 @@ private:
     ///
     void initAudioDecoder(const media::AudioInfo& parser);
 
-    DecodingState _decoding_state;
-
-    // Mutex protecting _playback_state and _decoding_state
-    // (not sure a single one is appropriate)
-    boost::mutex _state_mutex;
-
     // Setups the playback
     bool startPlayback();
 
@@ -484,7 +481,7 @@ private:
 
     /// Update the image/videoframe to be returned by next get_video() call.
     //
-    /// Uses by ::advance().
+    /// Used by update().
     ///
     /// Note that get_video will be called by Video::display(), which
     /// is usually called right after Video::advance(), so the result
@@ -496,7 +493,7 @@ private:
     /// If true, video is consumed/refreshed even if playhead is paused.
     /// By default this is false, but will be used on ::seek (user-reguested)
     ///
-    void refreshVideoFrame(bool alsoIfPaused=false);
+    void refreshVideoFrame(bool alsoIfPaused = false);
 
     /// Refill audio buffers, so to contain new frames since last run
     /// and up to current timestamp
@@ -537,6 +534,17 @@ private:
 
     DecodingState decodingStatus(DecodingState newstate = DEC_NONE);
 
+    /// Parse a chunk of input
+    /// Currently blocks, ideally should parse as much
+    /// as possible w/out blocking
+    void parseNextChunk();
+
+    DecodingState _decoding_state;
+
+    // Mutex protecting _playback_state and _decoding_state
+    // (not sure a single one is appropriate)
+    boost::mutex _state_mutex;
+    
     /// Video decoder
     std::auto_ptr<media::VideoDecoder> _videoDecoder;
 
@@ -550,7 +558,7 @@ private:
     bool _audioInfoKnown;
 
     /// Virtual clock used as playback clock source
-    std::auto_ptr<InterruptableVirtualClock> _playbackClock;
+    boost::scoped_ptr<InterruptableVirtualClock> _playbackClock;
 
     /// Playback control device 
     PlayHead _playHead;
@@ -561,11 +569,6 @@ private:
     // Current media handler
     media::MediaHandler* _mediaHandler;
 
-    /// Parse a chunk of input
-    /// Currently blocks, ideally should parse as much
-    /// as possible w/out blocking
-    void parseNextChunk();
-
     /// Input stream
     //
     /// This should just be a temporary variable, transferred
@@ -576,58 +579,19 @@ private:
     /// The buffered audio streamer
     BufferedAudioStreamer _audioStreamer;
 
-    /// Pop next queued status notification from the queue
-    //
-    /// Lock the statusMutex during operations
-    ///
-    /// @return The status code to notify, or invalidStatus when
-    ///     the queue is empty
-    ///
-    StatusCode popNextPendingStatusNotification();
-
-    /// Clear status notification queue
-    //
-    /// Lock the statusMutex during operations
-    ///
-    void clearStatusQueue();
-
-    // Queue of status notifications.
-    typedef std::deque<StatusCode> StatusQueue;
-
     /// List of status messages to be processed
-    StatusQueue _statusQueue;
+    StatusCode _statusCode;
 
     /// Mutex protecting _statusQueue
     boost::mutex statusMutex;
 
-    /// Last status code (to avoid consecutively notifying the same event)
-    StatusCode _lastStatus;
-
-    typedef std::pair<std::string, std::string> NetStreamStatus;
-
-    /// Get 'status' (first) and 'level' (second) strings for given status code
-    //
-    /// Any invalid code, out of bound or explicitly invalid (invalidCode) 
-    /// returns two empty strings.
-    ///
-    void getStatusCodeInfo(StatusCode code, NetStreamStatus& info);
-
-    /// Return a newly allocated information object for the given status
-    as_object* getStatusObject(StatusCode code);
-
-    /// hack for using a Timer to drive ::advance calls
-    static as_value advanceWrapper(const fn_call& fn);
-
-    /// Identifier of the advance timer
-    unsigned int _advanceTimer;
 };
 
+void netstream_class_init(as_object& global, const ObjectURI& uri);
 
-// Initialize the global NetStream class
-void netstream_class_init(as_object& global);
+void registerNetStreamNative(as_object& global);
 
-} // end of gnash namespace
+} // gnash namespace
 
-// __NETSTREAM_H__
 #endif
 
