@@ -1,46 +1,52 @@
-// 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
-// 
+// XML_as.h:  ActionScript 3 "XMLDocument" class, for Gnash.
+//
+//   Copyright (C) 2009, 2010 Free Software Foundation, Inc.
+//
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
-// 
+//
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+//
 
-#ifndef GNASH_ASOBJ_XML_H
-#define GNASH_ASOBJ_XML_H
+#ifndef GNASH_ASOBJ3_XMLDOCUMENT_H
+#define GNASH_ASOBJ3_XMLDOCUMENT_H
 
-#include "LoadableObject.h"
 #include "XMLNode_as.h"
-#include "log.h"
 #include "dsodefs.h"
 #include "StringPredicates.h"
 
 #include <map>
 #include <string>
 
-//#define DEBUG_MEMORY_ALLOCATION 1
 
 namespace gnash {
 
 // Forward declarations
 class fn_call;
 class URL;
-class LoaderThread;
 
-/// XML class and ActionScript object
-class XML_as : public XMLNode_as, public LoadableObject
+/// Implements XML (AS2) and flash.xml.XMLDocument (AS3) class.
+//
+/// This class interface is identical in AS3 and AS2; it is probably 
+/// included in AS3 for backward compatibility.
+//
+/// The class definition is necessary because XML is encoded differently
+/// in AMF.
+class XML_as : public XMLNode_as
 {
 public:
 
+    typedef std::string::const_iterator xml_iterator;
+
     enum ParseStatus {
             XML_OK = 0,
             XML_UNTERMINATED_CDATA = -2,
@@ -54,15 +60,22 @@ public:
             XML_MISSING_OPEN_TAG = -10
     };
 
-    XML_as();
+    enum LoadStatus {
+        XML_LOADED_UNDEFINED = -1,
+        XML_LOADED_FALSE = false,
+        XML_LOADED_TRUE = true
+    };
 
-    XML_as(const std::string& xml);
+    /// Create an XML object.
+    //
+    /// An XMLDocument is always user-created, so always starts with an
+    /// associated object.
+    XML_as(as_object& object);
+
+    XML_as(as_object& object, const std::string& xml);
 
     ~XML_as() {};
     
-    static void init(as_object& global);
-    static void registerNative(as_object& global);
-
     /// Convert the XML object to a string
     //
     /// This calls XMLNode::toString after adding an xmlDecl and
@@ -88,20 +101,6 @@ public:
         _docTypeDecl = docType;
     }
 
-    /// This is overridden to provide the 'status' and 'loaded' members,
-    /// which are NOT proper properties !
-    /// See actionscript.all/XML.as
-    ///
-    bool get_member(string_table::key name, as_value *val,
-		string_table::key nsname = 0);
-
-    /// This is overridden to provide the 'status' and 'loaded' members,
-    /// which are NOT proper properties !
-    /// See actionscript.all/XML.as
-    ///
-    bool set_member(string_table::key name, const as_value& val,
-                string_table::key nsname = 0, bool ifFound=false);
-
     // Methods
 
     /// Parses an XML document into the specified XML object tree.
@@ -113,64 +112,60 @@ public:
     ///
     void parseXML(const std::string& xml);
 
-    // An event handler that returns a what?
-    bool onLoad();
-
-    /// Escape using XML entities.
-    //
-    /// Note this is not the same as a URL escape.
-    static void escape(std::string& text);
-
-    /// Unescape XML entities.
-    //
-    /// Note this is not the same as a URL unescape.
-    static void unescape(std::string& text);
-
     XMLNode_as* createElement(const std::string& name);
 
     XMLNode_as* createTextNode(const std::string& name);
 
+    ParseStatus status() const {
+        return _status;
+    }
+
+    void setStatus(ParseStatus st) {
+        _status = st;
+    }
+
+    LoadStatus loaded() const {
+        return _loaded;
+    }
+
+    void setLoaded(LoadStatus st) {
+        _loaded = st;
+    }
+
 private:
 
-    typedef std::map<std::string, std::string> Entities;
-
-    static const Entities& getEntities();
-
     typedef std::map<std::string, std::string, StringNoCaseLessThan> Attributes;
 
-    void parseTag(XMLNode_as*& node, const std::string& xml, 
-            std::string::const_iterator& it);
+    void parseTag(XMLNode_as*& node, xml_iterator& it, xml_iterator end);
 
-    void parseAttribute(XMLNode_as* node, const std::string& xml, 
-            std::string::const_iterator& it, Attributes& attributes);
+    void parseAttribute(XMLNode_as* node, xml_iterator& it,
+            xml_iterator end, Attributes& attributes);
 
-    void parseDocTypeDecl(const std::string& xml, 
-            std::string::const_iterator& it);
+    void parseDocTypeDecl( xml_iterator& it, xml_iterator end);
 
-    void parseText(XMLNode_as* node, const std::string& xml, 
-            std::string::const_iterator& it);
+    void parseText(XMLNode_as* node, xml_iterator& it, xml_iterator end);
 
-    void parseXMLDecl(const std::string& xml, 
-            std::string::const_iterator& it);
+    void parseXMLDecl(xml_iterator& it, xml_iterator end);
 
-    void parseComment(XMLNode_as* node, const std::string& xml, 
-            std::string::const_iterator& it);
+    void parseComment(XMLNode_as* node, xml_iterator& it, xml_iterator end);
 
-    void parseCData(XMLNode_as* node, const std::string& xml, 
-            std::string::const_iterator& it);
+    void parseCData(XMLNode_as* node, xml_iterator& it, xml_iterator end);
  
-    /// Remove all children
+    /// Clear all properties.
+    //
+    /// This removes all children, resets doctype and xml decls, and
+    /// sets status to XML.
     void clear();
   
     /// \brief
     /// Return true if ignoreWhite property was set to anything evaluating
     /// to true.
-    bool ignoreWhite() const;
+    bool ignoreWhite();
 
     // -1 if never asked to load anything
     //  0 if asked to load but not yet loaded (or failure)
     //  1 if successfully loaded
-    int _loaded;
+    LoadStatus _loaded;
 
     ParseStatus _status;	
  
@@ -180,12 +175,24 @@ private:
 
 };
 
-}	// namespace gnash
 
+/// Escape using XML entities.
+//
+/// Note this is not the same as a URL escape.
+void escapeXML(std::string& text);
+void unescapeXML(std::string& text);
+
+/// Register the XML class.
+void xml_class_init(as_object& where, const ObjectURI& uri);
+
+/// Register XML native functions.
+void registerXMLNative(as_object& where);
+
+}	// namespace gnash
 #endif
 
-
-// Local Variables:
+// local Variables:
 // mode: C++
 // indent-tabs-mode: t
 // End:
+
