@@ -1,6 +1,7 @@
 // ExternalInterface_as.h:  ActionScript "ExternalInterface" class, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,26 +18,32 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-#ifndef __GNASH_ASOBJ_EXTERNALINTERFACE_H__
-#define __GNASH_ASOBJ_EXTERNALINTERFACE_H__
+#ifndef GNASH_ASOBJ_EXTERNALINTERFACE_H
+#define GNASH_ASOBJ_EXTERNALINTERFACE_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
-//#include <memory> // for auto_ptr
 
 namespace gnash {
 
 class as_object;
+class as_value;
+struct ObjectURI;
+class Global_as;
+class movie_root;
+}
+
+namespace gnash {
 
 /// Initialize the global ExternalInterface class
-void ExternalInterface_class_init(as_object& global);
+void externalinterface_class_init(gnash::as_object& where,
+                                  const gnash::ObjectURI& uri);
 
-/// Return a ExternalInterface instance (in case the core lib needs it)
-//std::auto_ptr<as_object> init_ExternalInterface_instance();
+void registerExternalInterfaceNative(as_object& global);
 
 } // end of gnash namespace
 
-// __GNASH_ASOBJ_EXTERNALINTERFACE_H__
 #endif
+
+// local Variables:
+// mode: C++
+// indent-tabs-mode: nil
+// End:
