@@ -1,6 +1,7 @@
 // Transform_as.h:  ActionScript "Transform" class, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,14 +21,11 @@
 #ifndef GNASH_ASOBJ_TRANSFORM_H
 #define GNASH_ASOBJ_TRANSFORM_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
 
 namespace gnash {
 
 class as_object;
-class ObjectURI;
+struct ObjectURI;
 
 /// Initialize the global Transform class
 void transform_class_init(as_object& where, const ObjectURI& uri);
