@@ -58,9 +58,7 @@
 ///
 /// - bool movie_root::notify_mouse_moved(int x, int y);
 /// - bool movie_root::notify_mouse_clicked(bool mouse_pressed, int mask);
-/// - bool movie_root::notify_key_event(key::code k, bool down);
-/// 
-/// 
+/// - bool keyEvent(key::code k, bool down);
 
 
 #ifndef GNASH_MOVIE_ROOT_H
@@ -74,15 +72,13 @@
 #include "dsodefs.h" // DSOEXPORT
 #include "MouseButtonState.h" // for composition
 #include "drag_state.h" // for composition
-#include "smart_ptr.h" // for memory management
-#include "URL.h" // for loadMovie
 #include "GnashKey.h" // key::code
 #include "Movie.h"
-#include "RunResources.h" // for initialization
 #include "gnash.h" // Quality
 #include "MovieClip.h"
 #include "SimpleBuffer.h" // for LoadCallback
 #include "MovieLoader.h"
+#include "ExternalInterface.h"
 
 #ifdef USE_SWFTREE
 # include "tree.hh"
@@ -97,6 +93,8 @@
 # define GNASH_PARANOIA_LEVEL 1
 #endif
 
+#include <map>
+#include <string>
 #include <vector>
 #include <deque>
 #include <list>
@@ -113,6 +111,8 @@ namespace gnash {
     class MovieClip;
     class VirtualClock;
     class IOChannel;
+    class RunResources;
+    class Button;
 }
 
 namespace gnash
@@ -146,8 +146,10 @@ struct DepthComparator
 /// The _root object is provided by getAsRoot().
 class DSOEXPORT movie_root : boost::noncopyable
 {
-
 public:
+    
+    /// Listeners container
+    typedef std::list<Button*> Listeners;
 
     class LoadCallback {
     public:
@@ -163,8 +165,7 @@ public:
         SimpleBuffer _buf;
         as_object* _obj;
     };
-        
-    typedef std::list<LoadCallback> LoadCallbacks;
+    typedef std::list<LoadCallback> LoadCallbacks;        
 
     typedef std::bitset<key::KEYCOUNT> Keys;
 
@@ -188,7 +189,8 @@ public:
     /// SWF playback, so for normal playback this pointer should not be
     /// used.
     Movie* init(movie_definition* def,
-            const MovieClip::MovieVariables& variables);
+            const MovieClip::MovieVariables& variables,
+            const MovieClip::MovieVariables& scriptables);
 
     /// Return the movie at the given level (0 if unloaded level).
     //
@@ -243,73 +245,58 @@ public:
     ///   after loading into it.
     void dropLevel(int depth);
 
-    /// Change display viewport coordinates
+    /// Change stage size
     //
-    /// This currently also change the display scale
-    /// but we should instead only do it if rescaling
-    /// is allowed.
-    ///
-    void set_display_viewport(int x0, int y0, int w, int h);
+    /// This may be smaller than the size of the root movie. It determines
+    /// how much of the movie is visible.
+    //
+    /// @param w    The width of the stage
+    /// @param h    The height of the stage.
+    void setDimensions(size_t w, size_t h);
 
-    /// \brief
-    /// Return the notional width of the stage, value depending
-    /// on scaleMode
-    unsigned getStageWidth() const;
+    /// Notional width of the stage, actual value depending on scaleMode
+    size_t getStageWidth() const;
 
-    /// \brief
-    /// Return the notional height of the stage, actual value depending
-    /// on scaleMode
-    unsigned getStageHeight() const;
+    /// Notional height of the stage, actual value depending on scaleMode
+    size_t getStageHeight() const;
 
-    /// \brief
-    /// The host app can use this to tell the movie when
-    /// user's mouse pointer has moved.
+    /// Inform the Stage that the mouse has moved.
     //
     /// Coordinates are in Stage Coordinate Space (pseudo-pixels units).
     ///
-    /// This function should return TRUE iff any action triggered
-    /// by the event requires redraw, see \ref events_handling for
-    /// more info.
+    /// @param x    The x co-ordinate in pixels.
+    /// @param y    The y co-ordinate in pixels.
+    /// @return     true if any action triggered requires a redraw.
     ///
     /// TODO: take twips (or float pixels), or we won't be able to
     ///       support sub-pixel accuracy in collision detection.
-    ///
-    bool notify_mouse_moved(int x, int y);
+    DSOEXPORT bool mouseMoved(boost::int32_t x, boost::int32_t y);
 
-    /// \brief
-    /// The host app can use this to tell the movie when the
-    /// user clicked or released the mouse button.
+    /// Inform the Stage that a mouse click has occurred.
     //
-    /// @param mouse_pressed
-    /// true if the mouse has been pressed, false if released
-    ///
-    /// @param mask
-    /// ???
-    ///
-    /// This function should return TRUE iff any action triggered
+    /// @param press    true for a mouse click, false for a release
+    /// @return         true if any action triggered requires a redraw.
+    DSOEXPORT bool mouseClick(bool press);
+
+    /// Inform the Stage that a mouse wheel has moved.
+    //
+    /// @param delta    The direction of the scroll: positive for up, negative
+    ///                 for down. Although values from about -3 to 3 are
+    ///                 documented, only -1 and 1 have been observed.
+    /// @return         true if any action triggered requires a redraw.
+    DSOEXPORT bool mouseWheel(int delta);
+
+    /// Tell the movie when the user pressed or released a key.
+    //
+    /// This function should return TRUE if any action triggered
     /// by the event requires redraw, see \ref events_handling for
     /// more info.
-    ///
-    bool notify_mouse_clicked(bool mouse_pressed, int mask);
+    DSOEXPORT bool keyEvent(key::code k, bool down);
 
-    /// \brief
-    /// The host app can use this to tell the movie when
-    /// user pressed or released a key.
-    //
-    /// This function should return TRUE iff any action triggered
-    /// by the event requires redraw, see \ref events_handling for
-    /// more info.
-    ///
-    bool notify_key_event(key::code k, bool down);
-
-    /// \brief
-    /// Use this to retrieve the last state of the mouse, as set via
-    /// notify_mouse_state(). 
+    /// Use this to retrieve the last state of the mouse.
     //
     /// Coordinates are in PIXELS, NOT TWIPS.
-    ///
-    void get_mouse_state(boost::int32_t& x, boost::int32_t& y,
-            boost::int32_t& buttons);
+    void get_mouse_state(boost::int32_t& x, boost::int32_t& y);
 
     void get_drag_state(drag_state& st);
 
@@ -452,35 +439,11 @@ public:
     /// Get a unique number for unnamed instances.
     size_t nextUnnamedInstance();
 
-    /// Notify still loaded DisplayObject listeners for key events
-    DSOEXPORT void notify_key_listeners(key::code k, bool down);
-
     /// Push a new DisplayObject listener for key events
-    void add_key_listener(DisplayObject* listener)
-    {
-        add_listener(_keyListeners, listener);
-    }
+    void add_key_listener(Button* listener);
 
     /// Remove a DisplayObject listener for key events
-    void remove_key_listener(DisplayObject* listener)
-    {
-        remove_listener(_keyListeners, listener);
-    }
-
-    /// Notify still loaded DisplayObject listeners for mouse events
-    DSOEXPORT void notify_mouse_listeners(const event_id& event);
-
-    /// Push a new DisplayObject listener for mouse events
-    void add_mouse_listener(DisplayObject* listener)
-    {
-        add_listener(_mouseListeners, listener);
-    }
-
-    /// Remove a DisplayObject listener for mouse events
-    void remove_mouse_listener(DisplayObject* listener)
-    {
-        remove_listener(_mouseListeners, listener);
-    }
+    void remove_key_listener(Button* listener);
 
     /// Get the DisplayObject having focus
     //
@@ -523,9 +486,6 @@ public:
     /// Return the DisplayObject currently being dragged, if any
     DisplayObject* getDraggingCharacter() const;
 
-    /// Return true if the mouse pointer is over an active entity
-    bool isMouseOverActiveEntity() const;
-
     bool testInvariant() const;
 
     /// The possible values of Stage.displayState
@@ -536,10 +496,10 @@ public:
 
     /// The possibile values of Stage.scaleMode
     enum ScaleMode {
-        showAll,
-        noScale,
-        exactFit,
-        noBorder
+        SCALEMODE_SHOWALL,
+        SCALEMODE_NOSCALE,
+        SCALEMODE_EXACTFIT,
+        SCALEMODE_NOBORDER
     };
 
     /// The possible horizonal positions of the Stage
@@ -564,6 +524,13 @@ public:
         STAGE_ALIGN_B
     };
 
+    /// The possibile values of AllowScriptAccess
+    enum AllowScriptAccessMode {
+        SCRIPT_ACCESS_NEVER,
+        SCRIPT_ACCESS_SAME_DOMAIN,
+        SCRIPT_ACCESS_ALWAYS
+    };
+
     /// Set the current display quality of the entire SWF.
     void setQuality(Quality q);
 
@@ -574,20 +541,32 @@ public:
     /// of the three possible positions for each dimension.
     void setStageAlignment(short s);
 
+    /// Sets the flag to allow interfacing with JavaScript in the browser.
+    /// This is disabled by default, but enabled for ExternalInterface.
+    void setAllowScriptAccess(AllowScriptAccessMode mode);
+    
+    /// Gets the current Access Mode for ExternalInterface.
+    AllowScriptAccessMode getAllowScriptAccess();
+
     typedef std::pair<StageHorizontalAlign, StageVerticalAlign> StageAlign;
 
     /// Returns the current alignment of the stage (left/right/centre, top/
     /// bottom/centre) as a std::pair
     StageAlign getStageAlignment() const;
 
-	/// Returns the current value of _showMenu which instructs the gui about
-	/// how much to display in the context menu
-	bool getShowMenuState() const;
-
-	/// Sets the value of _showMenu and calls the fscommand handler for the
-	/// current gui
-	void setShowMenuState( bool state );
+    /// Returns the current value of _showMenu which instructs the gui about
+    /// how much to display in the context menu
+    bool getShowMenuState() const;
+    
+    /// Sets the value of _showMenu and calls the fscommand handler for the
+    /// current gui
+    void setShowMenuState(bool state);
 
+    // This is a flag that specifies whether exceptions in ActionScript
+    // should be propogated to JavaScript in the browser.
+    void setMarshallExceptions(bool x) { _marshallExceptions = x; };
+    bool getMarshallExceptions() { return _marshallExceptions; };
+    
     /// Sets the Stage object's align mode.
     void setStageScaleMode(ScaleMode sm);
     
@@ -605,34 +584,21 @@ public:
 
     /// Action priority levels
     enum ActionPriorityLevel {
-
         /// Init actions, Init event handlers
-        apINIT=0,
-
+        PRIORITY_INIT,
         /// Construct event handlers
-        apCONSTRUCT=1,
-
-        /// EnterFrame event handlers
-        apENTERFRAME=2,
-
+        PRIORITY_CONSTRUCT,
         /// Frame actions, load handlers, unload handlers
-        apDOACTION=3,
-
+        PRIORITY_DOACTION,
         /// Last element used to easy computation of size...
-        apSIZE
-        
+        PRIORITY_SIZE
     };
 
     /// Push an executable code to the ActionQueue
-    void pushAction(std::auto_ptr<ExecutableCode> code, int lvl=apDOACTION);
+    void pushAction(std::auto_ptr<ExecutableCode> code, size_t lvl);
 
     /// Push an executable code to the ActionQueue
-    void pushAction(const action_buffer& buf, DisplayObject* target,
-            int lvl=apDOACTION);
-
-    /// Push a function code to the ActionQueue
-    void pushAction(as_function* func, DisplayObject* target,
-            int lvl=apDOACTION);
+    void pushAction(const action_buffer& buf, DisplayObject* target);
 
 #ifdef GNASH_USE_GC
     /// Mark all reachable resources (for GC)
@@ -645,7 +611,6 @@ public:
     /// - Timer targets (_intervalTimers)
     /// - Resources reachable by ActionQueue code (_actionQueue)
     /// - Key listeners (_keyListeners)
-    /// - Mouse listeners (_mouseListeners)
     /// - Any DisplayObject being dragged 
     ///
     void markReachableResources() const;
@@ -659,7 +624,7 @@ public:
     /// its turn comes. Characters are advanced in reverse-placement
     /// order (first registered is advanced last)
     ///
-    void addLiveChar(DisplayObject* ch)
+    void addLiveChar(MovieClip* ch)
     {
         // Don't register the object in the list twice 
 #if GNASH_PARANOIA_LEVEL > 1
@@ -760,10 +725,17 @@ public:
     /// (for browser communication mostly)
     void setHostFD(int fd)
     {
-        assert(fd > 0);
+        assert(fd >= 0);
         _hostfd = fd;
     }
 
+    /// Set a filedescriptor to use for host application requests
+    /// (for browser communication mostly)
+    void setControlFD(int fd)
+    {
+        _controlfd = fd;
+    }
+
     /// Get the filedescriptor to use for host application requests
     /// (for browser communication mostly)
     ///
@@ -773,6 +745,12 @@ public:
         return _hostfd;
     }
 
+    int getControlFD() const
+    {
+        return _controlfd;
+    }
+
+
     /// Abstract base class for FS handlers
     class AbstractFsCallback {
     public:
@@ -797,7 +775,7 @@ public:
     /// Call this to notify FS commands
     DSOEXPORT void handleFsCommand(const std::string& cmd,
             const std::string& arg) const;
-
+    
     /// Abstract base class for hosting app handler
     class AbstractIfaceCallback
     {
@@ -815,6 +793,9 @@ public:
         /// a question.
         virtual bool yesNo(const std::string& cmd) = 0;
 
+        /// Instruct the hosting application to exit.
+        virtual void exit() = 0;
+
         /// Send an error message to the hosting application.
         //
         /// This does not have to be implemented; the default is a no-op.
@@ -885,14 +866,33 @@ public:
     void getCharacterTree(tree<StringPair>& tr, tree<StringPair>::iterator it);
 #endif
 
-	/// Get URL of the SWF movie used to initialize this VM
-	//
-	/// This information will be used for security checks
-	///
-	const std::string& getOriginalURL() const { return _originalURL; }
+    /// Get URL of the SWF movie used to initialize this VM
+    //
+    /// This information will be used for security checks
+    ///
+    const std::string& getOriginalURL() const { return _originalURL; }
 
     const RunResources& runResources() const { return _runResources; }
 
+    void addExternalCallback(as_object *obj, const std::string &name,
+                             as_object *callback);
+
+    bool processInvoke(ExternalInterface::invoke_t *);
+
+    std::string callExternalCallback(const std::string &name, 
+                                     const std::vector<as_value>& args);
+    
+    std::string callExternalJavascript(const std::string &name, 
+                                       const std::vector<as_value>& args);
+
+    /// Removes a queued constructor from the execution queue
+    //
+    /// This is used to prevent construction of targets that are placed and
+    /// then removed in skipped frames. Callers are responsible for determining
+    /// whether it should be removed, for instance by checking for an 
+    /// onUnload handler.
+    void removeQueuedConstructor(DisplayObject* target);
+
 private:
 
     /// Set the root movie, replacing the current one if any.
@@ -920,6 +920,14 @@ private:
     ///
     void setRootMovie(Movie* movie);
 
+    /// Handle mouse events.
+    bool notify_mouse_listeners(const event_id& event);
+    
+    /// This function should return TRUE iff any action triggered
+    /// by the event requires redraw, see \ref events_handling for
+    /// more info.
+    bool fire_mouse_event();
+
     const RunResources& _runResources; 
 
     /// The URL of the original root movie.
@@ -938,9 +946,6 @@ private:
     /// Registered FsCommand handler, if any
     AbstractFsCallback* _fsCommandHandler;
 
-    /// Listeners container
-    typedef std::list<DisplayObject*> Listeners;
-
     /// Take care of dragging, if needed
     void doMouseDrag();
 
@@ -956,31 +961,9 @@ private:
     /// Execute expired timers
     void executeTimers();
 
-    /// Remove unloaded key and mouselisteners.
-    void cleanupUnloadedListeners()
-    {
-        cleanupUnloadedListeners(_keyListeners);
-        cleanupUnloadedListeners(_mouseListeners);
-    }
-
-    /// Erase unloaded DisplayObjects from the given listeners list
-    static void cleanupUnloadedListeners(Listeners& ll);
-
     /// Cleanup references to unloaded DisplayObjects and run the GC.
     void cleanupAndCollect();
 
-    /// Push a DisplayObject listener to the front of given container, if not
-    /// already present
-    static void add_listener(Listeners& ll, DisplayObject* elem);
-
-    /// Remove a listener from the list
-    static void remove_listener(Listeners& ll, DisplayObject* elem);
-
-    /// This function should return TRUE iff any action triggered
-    /// by the event requires redraw, see \ref events_handling for
-    /// more info.
-    bool fire_mouse_event();
-
     /// \brief
     /// Return the topmost entity covering the given point
     /// and enabled to receive mouse events.
@@ -1003,13 +986,6 @@ private:
     /// from the display lists
     void cleanupDisplayList();
 
-    /// Advance a live DisplayObject
-    //
-    /// @param ch
-    ///     The DisplayObject to advance, will NOT be advanced if unloaded
-    ///
-    static void advanceLiveChar(DisplayObject* ch);
-
     /// Advance all non-unloaded live chars
     void advanceLiveChars();
 
@@ -1027,7 +1003,7 @@ private:
     /// An invalidated stage will trigger complete redraw
     //
     /// So, this method should return true everytime a complete
-    /// redraw is needed. This is tipically only needed when
+    /// redraw is needed. This is typically only needed when
     /// the background changes.
     ///
     /// See setInvalidated() and clearInvalidated().
@@ -1038,55 +1014,54 @@ private:
     //
     /// Scanned in proprity order (lower first)
     ///
-    int minPopulatedPriorityQueue() const;
+    size_t minPopulatedPriorityQueue() const;
 
     /// Process all actions in the the given queue, till more actions
     /// are found in lower levels, in which case we have an earlier
     /// return.
-    int processActionQueue(int lvl);
+    size_t processActionQueue(size_t lvl);
 
     bool processingActions() const
     {
-        return (_processingActionLevel < apSIZE);
+        return (_processingActionLevel < PRIORITY_SIZE);
     }
 
     const DisplayObject* findDropTarget(boost::int32_t x, boost::int32_t y,
             DisplayObject* dragging) const;
 
     void handleActionLimitHit(const std::string& ref);
+
     /// A list of AdvanceableCharacters
     //
     /// This is a list (not a vector) as we want to allow
     /// ::advance of each element to insert new DisplayObjects before
     /// the start w/out invalidating iterators scanning the
     /// list forward for proper movie advancement
-    typedef std::list<DisplayObject*> LiveChars;
+    typedef std::list<MovieClip*> LiveChars;
 
     /// The list of advanceable DisplayObject, in placement order
     LiveChars _liveChars;
 
-
     /// A number of queues of code to execute
     //
     /// This is a deque because it needs no insertion in the middle but
     /// frequent push_back and pop_front. We also have to traverse it, so
     /// a std::queue is not usable.
     typedef std::deque<ExecutableCode*> ActionQueue;
-    ActionQueue _actionQueue[apSIZE];
+    ActionQueue _actionQueue[PRIORITY_SIZE];
 
     /// Process all actions in the queue
     void processActionQueue();
 
-    int m_viewport_x0, m_viewport_y0;
-
     /// Width and height of viewport, in pixels
-    int m_viewport_width, m_viewport_height;
+    size_t _stageWidth;
+    size_t _stageHeight;
 
     rgba m_background_color;
     bool m_background_color_set;
 
-    float m_timer;
-    int m_mouse_x, m_mouse_y, m_mouse_buttons;
+    boost::int32_t _mouseX;
+    boost::int32_t _mouseY;
 
     MouseButtonState  _mouseButtonState;
 
@@ -1095,9 +1070,8 @@ private:
     ObjectCallbacks _objectCallbacks;
 
     LoadCallbacks _loadCallbacks;
-
+    
     typedef std::map<int, Timer*> TimerMap;
-
     TimerMap _intervalTimers;
     unsigned int _lastTimerId;
 
@@ -1109,17 +1083,13 @@ private:
     /// Characters for listening key events
     Listeners _keyListeners;
 
-    /// Objects listening for mouse events (down,up,move)
-    Listeners _mouseListeners;
-
     /// The DisplayObject currently holding focus, or 0 if no focus.
     DisplayObject* _currentFocus;
 
     /// @todo fold this into m_mouse_button_state?
     drag_state m_drag_state;
 
-    typedef MovieClip* LevelMovie;
-    typedef std::map<int, LevelMovie> Levels;
+    typedef std::map<int, MovieClip*> Levels;
 
     /// The movie instance wrapped by this movie_root
     //
@@ -1139,13 +1109,13 @@ private:
     /// This is set to true if execution of scripts
     /// aborted due to action limit set or whatever else
     bool _disableScripts;
-
     int _processingActionLevel;
     
     /// filedescriptor to write to for host application requests
     //
     /// -1 if none
     int _hostfd;
+    int _controlfd;
 
     /// The display quality of the entire movie.
     //
@@ -1153,32 +1123,37 @@ private:
     /// does not rely on the presence of a renderer.
     Quality _quality;
 
+    /// The alignment of the Stage
     std::bitset<4u> _alignMode;
 
-	bool _showMenu;
-    
+    AllowScriptAccessMode _allowScriptAccess;
+    bool _marshallExceptions;
+
+    /// Whether to show the menu or not.
+    bool _showMenu;
+
+    /// The current scaling mode of the Stage.
     ScaleMode _scaleMode;
-    
+
+    /// The current state of the Stage (fullscreen or not).
     DisplayState _displayState;
     
-    // The maximum number of recursions e.g. when finding
-    // 'super', set in the ScriptLimits tag.
+    // Maximum number of recursions set in the ScriptLimits tag.
     boost::uint16_t _recursionLimit;
 
-    // The timeout in seconds for script execution, in the
-    // ScriptLimits tag.    
+    // Timeout in seconds for script execution, set in the ScriptLimits tag.
     boost::uint16_t _timeoutLimit;
 
     // delay between movie advancement, in milliseconds
-    unsigned int _movieAdvancementDelay;
+    size_t _movieAdvancementDelay;
 
     // time of last movie advancement, in milliseconds
-    unsigned int _lastMovieAdvancement;
+    size_t _lastMovieAdvancement;
 
+    /// The number of the last unnamed instance, used to name instances.
     size_t _unnamedInstance;
 
     MovieLoader _movieLoader;
-
 };
 
 /// Return true if the given string can be interpreted as a _level name
@@ -1200,5 +1175,5 @@ DSOEXPORT short stringToStageAlign(const std::string& s);
 
 // Local Variables:
 // mode: C++
-// indent-tabs-mode: t
+// indent-tabs-mode: nil
 // End:
