@@ -17,12 +17,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
+#include "RunResources.h"
 #include "smart_ptr.h" // GNASH_USE_GC
 #include "MovieClip.h"
 #include "sprite_definition.h"
 #include "ControlTag.h" // for dtor visibility
-#include "as_function.h" // for dtor visibility
-#include "SWFStream.h" // for use
+#include "as_function.h"
+#include "SWFStream.h"
 #include "GnashAlgorithm.h"
 #include "SWFParser.h"
 #include "namedStrings.h"
@@ -57,13 +58,6 @@ sprite_definition::createDisplayObject(Global_as& gl, DisplayObject* parent)
 
 sprite_definition::~sprite_definition()
 {
-	// Release our playlist data.
-	for (PlayListMap::iterator i=m_playlist.begin(), e=m_playlist.end();
-            i != e; ++i)
-	{
-		PlayList& pl = i->second;
-        deleteChecked(pl.begin(), pl.end());
-    }
 }
 
 /*private*/
@@ -124,8 +118,9 @@ sprite_definition::get_labeled_frame(const std::string& label,
 }
 
 sprite_definition::sprite_definition(movie_definition& m, SWFStream& in, 
-        const RunResources& runResources)
+        const RunResources& runResources, boost::uint16_t id)
 	:
+    movie_definition(id),
 	m_movie_def(m),
 	m_frame_count(0),
 	m_loading_frame(0),
@@ -135,11 +130,6 @@ sprite_definition::sprite_definition(movie_definition& m, SWFStream& in,
 	read(in, runResources);
 }
 
-/*
- * This function is not inlined to avoid having to include as_function.h
- * from sprite_definition.h. We need as_function.h for visibility of
- * as_function destructor by boost::intrusive_ptr
- */
 void
 sprite_definition::registerClass(as_function* the_class)
 {
