@@ -24,6 +24,8 @@
 #include "RGBA.h"
 #include "TextField.h"
 
+#include <boost/intrusive_ptr.hpp>
+#include <string>
 #include <boost/cstdint.hpp> // for boost::uint16_t and friends
  
 
@@ -187,7 +189,7 @@ private:
     /// Construct a DefineEditTextTag.
     //
     /// This should only be used from the loader() function.
-	DefineEditTextTag(SWFStream& in, movie_definition& m);
+	DefineEditTextTag(SWFStream& in, movie_definition& m, boost::uint16_t id);
 
     /// Read a tag from the SWFStream.
     void read(SWFStream& in, movie_definition& m);
