@@ -10,7 +10,6 @@
 #define GNASH_SHAPE_CHARACTER_DEF_H
 
 #include "DefinitionTag.h" // for inheritance of DefineShapeTag
-#include "smart_ptr.h" // GNASH_USE_GC
 #include "SWF.h"
 #include "ShapeRecord.h"
 
@@ -56,7 +55,7 @@ protected:
 private:
 
     DefineShapeTag(SWFStream& in, TagType tag, movie_definition& m,
-            const RunResources& r);
+            const RunResources& r, boost::uint16_t id);
 
     /// The actual shape data is stored in this record.
     const ShapeRecord _shape;
