@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,11 +19,10 @@
 #ifndef GNASH_SWF_DEFINETEXTTAG_H
 #define GNASH_SWF_DEFINETEXTTAG_H
 
+#include <vector>
 #include "DefinitionTag.h" // for inheritance
-#include "styles.h" 
-#include "rect.h" // for composition
+#include "SWFRect.h" // for composition
 #include "SWF.h"
-#include "movie_definition.h"
 #include "SWFMatrix.h"
 #include "TextRecord.h"
 
@@ -52,7 +52,7 @@ public:
 	/// Draw the string.
 	void display(Renderer& renderer, const StaticText& inst) const;
 	
-	const rect& bounds() const {
+	const SWFRect& bounds() const {
         // TODO: There is a _matrix field in the definition(!) that's
         // currently ignored. Don't know if it needs to be transformed... 
         return _rect; 
@@ -65,10 +65,10 @@ public:
     /// @param size Will contain the number of DisplayObjects in this
     ///             StaticText definition.
     bool extractStaticText(std::vector<const TextRecord*>& to, size_t& size)
-        const;
+      const;
 
-    virtual DisplayObject* createDisplayObject(DisplayObject* parent, int id)
-        const;
+    virtual DisplayObject* createDisplayObject(Global_as& gl,
+            DisplayObject* parent) const;
 
 private:
 
@@ -78,14 +78,17 @@ private:
     /// Construct a DefineTextTag.
     //
     /// This should only be constructed using the loader() functions.
-	DefineTextTag(SWFStream& in, movie_definition& m, TagType tag)
+	DefineTextTag(SWFStream& in, movie_definition& m, TagType tag,
+            boost::uint16_t id)
+        :
+        DefinitionTag(id)
     {
         read(in, m, tag);
     }
 
-	rect _rect;
+	SWFRect _rect;
 
-    SWFMatrix _matrix;
+        SWFMatrix _matrix;
 
 	void read(SWFStream& in, movie_definition& m, TagType tag);
 	
