@@ -18,6 +18,7 @@
 //
 
 #ifdef HAVE_CONFIG_H
+#include "RunResources.h"
 #include "gnashconfig.h" // HAVE_ZLIB_H, USE_SWFTREE
 #endif
 
@@ -29,6 +30,7 @@
 #include "SWFStream.h"
 #include "filter_factory.h"
 #include "GnashAlgorithm.h"
+#include "action_buffer.h"
 
 namespace gnash {
 namespace SWF {
@@ -39,7 +41,7 @@ PlaceObject2Tag::readPlaceObject(SWFStream& in)
     // Original place_object tag; very simple.
     in.ensureBytes(2 + 2);
     _id = in.read_u16();
-    m_depth = in.read_u16() + DisplayObject::staticDepthOffset;
+    _depth = in.read_u16() + DisplayObject::staticDepthOffset;
 
     // PlaceObject doesn't know about masks.
     m_clip_depth = DisplayObject::noClipDepthValue;
@@ -50,7 +52,7 @@ PlaceObject2Tag::readPlaceObject(SWFStream& in)
 
     if (in.tell() < in.get_tag_end_position())
     {
-        m_matrix.read(in);
+        m_matrix = readSWFMatrix(in);
         m_has_flags2 |= HAS_MATRIX_MASK;
         if (in.tell() < in.get_tag_end_position())
         {
@@ -62,7 +64,7 @@ PlaceObject2Tag::readPlaceObject(SWFStream& in)
     IF_VERBOSE_PARSE
     (
             log_parse(_("  PLACEOBJECT: depth=%d(%d) char=%d"),
-            	m_depth, m_depth - DisplayObject::staticDepthOffset,
+            	_depth, _depth - DisplayObject::staticDepthOffset,
             	_id);
             if (hasMatrix()) log_parse("  SWFMatrix: %s", m_matrix);
             if (hasCxform()) log_parse(_("  cxform: %s"), m_color_transform);
@@ -258,7 +260,7 @@ PlaceObject2Tag::readPlaceObject2(SWFStream& in)
     // PlaceObject2 specific flags
     m_has_flags2 = in.read_u8();
 
-    m_depth = in.read_u16()+DisplayObject::staticDepthOffset;
+    _depth = in.read_u16()+DisplayObject::staticDepthOffset;
 
     if ( hasCharacter( ))
     {
@@ -268,7 +270,7 @@ PlaceObject2Tag::readPlaceObject2(SWFStream& in)
 
     if ( hasMatrix() )
     {
-        m_matrix.read(in);
+        m_matrix = readSWFMatrix(in);
     }
 
     if ( hasCxform() )
@@ -279,7 +281,7 @@ PlaceObject2Tag::readPlaceObject2(SWFStream& in)
     if ( hasRatio() )
     {
         in.ensureBytes(2);
-        m_ratio = in.read_u16();
+        _ratio = in.read_u16();
     }
 
     if ( hasName() ) 
@@ -304,7 +306,7 @@ PlaceObject2Tag::readPlaceObject2(SWFStream& in)
 
     IF_VERBOSE_PARSE (
         log_parse(_("  PLACEOBJECT2: depth = %d (%d)"),
-            m_depth, m_depth-DisplayObject::staticDepthOffset);
+            _depth, _depth-DisplayObject::staticDepthOffset);
         if ( hasCharacter() ) log_parse(_("  char id = %d"), _id);
         if ( hasMatrix() )
         {
@@ -314,7 +316,7 @@ PlaceObject2Tag::readPlaceObject2(SWFStream& in)
         {
             log_parse(_("  cxform: %s"), m_color_transform);
         }
-        if ( hasRatio() ) log_parse(_("  ratio: %d"), m_ratio);
+        if ( hasRatio() ) log_parse(_("  ratio: %d"), _ratio);
         if ( hasName() ) log_parse(_("  name = %s"), m_name.c_str());
         if ( hasClipDepth() ) log_parse(_("  clip_depth = %d (%d)"), m_clip_depth, m_clip_depth-DisplayObject::staticDepthOffset);
         log_parse(_(" m_place_type: %d"), getPlaceType() );
@@ -339,7 +341,7 @@ PlaceObject2Tag::readPlaceObject3(SWFStream& in)
     boost::uint8_t bitmask = 0;
     std::string className;
 
-    m_depth = in.read_u16() + DisplayObject::staticDepthOffset;
+    _depth = in.read_u16() + DisplayObject::staticDepthOffset;
 
     // This is documented to be here, but real instances of 
     // tags with either className or hasImage defined are rare to
@@ -356,7 +358,7 @@ PlaceObject2Tag::readPlaceObject3(SWFStream& in)
     }
 
     if (hasMatrix()) {
-        m_matrix.read(in);
+        m_matrix = readSWFMatrix(in);
     }
 
     if (hasCxform()) {
@@ -365,7 +367,7 @@ PlaceObject2Tag::readPlaceObject3(SWFStream& in)
 
     if (hasRatio()) {
         in.ensureBytes(2);
-        m_ratio = in.read_u16();
+        _ratio = in.read_u16();
     }
     
     if (hasName()) {
@@ -416,12 +418,12 @@ PlaceObject2Tag::readPlaceObject3(SWFStream& in)
 
     IF_VERBOSE_PARSE (
 
-        log_parse(_("  PLACEOBJECT3: depth = %d (%d)"), m_depth,
-            m_depth - DisplayObject::staticDepthOffset);
+        log_parse(_("  PLACEOBJECT3: depth = %d (%d)"), _depth,
+            _depth - DisplayObject::staticDepthOffset);
         if (hasCharacter()) log_parse(_("  char id = %d"), _id);
         if (hasMatrix()) log_parse(_("  SWFMatrix: %s"), m_matrix);
         if (hasCxform()) log_parse(_("  cxform: %d"), m_color_transform);
-        if (hasRatio()) log_parse(_("  ratio: %d"), m_ratio);
+        if (hasRatio()) log_parse(_("  ratio: %d"), _ratio);
         if (hasName()) log_parse(_("  name = %s"), m_name);
         if (hasClassName()) log_parse(_("  class name = %s"), className);
         if (hasClipDepth()) log_parse(_("  clip_depth = %d (%d)"),
@@ -454,25 +456,25 @@ PlaceObject2Tag::read(SWFStream& in, TagType tag)
 
 /// Place/move/whatever our object in the given movie.
 void
-PlaceObject2Tag::execute(MovieClip* m, DisplayList& dlist) const
+PlaceObject2Tag::executeState(MovieClip* m, DisplayList& dlist) const
 {
-    switch ( getPlaceType() ) 
-    {
+    switch (getPlaceType()) {
+
       case PLACE:
           m->add_display_object(this, dlist);
-      break;
+          break;
 
       case MOVE:
           m->move_display_object(this, dlist);
-      break;
+          break;
 
       case REPLACE:
           m->replace_display_object(this, dlist);
-      break;
+          break;
 
       case REMOVE:
 		  m->remove_display_object(this, dlist);
-      break;
+          break;
     }
 }
 
