@@ -22,7 +22,7 @@
 #include "SWF.h" // for TagType definition
 #include "SWFMatrix.h" // for composition
 #include "cxform.h" // for composition 
-#include "DisplayObject.h" // BlendMode enum
+#include <string>
 #include <vector>
 
 // Forward declarations
@@ -73,7 +73,7 @@ namespace SWF {
 /// m_matrix:
 /// The SWFMatrix transform to apply to the newly created instance.
 ///
-/// m_ratio
+/// _ratio
 ///
 /// m_clip_depth:
 /// If != DisplayObject::noClipDepthValue, mark the created instance
@@ -94,7 +94,7 @@ public:
         m_has_flags2(0),
         m_has_flags3(0),
         _id(0),
-        m_ratio(0),
+        _ratio(0),
         m_clip_depth(0),
         _blendMode(0),
         _movie_def(def)
@@ -107,7 +107,7 @@ public:
     void read(SWFStream& in, TagType tag);
 
     /// Place/move/whatever our object in the given movie.
-    void execute(MovieClip* m, DisplayList& dlist) const;
+    void executeState(MovieClip* m, DisplayList& dlist) const;
 
     static void loader(SWFStream& in, TagType tag, movie_definition& m,
             const RunResources& r);
@@ -115,9 +115,10 @@ public:
     int getPlaceType() const { 
         return m_has_flags2 & (HAS_CHARACTER_MASK | MOVE_MASK);
     } 
-    int getRatio()     const { return m_ratio; }
+
+    int getRatio()     const { return _ratio; }
     int getClipDepth() const { return m_clip_depth; }
-    int getID()        const { return _id; }
+    boost::uint16_t getID()        const { return _id; }
     const std::string& getName() const { return m_name; }
     const SWFMatrix& getMatrix()    const { return m_matrix; }
     const cxform& getCxform()    const { return m_color_transform; }
@@ -164,7 +165,7 @@ private:
     boost::uint16_t _id;
     cxform  m_color_transform;
     SWFMatrix  m_matrix;
-    int     m_ratio;
+    int     _ratio;
     std::string m_name;
     int     m_clip_depth;
     
