@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,17 +18,12 @@
 #ifndef GNASH_SWF_SETBACKGROUNDCOLOR_TAG_H
 #define GNASH_SWF_SETBACKGROUNDCOLOR_TAG_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "ControlTag.h" // for inheritance
 #include "SWF.h" // for TagType definition
 #include "MovieClip.h" // for inlines (execute)
 #include "movie_definition.h" // for inlines (loader)
 #include "log.h" // for log_parse
 #include "RGBA.h" // for rgba class
-#include "GnashNumeric.h" // for frnd
 
 // Forward declarations
 namespace gnash {
@@ -59,10 +54,10 @@ private:
 	void read(SWFStream& in)
 	{
 		// may throw ParserException
-		m_color.read_rgb(in);
+		m_color = readRGB(in);
 
-		IF_VERBOSE_PARSE (
-		log_parse(_("  SetBackgroundColor: %s"), m_color.toString());
+		IF_VERBOSE_PARSE(
+            log_parse(_("  SetBackgroundColor: %s"), m_color);
 		);
 	}
 
@@ -76,22 +71,15 @@ public:
 	/// Tag header is assumed to have been read already
 	///
 	/// Can throw ParserException on premature end of input stream
-	///
 	SetBackgroundColorTag(SWFStream& in)
-	{
+    {
 		read(in);
 	}
 
-	void execute(MovieClip* m, DisplayList& /*dlist*/) const
-	{
+	void executeState(MovieClip* m, DisplayList& /*dlist*/) const {
 		m->set_background_color(m_color);
 	}
 
-	void execute_state(MovieClip* m, DisplayList& dlist) const
-	{
-		execute(m, dlist);
-	}
-
 	/// Set background color tag loader (SWF::SETBACKGROUNDCOLOR)
 	static void loader(SWFStream& in, TagType tag, movie_definition& m, 
             const RunResources& /*r*/)
