@@ -18,25 +18,28 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-
 #ifdef HAVE_CONFIG_H
 #include "gnashconfig.h" // HAVE_ZLIB_H, USE_SWFTREE
 #endif
 
+#include "tag_loaders.h"
+
+#include <set>
+#include <limits>
+#include <cassert>
+#include <utility> // for std::make_pair
+#include <boost/static_assert.hpp>
+
 #include "IOChannel.h" // for StreamAdapter inheritance
 #include "utility.h"
 #include "action_buffer.h"
 #include "Font.h"
 #include "log.h"
-#include "Geometry.h"
 #include "SWFStream.h"
-#include "LineStyle.h"
 #include "GnashImage.h"
 #include "zlib_adapter.h"
 #include "sprite_definition.h"
 #include "MovieClip.h"
-#include "swf_function.h"
-#include "as_function.h"
 #include "SWFMovieDefinition.h"
 #include "SWF.h"
 #include "swf/TagLoadersTable.h"
@@ -48,17 +51,15 @@
 #include "MediaHandler.h"
 #include "SimpleBuffer.h"
 #include "sound_handler.h"
-#include "ExportableResource.h"
 #include "MovieFactory.h"
+#include "RunResources.h"
+#include "Renderer.h"
+#include "Movie.h"
+#include "CachedBitmap.h"
 
 #ifdef HAVE_ZLIB_H
 #include <zlib.h>
 #endif
-#include <set>
-#include <limits>
-#include <cassert>
-#include <utility> // for std::make_pair
-#include <boost/static_assert.hpp>
 
 namespace gnash {
 
@@ -321,7 +322,7 @@ define_bits_jpeg_loader(SWFStream& in, TagType tag, movie_definition& m,
         IF_VERBOSE_PARSE(log_parse(_("No renderer, not adding bitmap")));
         return;
     }    
-    boost::intrusive_ptr<BitmapInfo> bi = renderer->createBitmapInfo(im);
+    boost::intrusive_ptr<CachedBitmap> bi = renderer->createCachedBitmap(im);
 
     // add bitmap to movie under DisplayObject id.
     m.addBitmap(id, bi);
@@ -383,7 +384,7 @@ define_bits_jpeg2_loader(SWFStream& in, TagType tag, movie_definition& m,
         IF_VERBOSE_PARSE(log_parse(_("No renderer, not adding bitmap")));
         return;
     }    
-    boost::intrusive_ptr<BitmapInfo> bi = renderer->createBitmapInfo(im);
+    boost::intrusive_ptr<CachedBitmap> bi = renderer->createCachedBitmap(im);
 
     // add bitmap to movie under DisplayObject id.
     m.addBitmap(id, bi);
@@ -538,8 +539,8 @@ define_bits_jpeg3_loader(SWFStream& in, TagType tag, movie_definition& m,
         IF_VERBOSE_PARSE(log_parse(_("No renderer, not adding bitmap")));
         return;
     }    
-    boost::intrusive_ptr<BitmapInfo> bi =
-        renderer->createBitmapInfo(static_cast<std::auto_ptr<GnashImage> >(im));
+    boost::intrusive_ptr<CachedBitmap> bi =
+        renderer->createCachedBitmap(static_cast<std::auto_ptr<GnashImage> >(im));
 
     // add bitmap to movie under DisplayObject id.
     m.addBitmap(id, bi);
@@ -645,8 +646,7 @@ define_bits_lossless_2_loader(SWFStream& in, TagType tag, movie_definition& m,
     inflate_wrapper(in, buffer.get(), bufSize);
     assert(in.tell() <= in.get_tag_end_position());
 
-    switch (bitmap_format)
-    {
+    switch (bitmap_format) {
 
         case 3:
         {
@@ -658,7 +658,7 @@ define_bits_lossless_2_loader(SWFStream& in, TagType tag, movie_definition& m,
                 boost::uint8_t* inRow = buffer.get() + 
                     colorTableSize * channels + j * pitch;
 
-                boost::uint8_t*    outRow = image->scanline(j);
+                boost::uint8_t* outRow = scanline(*image, j);
                 for (int i = 0; i < width; i++)
                 {
                     boost::uint8_t pixel = inRow[i * bytes_per_pixel];
@@ -680,7 +680,7 @@ define_bits_lossless_2_loader(SWFStream& in, TagType tag, movie_definition& m,
             for (int j = 0; j < height; j++)
             {
                 boost::uint8_t* inRow = buffer.get() + j * pitch;
-                boost::uint8_t* outRow = image->scanline(j);
+                boost::uint8_t* outRow = scanline(*image, j);
                 for (int i = 0; i < width; i++)
                 {
                     boost::uint16_t pixel = inRow[i * 2] |
@@ -707,7 +707,7 @@ define_bits_lossless_2_loader(SWFStream& in, TagType tag, movie_definition& m,
             for (int j = 0; j < height; j++)
             {
                 boost::uint8_t* inRow = buffer.get() + j * pitch;
-                boost::uint8_t* outRow = image->scanline(j);
+                boost::uint8_t* outRow = scanline(*image, j);
                 const int inChannels = 4;
 
                 for (int i = 0; i < width; ++i)
@@ -731,7 +731,7 @@ define_bits_lossless_2_loader(SWFStream& in, TagType tag, movie_definition& m,
         IF_VERBOSE_PARSE(log_parse(_("No renderer, not adding bitmap")));
         return;
     }    
-    boost::intrusive_ptr<BitmapInfo> bi = renderer->createBitmapInfo(image);
+    boost::intrusive_ptr<CachedBitmap> bi = renderer->createCachedBitmap(image);
 
     // add bitmap to movie under DisplayObject id.
     m.addBitmap(id, bi);
@@ -758,7 +758,7 @@ sprite_loader(SWFStream& in, TagType tag, movie_definition& m,
     assert(tag == SWF::DEFINESPRITE); // 39 - DefineSprite
 
     in.ensureBytes(2);
-    int    id = in.read_u16();
+    const boost::uint16_t id = in.read_u16();
 
     IF_VERBOSE_PARSE
     (
@@ -779,7 +779,7 @@ sprite_loader(SWFStream& in, TagType tag, movie_definition& m,
     );
 
     // will automatically read the sprite
-    sprite_definition* ch = new sprite_definition(m, in, r);
+    sprite_definition* ch = new sprite_definition(m, in, r, id);
 
     IF_VERBOSE_MALFORMED_SWF(
         if (!ch->get_frame_count()) {
@@ -792,151 +792,6 @@ sprite_loader(SWFStream& in, TagType tag, movie_definition& m,
 }
 
 
-//
-// export
-//
-
-
-void export_loader(SWFStream& in, TagType tag, movie_definition& m,
-		const RunResources& /*r*/)
-    // Load an export tag (for exposing internal resources of m)
-{
-    assert(tag == SWF::EXPORTASSETS); // 56
-
-    in.ensureBytes(2);
-    int    count = in.read_u16();
-
-    IF_VERBOSE_PARSE(
-        log_parse(_("  export: count = %d"), count);
-    );
-
-    // An EXPORT tag as part of a DEFINESPRITE
-    // would be a malformed SWF, anyway to be compatible
-    // we should still allow that. See bug #22468.
-    IF_VERBOSE_MALFORMED_SWF(
-        try {
-            dynamic_cast<SWFMovieDefinition&>(m);
-        }
-        catch (std::bad_cast& e) {
-            log_swferror(_("EXPORT tag inside DEFINESPRITE. Will export in "
-                    "top-level symbol table."));
-        }
-    );
-
-
-    // Read the exports.
-    for (int i = 0; i < count; i++)
-    {
-        in.ensureBytes(2);
-        boost::uint16_t id = in.read_u16();
-        std::string symbolName;
-        in.read_string(symbolName);
-
-        IF_VERBOSE_PARSE (
-            log_parse(_("  export: id = %d, name = %s"), id, symbolName);
-        );
-
-        // Fonts, DisplayObjects and sounds can be exported.
-        ExportableResource* f;
-        if ((f = m.get_font(id)) ||
-            (f = m.getDefinitionTag(id)) ||
-            (f = m.get_sound_sample(id))) {
-            
-            m.export_resource(symbolName, f);
-        }
-        else {
-            IF_VERBOSE_MALFORMED_SWF(
-            log_swferror(_("don't know how to export resource '%s' "
-                "with id %d (can't find that id)"),
-                symbolName, id);
-            );
-        }
-    }
-}
-
-
-//
-// import
-//
-
-
-void import_loader(SWFStream& in, TagType tag, movie_definition& m,
-		const RunResources& r)
-{
-    assert(tag == SWF::IMPORTASSETS || tag == SWF::IMPORTASSETS2);
-
-    std::string source_url;
-    in.read_string(source_url);
-
-    // Resolve relative urls against baseurl
-    URL abs_url(source_url, r.baseURL());
-
-    unsigned char import_version = 0;
-
-    if ( tag == SWF::IMPORTASSETS2 )
-    {
-        in.ensureBytes(2);
-        import_version = in.read_uint(8);
-        unsigned char reserved = in.read_uint(8);
-        UNUSED(reserved);
-    }
-
-    in.ensureBytes(2);
-    int count = in.read_u16();
-
-    IF_VERBOSE_PARSE
-    (
-        log_parse(_("  import: version = %u, source_url = %s (%s), count = %d"),
-            import_version, abs_url.str(), source_url, count);
-    );
-
-
-    // Try to load the source movie into the movie library.
-    boost::intrusive_ptr<movie_definition> source_movie;
-
-    try {
-        source_movie = MovieFactory::makeMovie(abs_url, r);
-    }
-    catch (gnash::GnashException& e) {
-        log_error(_("Exception: %s"), e.what());
-    }
-
-    if (!source_movie)
-    {
-        // Give up on imports.
-        log_error(_("can't import movie from url %s"), abs_url.str());
-        return;
-    }
-
-    // Quick consistency check, we might as well do
-    // something smarter, if we agree on semantic
-    if (source_movie == &m)
-    {
-        IF_VERBOSE_MALFORMED_SWF(
-            log_swferror(_("Movie attempts to import symbols from itself."));
-        );
-        return;
-    }
-
-    movie_definition::Imports imports;
-
-    // Get the imports.
-    for (int i = 0; i < count; i++)
-    {
-        in.ensureBytes(2);
-        boost::uint16_t    id = in.read_u16();
-        std::string symbolName;
-        in.read_string(symbolName);
-        IF_VERBOSE_PARSE (
-            log_parse(_("  import: id = %d, name = %s"), id, symbolName);
-        );
-        imports.push_back( std::make_pair(id, symbolName) );
-    }
-
-    m.importResources(source_movie, imports);
-}
-
-
 //
 // Sound
 //
@@ -1013,10 +868,11 @@ define_sound_loader(SWFStream& in, TagType tag, movie_definition& m,
         // then the amount allocated at *data (it may grow)
         const unsigned dataLength = in.get_tag_end_position() - in.tell();
 
-        // Allocate MediaHandler::getInputPadding() bytes more for the SimpleBuffer 
+        // Allocate MediaHandler::getInputPaddingSize() bytes more for the
+        // SimpleBuffer 
         size_t allocSize = dataLength;
-        media::MediaHandler* mh = media::MediaHandler::get(); // TODO: don't use this static !
-        if ( mh ) allocSize += mh->getInputPaddingSize();
+        media::MediaHandler* mh = r.mediaHandler();
+        if (mh) allocSize += mh->getInputPaddingSize();
 
         std::auto_ptr<SimpleBuffer> data( new SimpleBuffer(allocSize) );
 
