@@ -1,6 +1,7 @@
 // swf_event.h -- clip events (PlaceObject-defined)
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,10 +20,12 @@
 #ifndef GNASH_SWF_EVENT_H
 #define GNASH_SWF_EVENT_H
 
-#include "as_value.h" // for composition
-#include "action_buffer.h" // for composition
 #include "event_id.h" // for composition
 
+namespace gnash {
+    class action_buffer;
+}
+
 namespace gnash {
 
 //
