@@ -1,4 +1,4 @@
-// VideoInputFfmpeg.h: Video input processing using Gstreamer
+// VideoInputFfmpeg.h: Video input processing using Ffmpeg
 // 
 //   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 // 
@@ -40,7 +40,7 @@ VideoInputFfmpeg::VideoInputFfmpeg()
 
 void
 VideoInputFfmpeg::requestMode(size_t width, size_t height, double fps,
-        bool favorArea)
+        bool /*favorArea*/)
 {
     // TODO: check what mode is available and set the best match.
     _width = width;
