@@ -24,7 +24,7 @@
 
 #include "sound_handler.h" // for inheritance
 
-#include <vector> // for composition (Sounds)
+#include <string>
 #include <set> // for composition (InputStreams)
 #include <fstream> // for composition (file_stream)
 #include <SDL_audio.h>
@@ -99,9 +99,9 @@ private:
 
 public:
 
-    SDL_sound_handler();
+    SDL_sound_handler(media::MediaHandler* m);
 
-    SDL_sound_handler(const std::string& wave_file);
+    SDL_sound_handler(media::MediaHandler* m, const std::string& wave_file);
 
     ~SDL_sound_handler();
 
