@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,6 +21,7 @@
 #endif
 
 #include "SWFMatrix.h"
+#include "Point2d.h"
 #include <iostream>
 #include <sstream>
 #include <cassert>
@@ -77,7 +79,6 @@ main(int /*argc*/, char** /*argv*/)
     //  Test identity SWFMatrix.
     // 
     SWFMatrix identity; 
-    check(identity.is_valid());
     check_equals(identity.get_x_scale(), 1);
     check_equals(identity.get_y_scale(), 1);
     check_equals(identity.get_rotation(), 0);
