@@ -22,8 +22,6 @@
 
 #include "MovieTester.h"
 #include "MovieClip.h"
-#include "DisplayObject.h"
-#include "DisplayList.h"
 #include "log.h"
 #include "GnashKey.h" // for gnash::key::code
 
@@ -1005,6 +1003,45 @@ main(int /*argc*/, char** /*argv*/)
     check_pixel(x + 90 - 2, y + 2, 2, green, 2);
     check_pixel(x + 90 - 2, y + 90 - 2, 2, green, 2);
 
+    // Shape 14
+    x += 100;
+    check_pixel(x + 7, y + 2, 2, red, 2);
+    check_pixel(x + 7, y + 90 - 2, 2, red, 2);
+    check_pixel(x + 90 - 2, y + 2, 2, red, 2);
+    check_pixel(x + 90 - 2, y + 90 - 2, 2, red, 2);
+
+    // Shape 15
+    x += 100;
+    check_pixel(x + 7, y + 2, 2, green, 2);
+    check_pixel(x + 7, y + 90 - 2, 2, green, 2);
+    check_pixel(x + 90 - 2, y + 2, 2, green, 2);
+    check_pixel(x + 90 - 2, y + 90 - 2, 2, green, 2);
+
+    // 3 invalid fills follow
+
+    // Shape 16
+    x += 100;
+    check_pixel(x + 7, y + 2, 2, white, 2);
+    check_pixel(x + 7, y + 90 - 2, 2, white, 2);
+    check_pixel(x + 90 - 2, y + 2, 2, white, 2);
+    check_pixel(x + 90 - 2, y + 90 - 2, 2, white, 2);
+
+    // Shape 17
+    x += 100;
+    check_pixel(x + 7, y + 2, 2, white, 2);
+    check_pixel(x + 7, y + 90 - 2, 2, white, 2);
+    check_pixel(x + 90 - 2, y + 2, 2, white, 2);
+    check_pixel(x + 90 - 2, y + 90 - 2, 2, white, 2);
+
+    x = 0;
+    y += 100;
+
+    // Shape 18
+    check_pixel(x + 7, y + 2, 2, white, 2);
+    check_pixel(x + 7, y + 90 - 2, 2, white, 2);
+    check_pixel(x + 90 - 2, y + 2, 2, white, 2);
+    check_pixel(x + 90 - 2, y + 90 - 2, 2, white, 2);
+
     //----------------------------------------------------------
 	// TODO: check startDrag/stopDrag on the hit detector
 	// (hit 'd' key to toggle)
