@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2009, 2010 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -101,6 +101,13 @@ main(int argc, char** argv)
   SWFDisplayItem_setName(it1211, "mc1211"); 
   
   /* add onClipEvents */
+  SWFDisplayItem_addAction(it1211, // the inner most child
+    compileSWFActionCode(" _root.note('mc1211 onInitialize called'); "
+                         " _root.check_equals(this.__proto__, MovieClip.prototype);"
+                         " _root.check_equals(this._parent.__proto__, MovieClip.prototype);"
+                         " _root.check_equals(this._parent._parent.__proto__, MovieClip.prototype);"),
+    SWFACTION_INIT);
+
   SWFDisplayItem_addAction(it1211,
     compileSWFActionCode(" _root.note('mc1211 onLoad called'); "
                          " _root.x1 += '5+'; "),
@@ -174,6 +181,25 @@ main(int argc, char** argv)
   SWFDisplayItem_setName(it12, "mc12"); 
   
   /* add onClipEvents */
+  SWFDisplayItem_addAction(it12, 
+    compileSWFActionCode(" _root.note('mc12 onInitialize called'); "
+                         " _root.check_equals(this, _root.mc1.mc12);"
+                         " _root.check_equals(this.__proto__, MovieClip.prototype);"
+                         " _root.check_equals(this._parent.__proto__, MovieClip.prototype);"
+                         " _root.check_equals(this._parent.mc11.__proto__, MovieClip.prototype);"
+                         // test child __proto__
+                         " _root.check_equals(this.mc121.__proto__, MovieClip.prototype);"),
+    SWFACTION_INIT);
+
+  SWFDisplayItem_addAction(it12, // the inner most child
+    compileSWFActionCode(" _root.note('mc12 onConstruct called'); "
+                         " _root.check_equals(this, _root.mc1.mc12);"
+                         " _root.check_equals(this.__proto__, MovieClip.prototype);"
+                         " _root.check_equals(this._parent.__proto__, MovieClip.prototype);"
+                         " _root.check_equals(this._parent.mc11.__proto__, MovieClip.prototype);"
+                         " _root.check_equals(this.mc121.__proto__, MovieClip.prototype);"),
+    SWFACTION_CONSTRUCT);
+    
   SWFDisplayItem_addAction(it12,
     compileSWFActionCode(" _root.note('mc12 onLoad called'); "
                          " _root.x1 += '3+'; "),
@@ -224,7 +250,7 @@ main(int argc, char** argv)
   SWFMovie_nextFrame(mo); /* 4th frame of _root */
 
   /* checks */
-  xcheck_equals(mo, "_root.x1", "'1+2+3+4+5+6+7+8+9+10+11+12+13+14+15+'");
+  check_equals(mo, "_root.x1", "'1+2+3+4+5+6+7+8+9+10+11+12+13+14+15+'");
   add_actions(mo, " _root.totals(); stop(); ");
   SWFMovie_nextFrame(mo); /* 5th frame of _root */
   
@@ -235,6 +261,3 @@ main(int argc, char** argv)
 
   return 0;
 }
-
-
-
