@@ -1,6 +1,6 @@
 /***********************************************************************
  *
- *   Copyright (C) 2007, 2009 Free Software Foundation, Inc.
+ *   Copyright (C) 2007, 2009, 2010 Free Software Foundation, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -134,18 +134,21 @@ add_coverart(SWFMovie mo, int x, int y)
 		//"	_root.note(this+'.onUnload called');"
 		//"};"
 		"_level0.coverart.onMouseDown = function() {"
+        "t = this.createEmptyMovieClip('tc', 8);"
+        "_root.check(tc instanceof MovieClip);"
+        "_root.check(t instanceof MovieClip);"
 		//"  _root.note('_url is '+this._url);"
 		"  var lastUrlComponent = this._url.substring(this._url.lastIndexOf('/')+1);"
 		//"  _root.note('last component of _url is '+lastUrlComponent);"
 		"  _root.check_equals(lastUrlComponent, _level0.expectLoaded, '%s:%d');"
-		"  _root.check_equals(this.getDepth(), -16377);"
+		"  _root.check_equals(this.getDepth(), -16376);"
 		"  _root.check_equals(this.getBytesLoaded(), this.getBytesTotal());"
 		"  _root.check(this.getBytesLoaded() > 0);" /* assuming something was loaded here */
 		"  _root.note('bytesLoaded: '+this.getBytesLoaded());"
 		"  if ( Key.isDown(Key.SHIFT) ) { "
 		"	trace('SHIFT-click on coverart...');"
 		//"	_root.note('SHIFT-click on coverart...');"
-		"	_root.totals(18, '"__FILE__"');"
+		"	_root.totals(26, '"__FILE__"');"
 		"	_root.END_OF_TEST = true;"
 		" }"
 		"  else _root.note('2 tests run');"
@@ -250,7 +253,7 @@ main(int argc, char** argv)
 
 	dejagnuclip = get_dejagnu_clip((SWFBlock)get_default_font(mediadir), 10, 0, 0, 800, 600);
 	it = SWFMovie_add(mo, (SWFBlock)dejagnuclip);
-	SWFDisplayItem_moveTo(it, 0, 250);
+	SWFDisplayItem_moveTo(it, 0, 270);
 	SWFMovie_nextFrame(mo); 
 
 	/*****************************************************
@@ -299,6 +302,13 @@ main(int argc, char** argv)
 				" art.clear(); "
 				));
 
+	add_button(mo, 50, 240, "Unload", newSWFAction(
+		   		"_root.coverart.onUnload = function() {"
+				"  _root.note(this+'.onUnload called');"
+				"};"	
+				" _root.coverart.unloadMovie();"
+				));
+
 	/*****************************************************
 	 *
 	 * Add the coverart clip
