@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,8 +23,8 @@
 #define INPUT_FILENAME "simple_loop_test.swf"
 
 #include "MovieTester.h"
-#include "sprite_instance.h"
-#include "character.h"
+#include "MovieClip.h"
+#include "DisplayObject.h"
 #include "DisplayList.h"
 #include "log.h"
 
@@ -59,13 +59,13 @@ void testAll(MovieTester& tester)
 	typedef gnash::geometry::Range2d<int> Bounds;
 
 	Ranges invalidated;
-	sprite_instance* root = tester.getRootMovie();
+	MovieClip* root = tester.getRootMovie();
 	assert(root);
 
 	// FRAME 1/4 (start)
 
 	check_equals(root->get_frame_count(), 4);
-	check_equals(root->get_play_state(), sprite_instance::PLAY);
+	check_equals(root->getPlayState(), MovieClip::PLAYSTATE_PLAY);
 	check_equals(root->get_current_frame(), 0);
 	check_equals(root->getDisplayList().size(), 0); // no chars
 	invalidated = tester.getInvalidatedRanges();
@@ -75,37 +75,37 @@ void testAll(MovieTester& tester)
 
 	tester.advance(); // FRAME 2/4
 	
-	check_equals(root->get_play_state(), sprite_instance::PLAY);
+	check_equals(root->getPlayState(), MovieClip::PLAYSTATE_PLAY);
 	check_equals(root->get_current_frame(), 1);
 	check_equals(root->getDisplayList().size(), 1);
-	check( tester.findDisplayItemByDepth(*root, 2+character::staticDepthOffset) );
+	check( tester.findDisplayItemByDepth(*root, 2+DisplayObject::staticDepthOffset) );
 	invalidated = tester.getInvalidatedRanges();
 	check( invalidated.contains(Bounds(0, 0, 60, 60)) );
 
 	tester.advance(); // FRAME 3/4
 	
-	check_equals(root->get_play_state(), sprite_instance::PLAY);
+	check_equals(root->getPlayState(), MovieClip::PLAYSTATE_PLAY);
 	check_equals(root->get_current_frame(), 2);
 	check_equals(root->getDisplayList().size(), 2);
-	check( tester.findDisplayItemByDepth(*root, 2+character::staticDepthOffset) );
-	check( tester.findDisplayItemByDepth(*root, 3+character::staticDepthOffset) );
+	check( tester.findDisplayItemByDepth(*root, 2+DisplayObject::staticDepthOffset) );
+	check( tester.findDisplayItemByDepth(*root, 3+DisplayObject::staticDepthOffset) );
 	invalidated = tester.getInvalidatedRanges();
 	check( invalidated.contains(Bounds(60, 0, 120, 60)) );
 
 	tester.advance(); // FRAME 4/4
 	
-	check_equals(root->get_play_state(), sprite_instance::PLAY);
+	check_equals(root->getPlayState(), MovieClip::PLAYSTATE_PLAY);
 	check_equals(root->get_current_frame(), 3);
 	check_equals(root->getDisplayList().size(), 3);
-	check( tester.findDisplayItemByDepth(*root, 2+character::staticDepthOffset) );
-	check( tester.findDisplayItemByDepth(*root, 3+character::staticDepthOffset) );
-	check( tester.findDisplayItemByDepth(*root, 4+character::staticDepthOffset) );
+	check( tester.findDisplayItemByDepth(*root, 2+DisplayObject::staticDepthOffset) );
+	check( tester.findDisplayItemByDepth(*root, 3+DisplayObject::staticDepthOffset) );
+	check( tester.findDisplayItemByDepth(*root, 4+DisplayObject::staticDepthOffset) );
 	invalidated = tester.getInvalidatedRanges();
 	check( invalidated.contains(Bounds(120, 0, 180, 60)) );
 
 	tester.advance(); // FRAME 1/4 (loop back)
 	
-	check_equals(root->get_play_state(), sprite_instance::PLAY);
+	check_equals(root->getPlayState(), MovieClip::PLAYSTATE_PLAY);
 	check_equals(root->get_current_frame(), 0);
 	check_equals(root->getDisplayList().size(), 0);
 	invalidated = tester.getInvalidatedRanges();
@@ -113,31 +113,31 @@ void testAll(MovieTester& tester)
 
 	tester.advance(); // FRAME 2/4
 	
-	check_equals(root->get_play_state(), sprite_instance::PLAY);
+	check_equals(root->getPlayState(), MovieClip::PLAYSTATE_PLAY);
 	check_equals(root->get_current_frame(), 1);
 	check_equals(root->getDisplayList().size(), 1);
-	check( tester.findDisplayItemByDepth(*root, 2+character::staticDepthOffset) );
+	check( tester.findDisplayItemByDepth(*root, 2+DisplayObject::staticDepthOffset) );
 	invalidated = tester.getInvalidatedRanges();
 	check( invalidated.contains(Bounds(0, 0, 60, 60)) );
 
 	tester.advance(); // FRAME 3/4
 	
-	check_equals(root->get_play_state(), sprite_instance::PLAY);
+	check_equals(root->getPlayState(), MovieClip::PLAYSTATE_PLAY);
 	check_equals(root->get_current_frame(), 2);
 	check_equals(root->getDisplayList().size(), 2);
-	check( tester.findDisplayItemByDepth(*root, 2+character::staticDepthOffset) );
-	check( tester.findDisplayItemByDepth(*root, 3+character::staticDepthOffset) );
+	check( tester.findDisplayItemByDepth(*root, 2+DisplayObject::staticDepthOffset) );
+	check( tester.findDisplayItemByDepth(*root, 3+DisplayObject::staticDepthOffset) );
 	invalidated = tester.getInvalidatedRanges();
 	check( invalidated.contains(Bounds(60, 0, 120, 60)) );
 
 	tester.advance(); // FRAME 4/4
 	
-	check_equals(root->get_play_state(), sprite_instance::PLAY);
+	check_equals(root->getPlayState(), MovieClip::PLAYSTATE_PLAY);
 	check_equals(root->get_current_frame(), 3);
 	check_equals(root->getDisplayList().size(), 3);
-	check( tester.findDisplayItemByDepth(*root, 2+character::staticDepthOffset) );
-	check( tester.findDisplayItemByDepth(*root, 3+character::staticDepthOffset) );
-	check( tester.findDisplayItemByDepth(*root, 4+character::staticDepthOffset) );
+	check( tester.findDisplayItemByDepth(*root, 2+DisplayObject::staticDepthOffset) );
+	check( tester.findDisplayItemByDepth(*root, 3+DisplayObject::staticDepthOffset) );
+	check( tester.findDisplayItemByDepth(*root, 4+DisplayObject::staticDepthOffset) );
 	invalidated = tester.getInvalidatedRanges();
 	check( invalidated.contains(Bounds(120, 0, 180, 60)) );
 }
