@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -25,13 +26,13 @@
 #include <boost/shared_ptr.hpp>
 #include <dirent.h>
 #include <iostream>
-#include <stdarg.h>
+#include <cstdarg>
 #include <cstring>
 #include <string>
 #include <vector>
 
 #ifdef ENABLE_NLS
-# include <locale>
+# include <clocale>
 #endif
 
 #include "log.h"
@@ -41,7 +42,7 @@
 #include "buffer.h"
 #include "arg_parser.h"
 
-using namespace amf;
+using namespace cygnal;
 using namespace std;
 using namespace gnash;
 
@@ -189,19 +190,19 @@ main(int argc, char *argv[])
     Flv::previous_size_t   previous = 0;
     boost::shared_ptr<Flv::flv_tag_t> tag;
     
-    // Make sure it's an SOL file
+    // Make sure it's an FLV file
     if (stat(filespec.c_str(), &st) == 0) {
 	try {
             // Open the binary file
 	    ifstream ifs(filespec.c_str(), ios::binary);
-	    boost::shared_ptr<amf::Buffer> buf(new Buffer);
+	    boost::shared_ptr<cygnal::Buffer> buf(new Buffer);
             // Read just the initial 9 byte header
 	    ifs.read(reinterpret_cast<char *>(buf->reference()), sizeof(Flv::flv_header_t));
 	    log_debug("header is: %s",  hexify(buf->reference(), 9, false));
 	    boost::shared_ptr<Flv::flv_header_t> head = flv.decodeHeader(buf);
 	    if (head == 0) {
 		log_error("Couldn't decode the header! %s",  hexify(buf->reference(), 9, false));
-		exit(-1);
+		exit(EXIT_FAILURE);
 	    }
 	    if ((head->type & Flv::FLV_VIDEO) && (head->type & Flv::FLV_AUDIO)) {
                 cout <<"FLV File type: Video and Audio" << endl;
@@ -278,7 +279,7 @@ main(int argc, char *argv[])
  		       if (meta || all) {
  			   cout << "FLV Tag type is: MetaData" << endl;
  		       }
-		       boost::shared_ptr<amf::Element> metadata = flv.decodeMetaData(buf->reference(), bodysize);
+		       boost::shared_ptr<cygnal::Element> metadata = flv.decodeMetaData(buf->reference(), bodysize);
 		       if (meta && metadata) {
 			   metadata->dump();
 		       }
