@@ -23,11 +23,6 @@
 #endif
 
 #include "NullSoundHandler.h"
-#ifdef USE_FFMPEG
-# include "ffmpeg/MediaHandlerFfmpeg.h"
-#elif defined(USE_GST)
-# include "gst/MediaHandlerGst.h"
-#endif
 
 #include <ios>
 #include <iostream>
@@ -60,6 +55,7 @@
 #include "IOChannel.h" // for proper dtor call
 #include "GnashSleep.h" // for usleep comptibility.
 #include "StreamProvider.h"
+#include "RunResources.h"
 
 extern "C"{
 #ifdef HAVE_GETOPT_H
@@ -103,7 +99,6 @@ static long int delay = 0;
 
 const char *GPROC_VERSION = "1.0";
 
-using namespace std;
 using namespace gnash;
 
 static void usage (const char *);
@@ -158,7 +153,8 @@ public:
 	}
 };
 
-class EventCallback: public movie_root::AbstractIfaceCallback {
+class EventCallback: public movie_root::AbstractIfaceCallback
+{
 public:
 	std::string call(const std::string& event, const std::string& arg)
 	{
@@ -169,37 +165,37 @@ public:
 	    // These should return "true" if the mouse was visible before
 	    // the call.
 	    if ( event == "Mouse.hide" ) {
-		bool state = mouseShown;
-		mouseShown = false;
-		return state ? "true" : "false";
+            bool state = mouseShown;
+            mouseShown = false;
+            return state ? "true" : "false";
 	    }
 
 	    if ( event == "Mouse.show" ) {
-		bool state = mouseShown;
-		mouseShown = true;
-		return state ? "true" : "false" ;
+            bool state = mouseShown;
+            mouseShown = true;
+            return state ? "true" : "false" ;
 	    }
 	    
 	    // Some fake values for consistent test results.
 	    
 	    if ( event == "System.capabilities.screenResolutionX" ) {
-		return "800";
+            return "800";
 	    }
 
 	    if ( event == "System.capabilities.screenResolutionY" ) {
-		return "640";
+            return "640";
 	    } 
 
 	    if ( event == "System.capabilities.screenDPI" ) {
-		return "72";
+            return "72";
 	    }        
 
 	    if ( event == "System.capabilities.screenColor" ) {
-		return "Color";
+            return "Color";
 	    } 
 
 	    if ( event == "System.capabilities.playerType" ) {
-		return "StandAlone";
+            return "StandAlone";
 	    } 
 
 	    return "";
@@ -210,6 +206,10 @@ public:
     {
         return true;
     }
+
+    void exit() {
+        std::exit(EXIT_SUCCESS);
+    }
 };
 
 EventCallback eventCallback;
@@ -350,20 +350,11 @@ main(int argc, char *argv[])
     }
 
 
-    std::auto_ptr<gnash::media::MediaHandler> mediaHandler;
+    boost::shared_ptr<gnash::media::MediaHandler> mediaHandler;
     boost::shared_ptr<sound::sound_handler> soundHandler;
 
-#ifdef USE_FFMPEG
-    mediaHandler.reset( new gnash::media::ffmpeg::MediaHandlerFfmpeg() );
-#elif defined(USE_GST)
-    mediaHandler.reset( new gnash::media::gst::MediaHandlerGst() );
-#endif
-
-    if ( mediaHandler.get() )
-    {
-        gnash::media::MediaHandler::set(mediaHandler);
-        soundHandler.reset( new sound::NullSoundHandler() );
-    }
+    mediaHandler.reset(media::MediaFactory::instance().get(""));
+    soundHandler.reset(new sound::NullSoundHandler(mediaHandler.get()));
 
     boost::shared_ptr<StreamProvider> sp(new StreamProvider);
 
@@ -378,6 +369,7 @@ main(int argc, char *argv[])
 
         RunResources runResources(*i);
         runResources.setSoundHandler(soundHandler);
+        runResources.setMediaHandler(mediaHandler);
         runResources.setStreamProvider(sp);
         runResources.setTagLoaders(loaders);
 
@@ -465,7 +457,7 @@ play_movie(const std::string& filename, const RunResources& runResources)
     md->completeLoad();
 
     MovieClip::MovieVariables v;
-    m.init(md.get(), v);
+    m.init(md.get(), v, v);
 
     log_debug("iteration, timer: %lu, localDelay: %ld\n",
             cl.elapsed(), localDelay);
